import java.awt.*;
import java.io.*;
import java.util.Random;
import java.util.Arrays;

public class PPMGraphics extends Object {

  // the idea here is to create a ppm type graphics file from an input file

  public PPMGraphics(int m, int n, double [] d, int map) {
    setUniformRectangularGrid(m, n);
    init(map);
  }

  public PPMGraphics(double [][] d, int map) {
    if(d != null) {
      nx = d.length;
      ny = d[0].length;
    } else {
      nx = -1;
      ny = -1;
      status = BAD;
    }
    setUniformRectangularGrid(nx, ny);
    nxny = nx * ny;
    data = new double[nxny];
    for(int j=0; j<ny; j++) {
      int j2 = j * nx;
      for(int i=0; i<nx; i++) {
        int ind = i + j2;
        data[ind] = d[i][j];
      }
    }
    init(map);
  }

  public PPMGraphics(double [] xin, double [] yin, double [] din, int map) {
    if(xin == null) {
      nx = -1;
      status = BAD;
      return;
    }
    if(yin == null) {
      ny = -1;
      status = BAD;
      return;
    }
    if(din == null) {
      nxny = -1;
      status = BAD;
      return;
    }

    nxny = din.length;
    data = new double[nxny];
    for(int i=0; i<nxny; i++) data[i] = din[i];

    // test the dimensions of things
    int tnx = xin.length;
    int tny = yin.length;
    if(tnx != nxny || tny != nxny) {
      int tnxny = tnx * tny;
      if(tnxny != nxny) {
        nxny = -1;
        status = BAD;
        return;
      }
      x = new double[nxny];
      y = new double[nxny];
      nx = tnx;
      ny = tny;
      for(int j=0; j<ny; j++) {
        int j2 = j * nx;
        for(int i=0; i<nx; i++) {
          int ind = i + j2;
          x[ind] = xin[i];
          y[ind] = yin[j];
        }
      }
    } else {
      x = new double[nxny];
      y = new double[nxny];
      for(int i=0; i<nxny; i++) {
        x[i] = xin[i];
        y[i] = yin[i];
      }
    }

    init(map);

  }

  public PPMGraphics(double [] xin, double [] yin, double [][] din, int map) {

    if(xin == null) {
      nx = -1;
      status = BAD;
      return;
    }
    if(yin == null) {
      ny = -1;
      status = BAD;
      return;
    }
    if(din == null) {
      nxny = -1;
      status = BAD;
      return;
    }
    if(din != null) {
      nx = din.length;
      ny = din[0].length;
    } else {
      nx = -1;
      ny = -1;
      status = BAD;
    }
    nxny = nx * ny;

    data = new double[nxny];
    for(int j=0; j<ny; j++) {
      int j2 = j * nx;
      for(int i=0; i<nx; i++) {
        int ind = i + j2;
        data[ind] = din[i][j];
      }
    }

    // test the dimensions of things
    int tnx = xin.length;
    int tny = yin.length;
    if(tnx != nxny || tny != nxny) {
      int tnxny = tnx * tny;
      if(tnxny != nxny) {
        nxny = -1;
        status = BAD;
        return;
      }
      x = new double[nxny];
      y = new double[nxny];
      for(int j=0; j<ny; j++) {
        int j2 = j * nx;
        for(int i=0; i<nx; i++) {
          int ind = i + j2;
          x[ind] = xin[i];
          y[ind] = yin[j];
        }
      }
    } else {
      x = new double[nxny];
      y = new double[nxny];
      for(int i=0; i<nxny; i++) {
        x[i] = xin[i];
        y[i] = yin[i];
      }
    }

    init(map);

  }

  public void setUniformRectangularGrid(int m, int n) {
    // set up the block parameters
    if(m < 1) {
      status = BAD;
      return;
    } else {
      nx = m;
    }
    if(n < 1) {
      status = BAD;
      return;
    } else {
      ny = n;
    }
    nxny = nx * ny;
    // set up the grid
    x = new double[nxny];
    y = new double[nxny];
    double delx = 1.0 / ((double) nx);
    double dely = 1.0 / ((double) ny);
    for(int j=0; j<ny; j++) {
      int j2 = j * nx;
      for(int i=0; i<nx; i++) {
        int ind = i + j2;
        x[ind] = ( 0.5 + ((double) i) ) * delx;
        y[ind] = ( 0.5 + ((double) j) ) * dely;
      }
    }
  }

  public void init(int map) {
    filterMethod = NO_FILTER;
    // test the data to see if things are consistent
    if(checkData()) {
      // set up the data and compute the minimum and maximum values
      computeDomain();
      computeVicinity();
      computeMaxMinValues();
      fillColorMap();
    }
    // set the type of map
    if(map == 0) mapType = GREYSCALE_MAP;
    if(map == 1) mapType = COLOR_MAP;
    if(mapType < GREYSCALE_MAP) mapType = GREYSCALE_MAP;
    if(mapType > COLOR_MAP) mapType = COLOR_MAP;
  }

  public boolean checkData() {
    status = OK;
    if(x == null) return status = BAD;
    if(y == null) return status = BAD;
    if(data == null) return status = BAD;
    if(x.length != nxny) return status = BAD;
    if(y.length != nxny) return status = BAD;
    if(data.length != nxny) return status = BAD;
    return status;
  }

  public void computeDomain() {
    if(x != null) {
      xmin = x[0];
      xmax = xmin;
      for(int i=1; i<nxny; i++) {
        if(xmin > x[i]) xmin = x[i];
        if(xmax < x[i]) xmax = x[i];
      }
    } else {
      xmin = 0.0;
      xmax = 1.0;
    }
    if(y != null) {
      ymin = y[0];
      ymax = ymin;
      for(int j=1; j<nxny; j++) {
        if(ymin > y[j]) ymin = y[j];
        if(ymax < y[j]) ymax = y[j];
      }
    } else {
      ymin = 0.0;
      ymax = 1.0;
    }
    // set up the grid lines
    xgr = new double[nx+1];
    dx = new double[nx];
    if(nx > 1) {
      xgr[0] = x[0] - 0.5 * ( x[1] - x[0] );;
      for(int i=1; i<nx; i++) xgr[i] = 0.5 * ( x[i-1] + x[i] );
      xgr[nx] = x[nx-1] + 0.5 * ( x[nx-1] - x[nx-2] );
    } else {
      xgr[0] = x[0] - 0.5;
      xgr[ny] = x[ny] + 0.5;
    }
    for(int i=0; i<nx; i++) dx[i] = xgr[i+1] - xgr[i];
    ygr = new double[ny+1];
    dy = new double[ny];
    if(ny > 1) {
      ygr[0] = y[0] - 0.5 * ( y[ny] - y[0] );
      for(int j=1; j<ny; j++) ygr[j] = 0.5 * ( y[(j-1)*nx] + y[j*nx] );
      ygr[ny] = y[nxny-1] + 0.5 * ( y[nxny-1] - y[nxny-nx-1] );
    } else {
      ygr[0] = y[0] - 0.5;
      ygr[ny] = y[ny] + 0.5;
    }
    for(int j=0; j<ny; j++) dy[j] = ygr[j+1] - ygr[j];
  }

  public void computeVicinity() {
    // get sorted versions of the x spatial locations
    double [] v = new double[nxny];
    for(int i=0; i<nx; i++) v[i] = x[i];
    Arrays.sort(v);
    // compute the maximum distance
    double diff = Math.abs(v[nxny-1]-v[0]);
    for(int i=2; i<nxny; i++) {
      double tmp = Math.abs(v[i]-v[i-1]);
      if(tmp != 0.0 && tmp < diff) diff = tmp;
    }
    // get sorted versions of the y spatial locations
    v = new double[nxny];
    for(int i=0; i<nxny; i++) v[i] = y[i];
    Arrays.sort(v);
    // compute the maximum distance
    diff = Math.abs(v[nxny-1]-v[0]);
    for(int i=2; i<nxny; i++) {
      double tmp = Math.abs(v[i]-v[i-1]);
      if(tmp != 0.0 && tmp < diff) diff = tmp;
    }
    // use the vicinity factor
    vicinity = diff;
  }

  public void computeMaxMinValues() {
    if(data != null) {
      zmin = data[0];
      zmax = zmin;
      for(int l=1; l<nxny; l++) {
        if(zmin > data[l]) zmin = data[l];
        if(zmax < data[l]) zmax = data[l];
      }
    } else {
      zmin = 0.0;
      zmax = 1.0;
    }
    zbgValue = zmin - 100.;
  }

  public void fillColorMap() {
    if(mapType == PPMGraphics.GREYSCALE_MAP) {
      for(int i=0; i<255; i++) {
        color[i][0] = i;
        color[i][1] = i;
        color[i][2] = i;
      }
    }
    if(mapType == PPMGraphics.COLOR_MAP) {
      // set the white color
      color[254][0] = 255;
      color[254][1] = 255;
      color[254][2] = 255;
      for(int i=0; i<255; i++) {
        color[i][0] = red(i,256);
        color[i][1] = green(i,256);
        color[i][2] = blue(i,256);
        // enforce the appropriate conditions on the limits of the colors
        if(color[i][0] < 0)   color[i][0] = 0;
        if(color[i][0] > 255) color[i][0] = 255;
        if(color[i][1] < 0)   color[i][1] = 0;
        if(color[i][1] > 255) color[i][1] = 255;
        if(color[i][2] < 0)   color[i][2] = 0;
        if(color[i][2] > 255) color[i][2] = 255;
      }
      // set the last color to black
      color[255][0] = 0;
      color[255][1] = 0;
      color[255][2] = 0;
    }
  }

  public int blue(int ival, int ncols) {
    double xval = ((double) ival) / ncols;
    double realblue = Math.exp(-26.6403 + 39.467 * xval)
                        / ( 1.0 + Math.exp(-26.6403 + 39.467 * xval ) );
    return (int) (((double)ncols) * realblue );
  }

  public int green(int ival, int ncols) {
    double xval = ((double) ival) / ncols;
    double xarg = ( xval - 0.5 ) * ( xval - 0.5 );
    double realgreen = Math.exp(-34.1124 * xarg );
    return (int) (((double) ncols) * realgreen );
  }

  public int red(int ival, int ncols) {
    double xval = ((double) ival) / ncols;
    double realred = Math.exp(24.667 - 39.467 * xval)
               / ( 1.0 + Math.exp(24.667 - 39.467 * xval ) );
    return (int) (((double) ncols) * realred);
  }

  public void setLinearFilter() {
    filterMethod = LINEAR_FILTER;
  }

  public void setCubicFilter() {
    filterMethod = CUBIC_FILTER;
  }

  public void setNoClipping() {
    clippingMethod = NO_CLIPPING;
  }

  public void setRectangularClipping(double xl, double yl,
                                     double xr, double yr) {
    clippingMethod = RECTANGULAR_CLIPPING;
    xClipLeft = xl;
    yClipLeft = yl;
    xClipRight = xr;
    yClipRight = yr;
  }

  public void setUnitCircleClipping() {
    clippingMethod = UNIT_CIRCLE_CLIPPING;
    xClipCenter = 0.0;
    yClipCenter = 0.0;
    clipRadius = 1.0;
  }

  public void setCircleClipping(double xctr, double yctr, double rad) {
    clippingMethod = CIRCLE_CLIPPING;
    xClipCenter = xctr;
    yClipCenter = yctr;
    clipRadius = Math.abs(rad);
  }

  public boolean rectangularClipping(double xval, double yval) {
    if(xval < xClipLeft) return false;
    if(xval > xClipRight) return false;
    if(yval < yClipLeft) return false;
    if(yval > yClipRight) return false;
    return true;
  }

  public boolean unitCircleClipping(double xval, double yval) {
    double rval = Math.sqrt( xval * xval + yval * yval );
    if(rval <= 1.0) return true;
    return false;
  }

  public boolean circleClipping(double xval, double yval) {
    double xtmp = xval - xClipCenter; 
    double ytmp = yval - yClipCenter; 
    double rval = Math.sqrt( xtmp * xtmp + ytmp * ytmp );
    if(rval <= clipRadius) return true;
    return false;
  }

  public void setBorderColor(Color c) {
    borderColor = c;
  }

  public void setBorderValue(int b) {
    if(b>=0) borderValue = b;
  }

  public void setBorder(boolean b) {
    if(b) {
      border = true;
    } else {
      border = false;
    }
  }

  // this is used to build a simple image for visualizing
  public Image buildImage(Component co, int d) {
    return buildImage(co, d, d);
  }

  // this is used to build a simple image for visualizing
  public Image buildImage(Component co, int w, int h) {
    if(status == BAD) {
      System.out.println("Something Wrong with data in buildImage()");
      return null;
    }
    int tw = w;
    int th = h;
    if(border) {
      tw = w + 2 * borderValue;
      th = h + 2 * borderValue;
    }
    Image img = co.createImage(tw, th);
    Graphics gimg = img.getGraphics();
    gimg.setColor(borderColor);
    gimg.fillRect(0,0,tw,th);

    int nxpixels;
    int nypixels;
    // compute the maximum and minimum values
    computeMaxMinValues();

    // compute the color increment information
    double slope = 0.0;
    if(zmin != zmax) slope = 256.0 / ( zmax - zmin );
    double tint = - slope * zmin;

    // compute the aspect ratio of the data based on the domain
    double aspectratio = ( ygr[ny] - ygr[0] ) / ( xgr[nx] - xgr[0] );
    // get the number of pixels in the coordinate directions
    if(aspectratio > 1.0) {
      nxpixels = w;
      if(nx > nxpixels) nxpixels = nx;
      nypixels = ((int) (aspectratio * nxpixels));
    } else {
      nypixels = h;
      if(ny > nypixels) nypixels = ny;
      nxpixels = ((int) (nypixels / aspectratio));
    }
    // compute the increments in each direction
    double delx = ( xgr[nx] - xgr[0] ) / ((double) nxpixels);
    double dely = ( ygr[ny] - ygr[0] ) / ((double) nypixels);

    // check for the filter application
    setFilter();

    // loop over the pixels in the image
    double yval = ygr[ny] - 0.5 * dely;
    for(int j=0; j<nypixels; j++) {
      double xval = xgr[0] + 0.5 * delx;
      for(int i=0; i<nxpixels; i++) {
        int [] c = getColorValue(xval, yval);
        gimg.setColor(new Color(c[0], c[1], c[2]));
        gimg.fillRect(i+borderValue, j+borderValue, 1, 1);
        xval = xval + delx;
      }
      yval = yval - dely;
    }

    return img;
  }

  public void setFilter() {
    if(filterMethod == LINEAR_FILTER) filterCoefficients = linearFilter();
    if(filterMethod == CUBIC_FILTER) filterCoefficients = cubicFilter();
  }

  public double [][] linearFilter() {
    double [][] acoef = new double[nxny][4];
    double [][] amat = new double[4][4];
    double [] rhs = new double[4];
    double [] sol = new double[4];
    int [] ipvt = new int[4];
    double [] scal = new double[4];
    for(int j=0; j<ny; j++) {
      int j2 = j * nx;
      for(int i=0; i<nx; i++) {
        int ind = i + j2;
        // first row
        amat[0][0] = 1.0;
        amat[0][1] = xgr[i];
        amat[0][2] = ygr[j];
        amat[0][3] = xgr[i] * ygr[j];
        rhs[0] = corneravg(i, j, -1, -1);
        // second row
        amat[1][0] = 1.0;
        amat[1][1] = xgr[i+1];
        amat[1][2] = ygr[j];
        amat[1][3] = xgr[i+1] * ygr[j];
        rhs[1] = corneravg(i, j,  1, -1);
        // third row
        amat[2][0] = 1.0;
        amat[2][1] = xgr[i];
        amat[2][2] = ygr[j+1];
        amat[2][3] = xgr[i] * ygr[j+1];
        rhs[2] = corneravg(i, j, -1,  1);
        // last row
        amat[3][0] = 1.0;
        amat[3][1] = xgr[i+1];
        amat[3][2] = ygr[j+1];
        amat[3][3] = xgr[i+1] * ygr[j+1];
        rhs[3] = corneravg(i, j,  1,  1);
        // solution of the 4x4 system
        sol = gsolve(amat, rhs);
        // keep the solution for later use
        for(int l=0; l<4; l++) acoef[ind][l] = sol[l];
      }
    }
    return acoef;
  }

  public double [][] cubicFilter() {
    double [][] acoef = new double[nxny][10];
    double [][] amat = new double[5][5];
    double [] rhs = new double[5];
    double [] sol = new double[5];
    int [] ipvt = new int[5];
    double [] scal = new double[5];
    for(int j=0; j<ny; j++) {
      int j2 = j * nx;
      for(int i=0; i<nx; i++) {

        // get the appropriate grid values
        double xm = x[i];
        double ym = y[j];
        double xl = xm;
        double xr = xm;
        double yl = ym;
        double yr = ym;
        int indx = i;
        int indy = j2;
        int indxl = indx;
        int indxr = indx;
        int indyl = indy;
        int indyr = indy;
        if(i > 0) {
          xl = x[i-1];
          indxl = i + j2 - 1;
        } else {
          xl = x[0] - dx[0];
        }
        if(i < (nx-1)) {
          xr = x[i+1];
          indxr = i + j2 + 1;
        } else {
          xr = x[nx-1] + dx[nx-1];
        }
        if(j > 0) {
          yl = y[j-1];
          indyl = i + j2 - nx;
        } else {
          yl = y[0] - dy[0];
        }
        if(j < (ny-1)) {
          yr = y[j+1];
          indyr = i + j2 + nx;
        } else {
          yr = y[ny-1] + dy[ny-1];
        }

        // first row
        amat[0][0] = 1.0;
        amat[0][1] = xm;
        amat[0][2] = ym;
        amat[0][3] = xm * ym;
        amat[0][4] = ym * ym;
        rhs[0] = data[indx+indy];

        // second row
        amat[1][0] = 1.0;
        amat[1][1] = xl;
        amat[1][2] = yl;
        amat[1][3] = xl * yl;
        amat[1][4] = yl * yl;
        rhs[1] = data[indxl+indyl];

        // third row
        amat[2][0] = 1.0;
        amat[2][1] = xr;
        amat[2][2] = yl;
        amat[2][3] = xr * yl;
        amat[2][4] = yl * yl;
        rhs[2] = data[indxr+indyl];

        // fourth row
        amat[3][0] = 1.0;
        amat[3][1] = xl;
        amat[3][2] = yr;
        amat[3][3] = xl * yr;
        amat[3][4] = yr * yr;
        rhs[3] = data[indxl+indyr];

        // fifth row
        amat[4][0] = 1.0;
        amat[4][1] = xr;
        amat[4][2] = yr;
        amat[4][3] = xr * yr;
        amat[4][4] = yr * yr;
System.out.println("index:  " + (indxr+indyr));
System.out.println("index:  " + i + "  " + indxr);
System.out.println("index:  " + j + "  " + indyr);
System.out.println("data:  " + data[indxr+indyr]);
System.out.println("rhs:  " + rhs[4]);
        rhs[4] = data[indxr+indyr];

        // solution of the first 5x5 system
        sol = gsolve(amat, rhs);
        // keep the solution for later use
        for(int l=0; l<5; l++) acoef[indx+indy][l] = sol[l];

        // get the diagonal indices and spatial locations
        xm = x[i];
        ym = y[j];
        indx = i;
        indy = j2;
        indxl = indx;
        indxr = indx;
        indyl = indy;
        indyr = indy;
        if(i > 0) {
          xl = x[i-1];
          indxl = i - 1;
        } else {
          xl = x[1] - dx[0];
        }
        if(i < (nx-1)) {
          xr = x[i+1];
          indxr = i + 1;
        } else {
          xr = x[nx-1] + dx[nx-1];
        }
        if(j > 0) {
          yl = y[j-1];
          indyl = j2 - nx;
        } else {
          yl = y[1] - dy[0];
        }
        if(j < (ny-1)) {
          yr = y[j+1];
          indyr = j2 + nx;
        } else {
          yr = y[ny-1] + dy[ny-1];
        }

        // first row
        amat[0][0] = 1.0;
        amat[0][1] = xm;
        amat[0][2] = ym;
        amat[0][3] = xm * ym;
        amat[0][4] = ym * ym;
        rhs[0] = data[indx+indy];

        // second row
        amat[1][0] = 1.0;
        amat[1][1] = xl;
        amat[1][2] = yl;
        amat[1][3] = xl * yl;
        amat[1][4] = yl * yl;
        rhs[1] = data[indxl+indyl];

        // third row
        amat[2][0] = 1.0;
        amat[2][1] = xr;
        amat[2][2] = yl;
        amat[2][3] = xr * yl;
        amat[2][4] = yl * yl;
        rhs[2] = data[indxr+indyl];

        // fourth row
        amat[3][0] = 1.0;
        amat[3][1] = xl;
        amat[3][2] = yr;
        amat[3][3] = xl * yr;
        amat[3][4] = yr * yr;
        rhs[3] = data[indxl+indyr];

        // fifth row
        amat[4][0] = 1.0;
        amat[4][1] = xr;
        amat[4][2] = yr;
        amat[4][3] = xr * yr;
        amat[4][4] = yr * yr;
        rhs[4] = data[indxr+indyr];

        // solution of the first 5x5 system
        sol = gsolve(amat, rhs);
        // keep the solution for later use
        for(int l=0; l<5; l++) acoef[indx+indy][l+5] = sol[l];

      }
    }
    return acoef;
  }

  private double corneravg(int i, int j, int iflag, int jflag) {
    int i1 = i;
    if(iflag < 0) {
      if(i > 0) i1 = i + iflag;
    } else {
      if(i < (nx-1)) i1 = i + iflag;
    }
    int j1 = j;
    if(jflag < 0) {
      if(j > 0) j1 = j1 + jflag;
    } else {
      if(j < (ny-1)) j1 = j + jflag;
    }
    int [] index = new int[4];
    index[0] = i + j * nx;
    index[1] = i + j1 * nx;
    index[2] = i1 + j * nx;
    index[3] = i1 + j1 * nx;
    double sum = 0.0;
    for(int l=0; l<4; l++) sum = sum + data[index[l]];
    return (0.25 * sum);
  }

  private double [] gsolve(double [][] amat, double [] rhs) {
    int m = amat.length;
    int n = amat[0].length;
    int [] ipvt = new int [m];
    double [] s = new double [m];
    double [] b = new double [m];
    double [] sol = new double [m];
    for(int i=0; i<m; i++) {
      ipvt[i] = i;
      double smax = Math.abs(amat[i][0]);
      for(int j=1; j<n; j++) {
        double val = Math.abs(amat[i][j]);
        if(smax < val) smax = val;
      }
      s[i] = smax;
      b[i] = rhs[i];
    }
    for(int k=0; k<n-1; k++) {
      int j = k;
      double rmax = 0.0;
      for(int i=k; i<n; i++) {
        double r = Math.abs(amat[ipvt[i]][k]) / s[ipvt[i]];
        if(rmax < r) {
          j = i;
          rmax = r;
        }
      }
      int lk = ipvt[j];
      ipvt[j] = ipvt[k];
      ipvt[k] = lk;
      for(int i=k+1; i<n; i++) {
        double xmult = amat[ipvt[i]][k] / amat[lk][k];
        for(int l=k+1; l<n; l++) {
          amat[ipvt[i]][l] = amat[ipvt[i]][l] - xmult * amat[lk][l];
        }
        amat[ipvt[i]][k] = xmult;
      }
    }

    // solution
    for(int k=0; k<n-1; k++) {
      for(int i=k+1; i<n; i++) {
        b[ipvt[i]] = b[ipvt[i]] - amat[ipvt[i]][k] * b[ipvt[k]];
      }
    }

    if(Math.abs(amat[ipvt[m-1]][m-1]) > 0.000001) {
      sol[m-1] = b[ipvt[m-1]] / amat[ipvt[m-1]][m-1];
    } else {
      sol[m-1] = 0.0;
    }

    for(int i=m-2; i>=0; i--) {
      double sum = b[ipvt[i]];
      for(int l=i+1; l<n; l++) {
        sum = sum - amat[ipvt[i]][l] * sol[l];
      }
      sol[i] = sum / amat[ipvt[i]][i];
    }
    return sol;
  }

  public double evalLinearFilter(double xpt, double ypt, double[][] acoef) {
    double val = acoef[currentIndex][0]
               + acoef[currentIndex][1] * xpt
               + acoef[currentIndex][2] * ypt
               + acoef[currentIndex][3] * xpt * ypt;
    return val;
  }

  public double evalCubicFilter(double xpt, double ypt, double[][] acoef) {
    double xsq = xpt * xpt;
    double ysq = ypt * ypt;
    double val1 = acoef[currentIndex][0]
                + acoef[currentIndex][1] * xpt
                + acoef[currentIndex][2] * ypt
                + acoef[currentIndex][3] * xsq
                + acoef[currentIndex][4] * ysq;
    double val2 = acoef[currentIndex][5]
                + acoef[currentIndex][6] * xpt
                + acoef[currentIndex][7] * ypt
                + acoef[currentIndex][8] * xsq
                + acoef[currentIndex][9] * ysq;
    return 0.5*(val1+val2);
  }

  public int [] getColorValue(double xval, double yval) {
    int [] bgColor = {color[0][1], color[1][0], color[2][0]};

    // compute the function value
    double zval = evaluateFunction(xval, yval);
    if(filterMethod == LINEAR_FILTER)
                     zval = evalLinearFilter(xval, yval, filterCoefficients);
    if(clippingMethod > 0) {
      if(clippingMethod == RECTANGULAR_CLIPPING)
                           if(!rectangularClipping(xval, yval)) zval = zbgValue;
      if(clippingMethod == UNIT_CIRCLE_CLIPPING)
                           if(!unitCircleClipping(xval, yval)) zval = zbgValue;
      if(clippingMethod == CIRCLE_CLIPPING)
                           if(!circleClipping(xval, yval)) zval = zbgValue;
    }

    // test to see if the background value is to be returned
    if(zval == zbgValue) return bgColor;

    // compute the color increment information
    double slope = 0.0;
    if(zmin != zmax) slope = 256.0 / ( zmax - zmin );
    double tint = - slope * zmin;

    // compute a color value
    int imageval = ((int) (slope * zval + tint)) + 1;
    if(imageval < 1) imageval = 1;
    if(imageval > 255) imageval = 255;

    // set up the output value
    int [] c = new int[3];
    c[0] = color[imageval-1][0];
    c[1] = color[imageval-1][1];
    c[2] = color[imageval-1][2];
    return c;

  }

  public int [] getColorValue(int ival, int rnx, int jval, int rny) {
    int [] bgColor = {color[0][1], color[1][0], color[2][0]};
    // test for an output of bounds problem
    if(ival < 0) return bgColor;
    if(ival > rnx) return bgColor;
    if(jval < 0) return bgColor;
    if(jval > rny) return bgColor;
    // set up the increments and such
    double delx = 1.0 / ((double) rnx);
    double dely = 1.0 / ((double) rny);
    // find the domain location of the current pixel
    double xfactor = ((double) ival) / ((double) (rnx-1)) + 0.5 * delx;
    double yfactor = ((double) jval) / ((double) (rny-1)) + 0.5 * dely;
    double xval = xmin + xfactor * ( xmax - xmin );
    double yval = ymin + yfactor * ( ymax - ymin );
    // find the appropriate domain color value
    return getColorValue(xval, yval);
  }

  public double evaluateFunction(double xval, double yval) {
    int ival = 0;
    double xdiff = Math.abs(xval-x[0]);
    double ydiff = Math.abs(yval-y[0]);
    double distsq = xdiff * xdiff + ydiff * ydiff;
    int ind = 0;
    while(ind < nxny) {
      double xtmp = Math.abs(x[ind]-xval);
      double ytmp = Math.abs(y[ind]-yval);
      double dtmpsq = xtmp * xtmp + ytmp * ytmp;
      if(dtmpsq <= distsq) {
        distsq = dtmpsq;
        ival = ind;
      }
      ind++;
    }
    // current index
    currentIndex = ival;
    // return the data value
    return data[ival];
  }

  public boolean writePPM(File f, int d) {
    return writePPM(f, d, d);
  }

  public boolean writePPM(File f, int w, int h) {
    if(status == BAD) {
      System.out.println("Something Wrong with data in writePPM()");
      return false;
    }
    // set up the file information
    BufferedOutputStream fos;
    if(f == null) return false;
    if(f.isDirectory()) return false;

    int nxpixels;
    int nypixels;
    // compute the maximum and minimum values
    computeMaxMinValues();

    // compute the color increment information
    double slope = 0.0;
    if(zmin != zmax) slope = 256.0 / ( zmax - zmin );
    double tint = - slope * zmin;

    // compute the aspect ratio of the data based on the domain
    double aspectratio = ( ygr[ny] - ygr[0] ) / ( xgr[nx] - xgr[0] );
    // get the number of pixels in the coordinate directions
    if(aspectratio > 1.0) {
      nxpixels = w;
      if(nx > nxpixels) nxpixels = nx;
      nypixels = ((int) (aspectratio * nxpixels));
    } else {
      nypixels = h;
      if(ny > nypixels) nypixels = ny;
      nxpixels = ((int) (nypixels / aspectratio));
    }
    // compute the increments in each direction
    double delx = ( xgr[nx] - xgr[0] ) / ((double) nxpixels);
    double dely = ( ygr[ny] - ygr[0] ) / ((double) nypixels);

    // set up the filter
    setFilter();

    // set the first line of the file
    setPPMLine(w+2*borderValue, h+2*borderValue);
    // build a file writer
    try{fos = new BufferedOutputStream(new FileOutputStream(f));
    } catch (IOException fnfe) {
      System.out.println(fnfe.toString());
      return false;
    }
    // write out the first line to indicate that the file is a ppm file
    try{
      fos.write(ppmLine,0,ppmLine.length);
    } catch(IOException ioe) {
      return false;
    }
    // now write out the rest of the data
    System.out.println("writing PPM file:");
    int nxlim = nxpixels + 2 * borderValue;
    int nylim = nypixels + 2 * borderValue;
    double yval = ygr[ny] - 0.5 * dely;
    for(int j=0; j<nylim; j++) {
      double xval = xgr[0] + 0.5 * delx;
      for(int i=0; i<nxlim; i++) {
        int [] c = getColorValue(xval, yval);
        if(i < borderValue || i > (nxlim-borderValue)) {
          c[0] = 0;
          c[1] = 0;
          c[2] = 0;
        }
        if(j < borderValue || j > (nylim-borderValue)) {
          c[0] = 0;
          c[1] = 0;
          c[2] = 0;
        }
        char b0 = (char) c[0];
        char b1 = (char) c[1];
        char b2 = (char) c[2];
        try{
          fos.write(c[0]);
          fos.write(c[1]);
          fos.write(c[2]);
        } catch(IOException ioe) {
          return false;
        }
        xval = xval + delx;
      }
      yval = yval - dely;
    }
    // flush the file writer
    try {
      fos.flush();
    } catch(IOException ioe) {
      return false;
    }
    // close the file writer
    try{
      fos.close();
    } catch(IOException ioe) {
    }
    System.out.println("done writing PPM file:");
    return true;
  }

  private void setPPMLine(int w, int h) {
    String s = "P6 "
             + new Integer(w).toString().trim()
             + " "
             + new Integer(h).toString().trim()
             + " 255\n";
    ppmLine = s.getBytes();
  }

  public void setData(double [] d) {
    if(d == null) return;
    int n = d.length;
    if(n != nxny) return;
    data = new double[nxny];
    for(int i=0; i<nxny; i++) data[i] = d[i];
    computeMaxMinValues();
  }

  public void setData(double [][] d) {
    if(d == null) return;
    if(d.length != nx) return;
    if(d[0].length != ny) return;
    // grab the data
    data = new double[nxny];
    for(int j=0; j<ny; j++) {
      int j2 = j * nx;
      for(int i=0; i<nx; i++) {
        int ind = i + j2;
        data[ind] = d[i][j];
      }
    }
    computeMaxMinValues();
  }

  private void stopHere() {
    System.out.println("Stop Here");
    try{
      System.in.read();
      System.in.read();
    } catch(IOException ioe) {
    }
  }

  public static void main(String args[]) {
    Random ran = new Random();

    int nx = 16;
    double [] x = new double[nx];
    double delx = 1.0 / ((double) nx);
    x[0] = 0.5 * delx;
    for(int i=1; i<nx; i++) x[i] = x[i-1] + delx;

    int ny = 16;
    double dely = 1.0 / ((double) ny);
    double [] y = new double[ny];
    y[0] = 0.5 * dely;
    for(int j=1; j<ny; j++) y[j] = y[j-1] + dely;

    double [][] data = new double[nx][ny];
    double [] d1d = new double[nx*ny];
    for(int j=0; j<ny; j++) {
      int j2 = j * nx;
      for(int i=0; i<nx; i++) {
        data[i][j] = 100.0 * ran.nextDouble();
        d1d[i+j2] = data[i][j];
      }
    }

    System.out.println("----> test a");

    PPMGraphics ppmg = new PPMGraphics(nx, ny, d1d, PPMGraphics.GREYSCALE_MAP);
    File f = new File("testa.ppm");
//    ppmg.writePPM(f, 128, 128);

    System.out.println("----> test b");

    ppmg = new PPMGraphics(data, PPMGraphics.GREYSCALE_MAP);
    f = new File("testb.ppm");
//    ppmg.writePPM(f, 128, 128);

    System.out.println("----> test 1");

    ppmg = new PPMGraphics(x, y, data, PPMGraphics.GREYSCALE_MAP);
    f = new File("test1.ppm");
//    ppmg.writePPM(f, 128, 128);

    System.out.println("----> test 2");

//    // do a 1-d shading test in the x direction
    for(int j=0; j<ny; j++) {
      for(int i=0; i<nx; i++) {
        data[i][j] = i+1;
      }
    }
    ppmg.setData(data);
    f = new File("test2.ppm");
//    ppmg.writePPM(f, 128, 128);

    System.out.println("----> test 3");

    // do a 1-d shading test in the y direction
    for(int j=0; j<ny; j++) {
      for(int i=0; i<nx; i++) {
        data[i][j] = j+1;
      }
    }
    ppmg.setData(data);
    f = new File("test3.ppm");
//    ppmg.writePPM(f, 128, 128);

    System.out.println("----> test 4");

    // do a diagonal shading test
    for(int j=0; j<ny; j++) {
      for(int i=0; i<nx; i++) {
        data[i][j] = i+j+1;
      }
    }
    ppmg.setData(data);
    f = new File("test4.ppm");
//    ppmg.writePPM(f, 128, 128);

    System.out.println("----> test 5");

    // do a diagonal shading test
    for(int j=0; j<ny; j++) {
      for(int i=0; i<nx; i++) {
        data[i][j] = i+j+1;
      }
    }
    ppmg.setData(data);
    ppmg.setUnitCircleClipping();
    f = new File("test5.ppm");
//    ppmg.writePPM(f, 128, 128);

    System.out.println("----> test 6");

    // do a diagonal shading test
    for(int j=0; j<ny; j++) {
      for(int i=0; i<nx; i++) {
        data[i][j] = i+j+1;
      }
    }
    ppmg.setData(data);
    ppmg.setCircleClipping(0.25, 0.25, 0.5);
    f = new File("test6.ppm");
//    ppmg.writePPM(f, 128, 128);

    System.out.println("----> test 7");

    // do a diagonal shading test
    for(int j=0; j<ny; j++) {
      for(int i=0; i<nx; i++) {
        data[i][j] = i+j+1;
      }
    }
    ppmg.setData(data);
    ppmg.setRectangularClipping(0.33, 0.33, 0.57, 0.75);
    f = new File("test7.ppm");
//    ppmg.writePPM(f, 128, 128);

    System.out.println("----> test 8");

    // do a diagonal shading test
    for(int j=0; j<ny; j++) {
      for(int i=0; i<nx; i++) {
        data[i][j] = i+1;
      }
    }
    ppmg.setData(data);
    ppmg.setNoClipping();
    ppmg.setLinearFilter();
    ppmg.setBorder(true);
    ppmg.setBorderValue(10);
    f = new File("test8.ppm");
    ppmg.writePPM(f, 128, 128);

    System.out.println("----> test 9");

//    // do a diagonal shading test
//    for(int j=0; j<ny; j++) {
//      for(int i=0; i<nx; i++) {
//        data[i][j] = i+1;
//      }
//    }
//    ppmg.setData(data);
//    ppmg.setNoClipping();
//    ppmg.setCubicFilter();
//    f = new File("test9.ppm");
//    ppmg.writePPM(f, 128, 128);

  }

  private int nx;
  private int ny;
  private int nxny;
  private double [] x;
  private double [] y;
  private double [] xgr;
  private double [] ygr;
  private double [] dx;
  private double [] dy;
  private double [] data;
  private int currentIndex;

  private double xmin;
  private double xmax;
  private double ymin;
  private double ymax;
  private double zmin;
  private double zmax;
  private double zbgValue;

  private boolean status = true;

  private double [][] filterCoefficients = null;

  private File ppmFile;
  private byte [] ppmLine;
  private int [][] color = new int [256][3];
  private double vicinity;

  private int mapType = 1;
  public static final int GREYSCALE_MAP = 0;
  public static final int COLOR_MAP = 1;

  private static final boolean OK = true;
  private static final boolean BAD = false;

  private int filterMethod = 0;
  private static final int NO_FILTER = 0;
  private static final int LINEAR_FILTER = 1;
  private static final int CUBIC_FILTER = 2;
  private int clippingMethod = 0;
  private static final int NO_CLIPPING = 0;
  private static final int RECTANGULAR_CLIPPING = 1;
  private double xClipLeft;
  private double yClipLeft;
  private double xClipRight;
  private double yClipRight;
  private static final int UNIT_CIRCLE_CLIPPING = 2;
  private static final int CIRCLE_CLIPPING = 3;
  private double clipRadius;
  private double xClipCenter;
  private double yClipCenter;

  private boolean border = false;
  private Color borderColor = Color.black;
  private int borderValue = 0;

}
