/*
 * SimulatorGUI class: preproCode
 *
 */

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.Arrays;
import java.util.Observable;
import java.util.Random;
import java.util.StringTokenizer;

public class PreproCode extends Observable implements Runnable { 

  private static String userHome = System.getProperty("user.home");

  private static String preproCodeLocation = userHome
                           + File.separator
                           + "/java/JHomogenizer-new/src/f77/PermFunctions/prepro";

  private static String preproCodeInputFileName = "__upscaling__";

  public PreproCode()
  {
    // set up the computational parameters
    done = false;
    ok = false;

    // set up the thread and run it to compute the average value for the data.
    computeThread = new Thread(this);
    computeThread.start();

  }

  public void run() {
    if(computeValue()) {
      ok = true;
    } else {
      ok = false;
    }
    done = true;
    computeThread = null;
  }

  // Step 2.
  // Methods for setting and getting the executable location and the names of
  // the input and  output files. you might want to use these from another
  // piece of code.

  public void setHomCodeLocation(String s) {
    preproCodeLocation = s;
  }
  public void setHomCodeInputFileName(String s) {
    preproCodeInputFileName = s;
  }
  public String getHomCodeLocation() {
    return preproCodeLocation;
  }
  public String getHomCodeInputFileName() {
    return preproCodeInputFileName;
  }

  /*
   * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   *                                                                         *
   *    methods for simple averaging                                         *
   *                                                                         *
   * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   */

  public boolean computeValue() {
    String fileSeparator = File.separator;

    // check to make sure that the software location and input and output
    // file information has been set

    if(preproCodeLocation == null) return false;
    if(preproCodeInputFileName == null) return false;

    // Step 3.
    // this line sets the name of the shell script that will be used to
    // execute the native code

    String shellName = "__runprepro__";

    File fd = new File(shellName);
    try{
      FileWriter fw = new FileWriter(fd);
      String line;
      try{

        line = "# shell script to run HomCode from JHomogenizer\n";
        fw.write(line, 0, line.length());
        line = "# generated automatically - can be erased at any time\n\n";
        fw.write(line, 0, line.length());

        line = "# run the code\n"
             + preproCodeLocation + " " + preproCodeInputFileName
             + "\n\n";
        fw.write(line, 0, line.length());

      } catch(IOException ioe) {
        return false;
      }
      fw.flush();
      fw.close();

    } catch(IOException ioe) {
      return false;
    }

    // we need to tell the system that the shell script is exectuable
    ExternalProcess plusXProcess = new ExternalProcess(null, 2000);
    String command = "chmod +x "
                   + shellName;
    plusXProcess.setCommand(command);
    plusXProcess.runCommand();
    // test to see if the code is done running or not
    while(plusXProcess.getStatus()) {
      try {
        Thread.sleep(100);
      } catch(InterruptedException ie) {}
    }

    // execute the command line
    ExternalProcess preproCodeProcess = new ExternalProcess(null, 2000);
    command = "/bin/bash  ."+fileSeparator + shellName;
    preproCodeProcess.setCommand(command);
    preproCodeProcess.runCommand();

    // test to see if the code is done running or not
    while(preproCodeProcess.getStatus()) {
      try {
        Thread.sleep(1000);
      } catch(InterruptedException ie) {}
    }

    // now we are ready to pull the data back into the deal if the code
    // ran correctly
    if(preproCodeProcess.getExitValue() != 0) return false;

    notifyObservers();
    return true;

  }
 
  public boolean isValueOK() {
    return ok;
  }
 
  public boolean getStatus() {
    return done;
  }
 
 
  /*
   * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   *                                                                         *
   *    main method for testing the interface                                *
   *                                                                         *
   * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
   */
  public static void main(String args[]) {
    String line;
    PreproCode aa;
      aa = new PreproCode();
      while(!aa.getStatus()) {
        System.out.println("isValueOK() test:   " + aa.isValueOK());
        System.out.println("getStatus() test:   " + aa.getStatus());
        System.out.println("   ");
        // sleep the process and test again
        try { 
             Thread.sleep(1000);
        } catch(InterruptedException ie) {
          break;
        }
      }
  }

  // a thread variable for running the neccesary averaging
  private Thread computeThread;
  private boolean done = false;
  private boolean ok = false;
}
