public class TensorProduct2D extends Basis2DObject{
  // constructor with all arguments
  public TensorProduct2D() {
    this(null, null);
  }
  public TensorProduct2D(Basis1DObject bx, Basis1DObject by) {
    basisx = bx;
    basisy = by;
  }
  public double evaluate(double x, double y) {
    double xval = basisx.evaluate(x);
    double yval = basisy.evaluate(y);
    return xval * yval;
  }
  public double evaluate(double x, double xleft, double xright,
                         double y, double yleft, double yright) {
    double xval = basisx.evaluate(x, xleft, xright);
    double yval = basisy.evaluate(y, yleft, yright);
    return xval * yval;
  }
  public double dbydx(double x, double y) {
    double xval = basisx.derivative(x);
    double yval = basisy.evaluate(y);
    return xval * yval;
  }
  public double dbydx(double x, double xleft, double xright,
                      double y, double yleft, double yright) {
    double xval = basisx.derivative(x, xleft, xright);
    double yval = basisy.evaluate(y, yleft, yright);
    return xval * yval;
  }
  public double dbydy(double x, double y) {
    double xval = basisx.evaluate(x);
    double yval = basisy.derivative(y);
    return xval * yval;
  }
  public double dbydy(double x, double xleft, double xright,
                      double y, double yleft, double yright) {
    double xval = basisx.evaluate(x, xleft, xright);
    double yval = basisy.derivative(y, yleft, yright);
    return xval * yval;
  }
  public void setDomain(double xl, double xr,
                        double yl, double yr) {
    basisx.setDomain(xl,xr);
    basisy.setDomain(yl,yr);
  }
  public double getXLeftDomain() {
    return basisx.getLeftDomain();
  }
  public double getXRightDomain() {
    return basisx.getRightDomain();
  }
  public double getYLeftDomain() {
    return basisy.getLeftDomain();
  }
  public double getYRightDomain() {
    return basisy.getRightDomain();
  }
  public static void main(String args[]) {
    PiecewiseConstant1D pwcx = new PiecewiseConstant1D();
    PiecewiseConstant1D pwcy = new PiecewiseConstant1D();
    TensorProduct2D tp2d
         = new TensorProduct2D((Basis1DObject) pwcx, (Basis1DObject) pwcy);
    double dx = 0.01;
    double xleft = 0.0;
    double xright = 1.0;
    double dy = 0.05;
    double yleft = -0.5;
    double yright = 1.0;
    for(int j=-4; j<10; j++) {
      double yval = j * dy;
      for(int i=-4; i<10; i++) {
        double xval = i * dx;
        System.out.println("  " + i + "  "
           + tp2d.evaluate(xval,xleft,xright,yval,yleft,yright)
           + "  " + tp2d.dbydx(xval,xleft,xright,yval,yleft,yright)
           + "  " + tp2d.dbydy(xval,xleft,xright,yval,yleft,yright));
      }
    }
  }

  private Basis1DObject basisx;
  private Basis1DObject basisy;

}
