c
c pressure matrix storage block
c
c this common block contains the storage for the velocity mass matrices
c of the form
c
c         [  m11  m12  m13  ]
c     m = [  m21  m22  m23  ]
c         [  m31  m32  m33  ]
c
c and the right hand side of the equations obtained in the mixed
c finite element method, the matrices have a beginning letter a, but then
c have the exact name as above for the 2-4 characters of the name of the
c array. the d in the name stands for diagonal and the s and u stand for
c super and sub diagonals respectively.
c
c   am11_ - the diagonal for the matrix in the x component of the
c           velocity
c   am22_ - the diagonal for the matrix in the y component of the
c           velocity
c   am33_ - the diagonal for the matrix in the z component of the
c           velocity
c   am12 - the contributions due to the (1,2) entry in the inverted permeability
c          tensor. note that this is not the matrix but is information used in
c          the matrix multiply to get the action of the matrix
c   am13 - the contributions due to the (1,3) entry in the inverted permeability
c          tensor. note that this is not the matrix but is information used in
c          the matrix multiply to get the action of the matrix
c   am21 - the contributions due to the (2,1) entry in the inverted permeability
c          tensor. note that this is not the matrix but is information used in
c          the matrix multiply to get the action of the matrix
c   am23 - the contributions due to the (2,3) entry in the inverted permeability
c          tensor. note that this is not the matrix but is information used in
c          the matrix multiply to get the action of the matrix
c   am31 - the contributions due to the (3,1) entry in the inverted permeability
c          tensor. note that this is not the matrix but is information used in
c          the matrix multiply to get the action of the matrix
c   am32 - the contributions due to the (3,2) entry in the inverted permeability
c          tensor. note that this is not the matrix but is information used in
c          the matrix multiply to get the action of the matrix
c
      common /hommfelineq/
     1    am11d(nhomvelx),am11s(nhomvelx),am11u(nhomvelx)
     1   ,am22d(nhomvely),am22s(nhomvely),am22u(nhomvely)
     1   ,am33d(nhomvelz),am33s(nhomvelz),am33u(nhomvelz)
     1   ,am12(nhommfe),am13(nhommfe),am21(nhommfe)
     1   ,am23(nhommfe),am31(nhommfe),am32(nhommfe)
c
c the right hand side vector for the linear system of equations
c
c   rhshommfe - the right hand side for the pressure part of the equations in
c               the mixed finite element method
c   rhshomvx - the right hand side for the first compnent of the velocity in the
c           mixed finite element method
c   rhshomvy - the right hand side for the second compnent of the velocity in
c           the mixed finite element method
c   rhshomvz - the right hand side for the third compnent of the velocity in the
c           mixed finite element method
c
     1,rhshommfe(nhommfe,3),rhshomvx(nhomvelx,3),rhshomvy(nhomvely,3)
     1,rhshomvz(nhomvelz,3)
c
c the diagonal term due to the time dependence in the parabolic equation
c
c   diaghommfe - the time dependent part on the left hand side in the pressure
c             equation. note that this adds to the diagonal of the mixed
c             finite element pressure equation matrix
c
     1,diaghommfe(nhommfe)
c
c parameters to control the computation of the matrices
c
c   ihommfeprmchng - a flag to determine if the mobility matrix need to be
c                 recomputed at the ned time step
c                 ihommfeprmchng = 0; no need to recompute the matrix 
c                 ihommfeprmchng = 1; need to recompute the matrix 
c   ihommfemobmat - a flag to determine if the mobility matrix need to be
c                recomputed at the ned time step
c                imobmat = 0; no need to recompute the matrix 
c                imobmat = 1; need to recompute the matrix 
c   ihommfemferdc - this parameter tells the code to reduce the mass matrix for
c                the diagonal velocity blocks
c                ihommfemferdc = 1; reduce
c                ihommfemferdc = anything else; no reduction
     1,ihommfeprmchng,ihommfemobmat,ihommfemferdc
c
c mass matrix storage
c
c   admssx,asmssx - arrays containing the mass matrix in the x direction
c   admssy,asmssy - arrays containing the mass matrix in the y direction
c   admssz,asmssz - arrays containing the mass matrix in the z direction
c
     1               ,admssx(nhomvelx),asmssx(nhomvelx)
     1               ,admssy(nhomvely),asmssy(nhomvely)
     1               ,admssz(nhomvelz),asmssz(nhomvelz)
c
c convergence and iteration parameters for the linear solution process
c
c   maxiter - the maximum number iteration allowed in the iterative solver
c   hommfecgeps - the error tolerance for the residual norm in the iterative
c              solver
c   hommfecgthrshl - the lower threshold for the norm of the residual in the
c                 iterative solver
c   hommfecgthrshu - the upper threshold for the norm of the residual in the
c                 iterative solver
c
c the last two parameters are use to force the iterative solver to reduce the
c residual to a specified value (hommfecgthrshl). once this threshold is
c achieved in the iterates the upper value (hommfecgthrshu) is used to tell the
c code to kick out because the residual got too large. note that the solver
c always keeps the solution with the smallest residual.
c
     1,maxiter,hommfecgeps,hommfecgthrshl,hommfecgthrshu
