c
c this common block contains information and storage for the
c permeabilities for use in the the simulator.
c
c General comments about the data structures:
c
c 1. the arrays are one-dimensional for all cases. the arrays are ordered
c    lexicographically in the spatial grid.
c
c 2. The description of the variables is given below.
c
c 3. the parameter statements are included in this file to keep things a little
c    easier in terms of coding subroutines.
c
c 4. this common block MUST appear before all other permeability common blocks
c    like homog_common so that the parameter statement can be loaded for
c    setting up the array lengths
c
c parameter definitions for the permeability blocks
c
c  the following parameter statements set up the storage and the like for the
c 3-d permeability code
c
c   nperm - the maximum number of permeability nodes allowable in the simulation
c   nodeprm - the maximum number of nodes allowed in either coordinate
c             direction. Note that this should be the maximum of the
c             desired nodes in the x direction and the y direction
c
c relationships
c
c   nnprm = nxprm * nyprm * nzprm
c
c and then
c
c   nperm >= nnprm
c   nodeprm >= max(nxperm,nyperm,nzperm)
c
c note that upon change of the parameter file here, the entire library
c and all drivers must be recompiled and linked in order to ensure that
c the simulators are on the same wavelength. There are facilities
c already built to handle this in the utility directory under sim2p2d.
c
      parameter(nodeprm=300, nperm=100000)
c
c permeability variables
c
c   perm** - the array of permeability values for the components
c   perm**old - the array of old permeability values for the components
c               normally used for temporary storage
c   nxprm - the number of permeability nodes in the first coordinate direction
c   nyprm - the number of permeability nodes in the second coordinate direction
c   nzprm - the number of permeability nodes in the third coordinate direction
c
      common /permeability/ permxx(nperm),permxxold(nperm)
     1,                     permxy(nperm),permxyold(nperm)
     1,                     permxz(nperm),permxzold(nperm)
     1,                     permyx(nperm),permyxold(nperm)
     1,                     permyy(nperm),permyyold(nperm)
     1,                     permyz(nperm),permyzold(nperm)
     1,                     permzx(nperm),permzxold(nperm)
     1,                     permzy(nperm),permzyold(nperm)
     1,                     permzz(nperm),permzzold(nperm)
     1,nxprm,nyprm,nzprm,nxprmm1,nyprmm1,nzprmm1,nxprmp1,nyprmp1
     1,nzprmp1,nnprm,nnprmvx,nnprmvy,nnprmvz,nprmtot,nprmvx2d
     1,nprmvy2d,nprmvz2d
c
c some extra parameters for the higher dimensional comptuations
c
c   nxnyprm = nxprm * nyprm
c   nynzprm = nyprm * nzprm
c   nxnzprm = nxprm * nzprm
c
     1,nxnyprm,nynzprm,nxnzprm
c
c permeability mesh variables
c
c   xprm - the cell centers of the first coordinate in the permeability mesh
c   yprm - the cell centers of the second coordinate in the permeability mesh
c   zprm - the cell centers of the third coordinate in the permeability mesh
c   xgrprm - the grid lines of the first coordinate in the permeability mesh
c   ygrprm - the grid lines of the second coordinate in the permeability mesh
c   zgrprm - the grid lines of the third coordinate in the permeability mesh
c   dxprm - the cell sizes in the first coordinate in the permeability mesh
c   dyprm - the cell sizes in the second coordinate in the permeability mesh
c   dzprm - the cell sizes in the third coordinate in the permeability mesh
c
     1,xprm(0:nodeprm+1),yprm(0:nodeprm+1),zprm(0:nodeprm+1)
     1,xgrprm(0:nodeprm+1),ygrprm(0:nodeprm+1),zgrprm(0:nodeprm+1)
     1,dxprm(nodeprm),dyprm(nodeprm),dzprm(nodeprm)
c
c mask array for the permeability variable
c
c   maskprm - vector to mask out some permeability nodes - normally used to
c             communicate input to the code or to fix permeability that are
c             constant and known in the domain
c
     1,maskprm(nperm)
c
c the temporary mask vectors for the permeability and velocity variables
c
c   ibitprm - temporary masking vector for the permeability variable
c   ibitprmvx - temporary masking vector for the permeability grid line variable
c   ibitprmvy - temporary masking vector for the permeability grid line variable
c   ibitprmvz - temporary masking vector for the permeability grid line variable
c
     1,ibitprm(nperm),ibitprmvx(nperm),ibitprmvy(nperm),ibitprmvz(nperm)
c
c numerical methods information for the permeability routine
c
c   methavg - the method to use for the permeability translation
c             methavg=1; homogenization
c             methavg=2; arithmetic averaging
c             methavg=3; harmonic averaging
c             methavg=4; geometric averaging
c   prmcgtol - the tolerance to use in the conjugate gradient routine for
c              convergence
c
     1,methavg,prmcgtol
c
c the eigensystem or principle component information for the permeability
c
c   prmeigval - the eigenvalues or characteristic values
c   prmeigvec - the eigenvectors or characteristic directions
c   prmeigang - the angles that define the direction of the characteristic
c               coordinate system relative to the grid
c   prmdet - the product of the eigenvalues of the tensor at the points
c  
     1,prmeigval(nperm,3),prmeigvec(nperm,3,3),prmeigang(nperm,3)
     1,prmdet(nperm)
c
c zone map information - if there are only a few different values of the
c permeability a permeability zone map can be created.
c
c   iprmmap - in integer array that differentiates between various zones
c              of permeability - this is a single array
c  
     1,iprmmap(nperm)
c
c minimum and maximum values
c
c   prmprmmin - the minimum pressure computed in the simulation
c   prmprmmax - the maximum pressure computed in the simulation
c
     1,prmprmmin,prmprmmax
