c     Programme dispersion2d.f
c     
c     Programme principal permettant de traiter des cas 
c     de milieux poreux fractures en 2 D
c
c     Pour calcul du tenseur de dispersion dans un milieu fracture
c
c     
c     Ecrit par Benedicte VIDAL
c     Stage IPSN
c         suivi par Brahim AMAZIANE et Christophe SERRES
c     
c     avril - septembre 2001
      
      program dispersion
      
*     definition des parameters ou nv est la dimension des vecteurs ie le nombre d inconnues 
*     et nm est la dimension de la matrice       
* pour site ASPO : MAIS TAILLE DES VECTEURS TROP GRANDE OU TROP DE VECTEURS DE GRANDE TAILLE
*      parameter (nv=25000002,nm=225000002)
*     pour sites TOURNEMIRE, VIENNE
      parameter (nv=26000,nm=230000)
      
*      implicit none
      
c     declaration des variables
*     gh pour gradient hydraulique
*     vf=(vf1,vf2) vecteur vitesse v0
*     kvf=(kvf1,kvf2) vecteur moyenne de vf
*     p11,p22 correspondent aux elements k11 et k22 de K
*     kp11,kp22 correspondent aux elements k11* et k22* de K*
*     xx et yy contiennent les milieux des elements (nxh*nhr,nyh*nyhr)
*     
      integer nxh,nyh,nxhf,nyhf,nxhr,nyhr
      integer nmat,nmorse,nvalx,nvaly,nvalfx,nvalfy,ntot,ntotf,
     *     ntotv,nvar,nvarf,nfxr,nfyr,nfx,nfy
      real diff,difm,ghx,ghy,p11,p22,kp11,kp22
      real x(nv),y(nv),xx(nv+2),yy(nv+2)
      real vf1,vf2,kvf1,kvf2
      real dx(nv),dy(nv),dxy(nv),dyx(nv)
      real v1(nv),v2(nv)
      real zx,zy
      real zcx,zcxy,zcyx,zcy
      character*200 buf
      integer i
      
      common/morse/nmat,nmorse
      common/morse1/nvalx,nvaly,nvalfx,nvalfy
      common/morse2/ntot,ntotf,ntotv,nvar,nvarf
      common/morse3/nfxr,nfyr
      
c     Lecture des donnees dans le fichier de donnees
      open (10,FILE='dispInput.dat',STATUS='old')
      read (10,*) buf,buf,nxh,buf,nyh,buf,nxhf,buf,nyhf,buf,nxhr,buf,
     *  nyhr,buf,p11,buf,kp11,buf,diff,buf,difm,buf,ghx,buf,ghy
      close (10)  
      
      p22=p11
      kp22=kp11
      
*     cas si les diffusions dans la matrice et la fracture sont identiques
      if (diff .eq. difm) then
         difm=difm+1e-24
      end if   
      
c     Calcul des milieux des cellules
      CALL mil_cel(nxh,nyh,nxhr,nyhr,x,y,xx,yy)
      
c     calcul de termes intervenant dans le calcul du tenseur D*
      nvalx=nxh*nxhr
      nvaly = nyh * nyhr
      nvalfx = nxhf * nxhr
      nvalfy = nyhf * nyhr
      ntot = nxh * nyh
      ntotf = nxhf * nyhf
      ntotv = ntot - ntotf
      nvar = nvalx * nvaly
      nvarf = nvalfx * nvalfy
      nfx = 0.5 * ( nxh - nxhf )
      nfy = 0.5 * ( nyh - nyhf )
      nfxr = nfx * nxhr
      nfyr = nfy * nyhr
      
*     nombre d inconnues
      nmat = nvar
*     taille de la matrice morse
      nmorse=nmat*9
      
c     Remplissage des vecteurs dx,dy,dxy,dyx 
      CALL remplissage_d(diff,difm,nxh,nyh,nxhf,nyhf,nfx,nfy,dx,dy,
     *     dxy,dyx)
      
*     calcul de la vitesse
      vf1=-p11*ghx
      vf2=-p22*ghy

      
      write(*,*)'vf1',vf1,'\t','\tvf2',vf2      
      
*     calcul de la vitesse macro
      kvf1=-ghx*kp11
      kvf2=-ghy*kp22

      
c      write(*,*)'<vf1>',kvf1,'\t','\t<vf2>',kvf2
c      pause
      
*     remplissage des vecteurs pour la vitesse      
      call remplissage_v(vf1,vf2,nxh,nyh,
     *     nfx,nfy,v1,v2)       

c     appel de la procedure pour le calcul de D**
      CALL homogab2_disp(nv,nm,v1,v2,kvf1,kvf2,dx,dxy,dyx,dy,x,y,xx,
     *     yy,nxh,nyh,nxhf,nyhf,nxhr,nyhr,zcx,zcxy,zcyx,zcy,zx,zy,0)
      
      
*     affichage a l ecran du tenseur homogeneise D**
      write(*,*)'*****************************************************'
     *     ,'*******'
      write(*,*)'\t \t TENSEUR HOMOGENEISE D**'
      write(*,*)
      write(*,*)zcx,'\t',zcxy
      write(*,*)zcyx,'\t',zcy
      write(*,*)
      write(*,*)'**************************************************'
     *     ,'**********'

      open (20, FILE='dispOutput.dat')
      write(20, *) '1'
      write(20, *) '1'
      write(20, *) '1'
      write(20, *) zcx
      write(20, *) zcxy
      write(20, *) '0.0'
      write(20, *) zcyx
      write(20, *) zcy
      write(20, *) '0.0'
      write(20, *) '0.0'
      write(20, *) '0.0'
      write(20, *) '1.0'
      close (20)

c      pause 'FIN DU PROGRAMME'
      
      end
c     fin du programme principal
      
      

********************************************************************************
c     Calcul des milieux des sous cellules
      
      subroutine mil_cel(nxh,nyh,nxhr,nyhr,x,y,xx,yy)

* nxh : nombre de sous cellules selon x
* nyh : nombre de sous cellules selon y
* nxhr : raffinement selon x
* nyhr : raffinement selon y
*   x : vecteur contenant les abscisses des milieux des sous cellules
*   y : vecteur contenant les ordonnees des milieux des sous cellules
*  xx : vecteur contenant les abscisses des milieux des elements
*  yy : vecteur contenant les ordonnees des milieux des elements
      
      implicit none
      
      real x(*),y(*),xx(*),yy(*)
      integer i,nxh,nyh,nxhr,nyhr
      
      do 1 i=1,nxh
         x(i)=(2*FLOAT(i)-1)/(2*nxh)
 1    continue
      
      do 2 i=1,nyh
         y(i)=(2*FLOAT(i)-1)/(2*nyh)
 2    continue
      
      xx(1)=0.0
      yy(1)=0.0
      do 3 i=2,(nxh+1)*nxhr
         xx(i)=(FLOAT(i-1))/(nxh*nxhr)
 3    continue
      
      do 4 i=2,(nyh+1)*nyhr
         yy(i)=(FLOAT(i-1))/(nyh*nyhr)
 4    continue
      
      end
      
*********************************************************************************
c     Remplissage des vecteurs dx,dy,dxy,dyx 
      
      subroutine remplissage_d(diff,difm,nxh,nyh,nxhf,nyhf,
     *     nfx,nfy,dx,dy,dxy,dyx)

* diff : diffusion dans la fracture
* difm : diffusion dans la matrice
*  nxh : nombre de sous cellules selon x
*  nyh : nombre de sous cellules selon y
* nxhf : nombre de sous cellules constituant la matrice selon x
* nyhf : nombre de sous cellules constituant la matrice selon y
*  nfx : taille de la fracture selon x (ie nombre de sous cellules entre la frontiere exterieure de la cellule et la frontiere de la fracture et la matrice) 
*  nfy : taille de la fracture selon y (ie nombre de sous cellules entre la frontiere exterieure de la cellule et la frontiere de la fracture et la matrice) 
*   dx : vecteur contenant l'element 1,1 du tenseur de diffusion correspondant a chaque sous cellules
*  dxy : vecteur contenant l'element 1,2 du tenseur de diffusion correspondant a chaque sous cellules
*  dyx : vecteur contenant l'element 2,1 du tenseur de diffusion correspondant a chaque sous cellules   
*   dy : vecteur contenant l'element 2,2 du tenseur de diffusion correspondant a chaque sous cellules   
      
      implicit none
      
      real dx(*),dy(*),dxy(*),dyx(*)
      real diff,difm
      integer i,nxh,nyh,nxhf,nyhf,j,nfx,nfy
      
      j=1
      
      do 1 i=1,nxh*nyh
         if (((i .ge. 1) .and. (i .le. nfy*nxh+nfx)) .or. 
     *        ((i . ge. (nyh-nfy)*nxh) 
     *        .and. (i .le. nxh*nyh))) then
            dx(i)=diff
            dy(i)=dx(i)
            
         else
            if ((j .le. nfx) .or. (j .ge. nxh-nfx+1)) then
               dx(i)=diff
               dy(i)=dx(i)
               
            else
               dx(i)=difm
               dy(i)=dx(i)
               
            end if        
         end if 
         
         dxy(i)=0.
         dyx(i)=0.
         
         if (mod(i,nxh) .eq. 0) then
            j=1
         else
            j=j+1
         end if
 1    continue
      
      end
      
***********************************************************************************
      
      SUBROUTINE remplissage_v(vf1,vf2,nxh,nyh,
     *     nfx,nfy,v1,v2)
      
      implicit none

*  vf1 : premiere coordonnees du vecteur vitesse
*  vf2 : seconde coordonnees du vecteur vitesse
*  nxh : nombre de sous cellules selon y
*  nyh : nombre de sous cellules selon y
*  nfx : taille de la fracture selon x (ie nombre de sous cellules entre la frontiere exterieure de la cellule et la frontiere de la fracture et la matrice) 
*  nfy : taille de la fracture selon y (ie nombre de sous cellules entre la frontiere exterieure de la cellule et la frontiere de la fracture et la matrice) 
*   v1 : vecteur contenant la premiere coordonne du vecteur vitesse pour chaque sous cellule
*   v2 : vecteur contenant la seconde coordonne du vecteur vitesse pour chaque sous cellule
      
      real vf1,vf2,v1(*),v2(*)
      integer j,i,nfx,nfy,nxh,nyh
      
      j=1
      
      do 1 i=1,nxh*nyh
         if (((i .ge. 1) .and. (i .le. nfy*nxh+nfx)) .or. 
     *        ((i . ge. (nyh-nfy)*nxh) 
     *        .and. (i .le. nxh*nyh))) then
            v1(i)=vf1
            v2(i)=vf2
         else
            if ((j .le. nfx) .or. (j .ge. nxh-nfx+1)) then
               v1(i)=vf1
               v2(i)=vf2             
            else
               v1(i)=0.
               v2(i)=0.             
            end if        
         end if 
         
         if (mod(i,nxh) .eq. 0) then
            j=1
         else
            j=j+1
         end if
 1    continue
      
      END





***** approx.f
* 
*     subroutine qui calcul l'approximation d itegrale
*
*



      real function approx(x1,x2,x3,x4)

      implicit none
      real x1,x2,x3,x4

      approx=x1+x2+x3+x4

      return

      end
      
      
***************************************************************************
** deriveesfbase.f


*     fichier contenant les derivees des fonctions de base sur K^ 
*     selon x et y
*
*              3----------4
*              |          |
*              |          |
*              |    K^    |
*              |          |
*              |          |
*              1----------2
*
*
*

*     dfb1dx   derivee par rapport a x de la fonction de base 
*               associee au sommet 1
*     dfb2dx   derivee par rapport a x de la fonction de base 
*               associee au sommet 2
*     dfb3dx   derivee par rapport a x de la fonction de base 
*               associee au sommet 3
*     dfb4dx   derivee par rapport a x de la fonction de base 
*               associee au sommet 4
*     dfb1dy   derivee par rapport a y de la fonction de base 
*               associee au sommet 1
*     dfb2dy   derivee par rapport a y de la fonction de base 
*               associee au sommet 2
*     dfb3dy   derivee par rapport a y de la fonction de base 
*               associee au sommet 3
*     dfb4dy   derivee par rapport a y de la fonction de base 
*               associee au sommet 4



      real function dfb1dx(x,y)

      implicit none
      real x,y

      dfb1dx=-(1-y)/4

      return

      end

***************************************************************************

      real function dfb2dx(x,y)

      implicit none
      real x,y

      dfb2dx=(1-y)/4

      return

      end

***************************************************************************

      real function dfb3dx(x,y)

      implicit none
      real x,y

      dfb3dx=-(1+y)/4

      return

      end

***************************************************************************

      real function dfb4dx(x,y)

      implicit none
      real x,y

      dfb4dx=(1+y)/4

      return

      end

***************************************************************************
      real function dfb1dy(x,y)

      implicit none
      real x,y

      dfb1dy=-(1-x)/4

      return

      end

***************************************************************************

      real function dfb2dy(x,y)

      implicit none
      real x,y

      dfb2dy=-(1+x)/4

      return

      end

***************************************************************************

      real function dfb3dy(x,y)

      implicit none
      real x,y

      dfb3dy=(1-x)/4

      return

      end

***************************************************************************

      real function dfb4dy(x,y)


      implicit none
      real x,y

      dfb4dy=(1+x)/4

      return

      end
***************************************************************************
** deriveesfbase.f


*     fichier contenant les derivees des fonctions de base sur K^ 
*     selon x et y
*
*              3----------4
*              |          |
*              |          |
*              |    K^    |
*              |          |
*              |          |
*              1----------2
*
*
*

*     dft1dx   derivee par rapport a x de la fonction test 
*               associee au sommet 1
*     dft2dx   derivee par rapport a x de la fonction test 
*               associee au sommet 2
*     dft3dx   derivee par rapport a x de la fonction test 
*               associee au sommet 3
*     dft4dx   derivee par rapport a x de la fonction test 
*               associee au sommet 4
*     dft1dy   derivee par rapport a y de la fonction test 
*               associee au sommet 1
*     dft2dy   derivee par rapport a y de la fonction test 
*               associee au sommet 2
*     dft3dy   derivee par rapport a y de la fonction test 
*               associee au sommet 3
*     dft4dy   derivee par rapport a y de la fonction test 
*               associee au sommet 4



      real function dft1dx(x,y,a,b)

      implicit none
      real x,y,a,b

      dft1dx=(-1./2-a*3./2*x)*((1-y)/2+b*3./4*(1-y*y))

      return

      end

***************************************************************************

      real function dft2dx(x,y,a,b)

      implicit none
      real x,y,a,b

      dft2dx=(1./2-a*3./2*x)*((1-y)/2+b*3./4*(1-y*y))

      return

      end

***************************************************************************

      real function dft3dx(x,y,a,b)

      implicit none
      real x,y,a,b

      dft3dx=(-1./2-a*3./2*x)*((1+y)/2+b*3./4*(1-y*y))

      return

      end

***************************************************************************

      real function dft4dx(x,y,a,b)

      implicit none
      real x,y,a,b

      dft4dx=(1./2-a*3./2*x)*((1+y)/2+b*3./4*(1-y*y))

      return

      end

***************************************************************************
      real function dft1dy(x,y,a,b)

      implicit none
      real x,y,a,b

      dft1dy=(-1./2-b*3./2*y)*((1-x)/2+a*3./4*(1-x*x))

      return

      end

***************************************************************************

      real function dft2dy(x,y,a,b)

      implicit none
      real x,y,a,b

      dft2dy=(-1./2-b*3./2*y)*((1+x)/2+a*3./4*(1-x*x))

      return

      end

***************************************************************************

      real function dft3dy(x,y,a,b)

      implicit none
      real x,y,a,b

      dft3dy=(1./2-b*3./2*y)*((1-x)/2+a*3./4*(1-x*x))
 
      return

      end

***************************************************************************

      real function dft4dy(x,y,a,b)

      implicit none
      real x,y,a,b

      dft4dy=(1./2-b*3./2*y)*((1+x)/2+a*3./4*(1-x*x))

      return

      end
***************************************************************************
** fonctionsbase.f


*     fichier contenant les fonctions de base sur K^
*
*              3----------4
*              |          |
*              |          |
*              |    K^    |
*              |          |
*              |          |
*              1----------2
*
*
*

*     fb1   fonction de base pour le sommet 1
*     fb2   fonction de base pour le sommet 2
*     fb3   fonction de base pour le sommet 3
*     fb4   fonction de base pour le sommet 4



      real function fb1(x,y)

      implicit none

      real x,y
      fb1=(1-x)*(1-y)/4

      return

      end

***************************************************************************

      real function fb2(x,y)

      implicit none
      real x,y

      fb2=(1+x)*(1-y)/4

      return

      end

***************************************************************************

      real function fb3(x,y)

      implicit none
      real x,y

      fb3=(1-x)*(1+y)/4

      return

      end

***************************************************************************

      real function fb4(x,y)

      implicit none

      real x,y
      fb4=(1+x)*(1+y)/4

      return

      end
***************************************************************************
** fonctionstests.f


*     fichier contenant les fonctions de base sur K^
*
*              3----------4
*              |          |
*              |          |
*              |    K^    |
*              |          |
*              |          |
*              1----------2
*
*
*

*     ft1   fonction test pour le sommet 1
*     ft2   fonction test pour le sommet 2
*     ft3   fonction test pour le sommet 3
*     ft4   fonction test pour le sommet 4



      real function ft1(x,y,a,b)

      implicit none
      real x,y,a,b

      ft1=((1-x)/2+a*3./4*(1-x*x))*((1-y)/2+b*3./4*(1-y*y))

      return

      end

***************************************************************************

      real function ft2(x,y,a,b)

      implicit none
      real x,y,a,b

      ft2=((1+x)/2+a*3./4*(1-x*x))*((1-y)/2+b*3./4*(1-y*y))

      return

      end

***************************************************************************

      real function ft3(x,y,a,b)

      implicit none
      real x,y,a,b

      ft3=((1-x)/2+a*3./4*(1-x*x))*((1+y)/2+b*3./4*(1-y*y))

      return

      end

***************************************************************************

      real function ft4(x,y,a,b)

      implicit none
      real x,y,a,b

      ft4=((1+x)/2+a*3./4*(1-x*x))*((1+y)/2+b*3./4*(1-y*y))

      return

      end
C Fichier gmres.f
C
C---------------------------------------------
      subroutine gmres(ns,a,mia,mja,x,eps,nkr,nit,b)
C---------------------------------------------
C Ce sous-programme resout le systeme lineaire $Ax=b$
C par la methode GMRES(nkr), i.e. avec une dimension nk 
C de l"espace de Krylov limitee a nkr, ce qui necessite
C plusieurs reinitialisations successives, dont on limite
C arbitrairement le nombre a nit. Le test d"arret de la 
C methode porte sur la norme euclidienne du residu.
C
C Cette methode est basee sur la construction, selon la 
C methode d"Arnoldi, d"une base orthonormee de l"espace
C de Krylov, base qui est stockee dans le tableau v; plus 
C precisement, v(i,j) contient la i eme composante du  
C j eme vecteur de base. 
C
C Simultanement, est construit le tableau h, qui,
C au depart, est defini par h(i,j)=$(A v_i,v_j)$;
C la j eme colonne de ce tableau h represente alors
C la j eme colonne de la matrice de Hessenberg $\bar H^{nk}$,
C matrice qui est d"ordre $(nk+1)\times nk$ et qui est
C ensuite factorisee sous forme $Q^{nk} R^{nk}$, avec $Q^{nk}$ 
C matrice orthogonale d"ordre $(nk+1)\times (nk+1)$ et
C $R^{nk}$ matrice triangulaire superieure d"ordre 
C $(nk+1)\times nk$, de derniere ligne nulle, et dont tous
C les elements diagonaux sont strictement positifs.
C
C Dans un deuxieme temps, h(.,j) contient la colonne 
C d"indice j de la matrice triangulaire superieure $R^{nk}$,
C apres avoir effectue la j eme rotation (dont le cosinus
C de l"angle est note cos, le sinus etant 
C note sin), rotation dont le but est d"annuler le 
C coefficient d"indice $(j+1)j$ dans la matrice 
C de Hessenberg $\bar H^{kr}$.
C
C La tableau r contient le second membre 
C $g^{nk}=(Q^{nk})^T ||r^0||  e_1^{nk+1}$ du probleme 
C de minimisation d"inconnue $y^{nk}$ a resoudre.  
C Ensuite, ce tableau r contient la solution $y^{nk}$ qui
C est calculee par resolution d"un systeme triangulaire 
C superieur, de matrice $R^{nk}$. Cette operation se fait
C dans le sous-programme soln.
C
C La solution approchee a l"etape nk de l"algorithme est
C stockee dans le tableau x, tableau qui contient au depart
C l"initialisation $x^0$ de la solution. Cette operation se
C fait egalement dans le sous-programme soln.
C
C Nous calculons ensuite le "vecteur residu" qui nous est 
C necessaire pour l"initialisation de l"etape it suivante; 
C simultanement sa norme nous sert de test d"arret.
C
C Remarque 1:
C -----------
C Nous avons ici choisi de construire d"abord la base
C de l"espace de Krylov puis de factoriser  dans un
C deuxieme temps la matrice de Hessenberg.
C
C Remarque 2:
C -----------
C La matrice $A$ de taille ns*ns, ainsi que la second membre 
C $b$ de taille ns du systeme lineaire a resoudre,
C n"intervenant que dans le calcul du residu initial,
C puis dans la construction des vecteurs de base, nous ne
C les avons pas stockes dans ce sous programme.
C Ils ne figurent donc que dans un sous-programme, que nous
C avons note "pmat(ns,x,ax,ind)": pour un vecteur $x(ns)$
C donne (stocke dans le tableau x(ns)), et pour un 
C indicateur fixe ind, le sous programme pmat calcule et 
C stocke dans le tableau ax:
C - le produit $A * x$, si ind vaut 0,
C - la combinaison lineaire $b-A*x$, si ind vaut 1.
C Ce sous-programme dependant du type de stockage de la 
C matrice $A$ envisage, nous ne le donnons pas ici. Il est a
C creer par l"utilisateur en fonction de ses besoins.
C 
C 
C          Programme ecrit par Pascal Joly
C          (Laboratoire d"Analyse Numerique,
C          Universite Pierre et Marie Curie),
C          modifie par Brigitte Lucquin (Juin 95).
C 
C 

      implicit none

      integer ns,nkr
      real  x(ns+2),v(ns+2,nkr+2),a(*),
     >           h(nkr+2,nkr+2),r(nkr+2)
      real aux(ns+2),aux1(ns+2)
      integer ind,nit,i,j,it,nk,il,ip
      real eps,prods
      integer mia(*),mja(*)
      real b(*)
      real res,rec,zero,tem,sin,cos,hij,hipj,raux,gamma,dem,hii,hipi

      DATA zero/1.e-8/
*      rec    = zero  
      rec=eps       
*      write(*,*)ns
*      pause

C
C...initialisation
C
      ind=1

      do 100 i=1,ns
         aux(i)=0.0
 100  continue

      CALL pmat(ns,x,aux,ind,a,mia,mja,b)
*      write(*,*)'DANS GMRES'
*      write(*,*)'AUX APRES PMAT'
*      write(*,*)(aux(i),i=1,ns)
      res= prods(ns,aux,aux)
      res= sqrt(res)

      IF(res .LT. rec) then
         write(*,*)'RES<REC'
         GOTO 4
      end if
*      rec = max(rec,res*eps)
      write(*,*)'res=',res
      write(*,*)'res*eps=',res*eps 
           
C
C...demarrage des iterations
C      
      write(*,*)nit                 
      DO 1 it=1,nit

        nk=nkr
C                                                                     
C...orthogonalisation d'Arnoldi
C
        DO 10 i=1,ns
          aux(i) = aux(i)/res
          r(i)=0.
10      CONTINUE
        r(1)   = res

        DO 11 j=1,nkr
          DO 21 i=1,ns                                                     
            v(i,j) = aux(i)                                              
21        CONTINUE

          ind=0
                           
          CALL pmat(ns,aux,aux1,ind,a,mia,mja,b)  

          DO 22 i=1,ns
            aux(i)=aux1(i)
22        CONTINUE  
          
          DO 23 i=1,j
            tem= prods(ns,aux1,v(1,i))
            h(i,j) = tem
            DO 24 il=1,ns
              aux(il)=aux(il)-tem*v(il,i)
24          CONTINUE
23        CONTINUE
          
          dem= prods(ns,aux,aux)
          dem=sqrt(dem)
          h(j+1,j) = dem
          IF (dem .LT. rec) THEN
            nk=j
            GOTO 5
          ENDIF
          DO 25 i=1,ns
            aux(i) = aux(i)/dem
25          CONTINUE
11      CONTINUE

C
C...triangularisation et modif. du second membre
C
5       CONTINUE
        DO 31 i=1,nk
          ip=i+1
          hii=h(i,i)
          hipi=h(ip,i)
          gamma=hii*hii+hipi*hipi
          gamma=sqrt(gamma)
          gamma=1./gamma
          cos=hii*gamma
          sin=-hipi*gamma
          DO 41 j=i,nk
            hij=h(i,j)
            hipj=h(ip,j)
            h(i,j) = cos*hij - sin*hipj
            h(ip,j) = sin*hij + cos*hipj
41        CONTINUE
          raux=r(i)
          r(i)=cos*raux
          r(ip)=sin*raux
31      CONTINUE

C
C...resolution du systeme triangulaire superieur
C
        CALL soln(nk,ns,r,h,v,x)

c
C...calcul du residu
C
        ind=1

        CALL pmat(ns,x,aux,ind,a,mia,mja,b)
        res= prods(ns,aux,aux)
        res= sqrt(res)
        if ((mod(it,100) .eq. 0) .or. (mod(it,nit) .eq. 0 )) then
           write(*,*)res
        end if
        IF(res .LT. rec) THEN
          GOTO 4
        ELSEIF(it.GE.nit) THEN
          write(*,*)'nbre d iterations =',it
          write(*,*)'eps=',rec
          write(*,*)'res=',res
          write(*,*) '*********  NON CONVERGENCE *********'
c          pause 'FIN DE GMRES'
          GOTO 6
        ENDIF
1     CONTINUE
C
C...sortie des resultats
C
4     CONTINUE
      write(*,*) 'RES<REC'
      write(*,*)'nbre d iterations =',it
      write(*,*)'eps=',rec
      write(*,*)'res=',res
c      pause 'FIN DE GMRES'
      write(*,*)

6     CONTINUE
C
C
      END
C
C
C**********************************
C
C     BIBLIOTHEQUE
C
C**********************************
C
C

C
C---------------------------------------------
      SUBROUTINE soln(nk,ns,r,h,v,x)
C---------------------------------------------
C Ce sous-programme resout le systeme triangulaire 
C superieur permettant de calculer la solution 
C du probleme de minimisation dans l"algorithme 
C GMRES. La solution approchee a l"etape nk est 
C calculee et stokee dans le vecteur x.
C
C   
*      PARAMETER (nsMax=4000,nkrMax=5000)
      
      implicit none

      integer ns,nk
*      real  x(nsmax),v(nsmax,nkrmax),
*     >           h(nkrmax,nkrmax),r(nkrmax)
      real  x(*),v(ns+2,nk+2),
     >           h(nk+2,nk+2),r(*)
      integer i,j,il
      real tem
C
*      write(*,*)'DANS SOLN, nk=',nk
*      write(*,*)'H'
*      write(*,*)(h(i,i),i=1,nk)
*      pause
      DO 1 i=nk,1,-1                                                
        tem = r(i)/h(i,i)
        r(i)=tem                                                    
        DO 2 j=i-1,1,-1
          r(j)=r(j)-h(j,i)*tem                                                    
2       CONTINUE
1     CONTINUE
      
*      write(*,*)(r(i),i=1,nk)
*      pause
C     
      DO 3 i=1,nk
        tem = r(i)
        DO 4 il=1,ns
          x(il)=x(il)+tem*v(il,i)
4       CONTINUE
3     CONTINUE
      
*      write(*,*)(x(i),i=1,nk)
*      pause
      END
C
C--------------------------------------
      real FUNCTION   prods(n,x,y)                                           
C--------------------------------------
C Cette fonction calcule le produit scalaire de 2 vecteurs
C x et y de taille n.
C
C
      implicit none
      integer n,i
      real x(n+2),y(n+2) 
      real som

      som = 0.                                                    
      DO 1 i=1,n                                                      
1       som = som + x(i)*y(i)                                         
      prods = som                                                     
      END
*** fichier pmat.f

*** procedure qui permet la multiplication d'une matrice (stockee sous forme morse) avec un vecteur : A*x= ax


      subroutine pmat(n,x,ax,ind,a,ia,ja,b)
*  n : dimension du systeme
*  x : vecteur multiplicateur
*  a : vecteur contenant les elements de la matrice A
* ia : vecteur contenant la position dans a des premiers elements non nuls de chaque ligne
* ja : vecteur contenat les colonnes de chaque element de a
*  b : vecteur second membre du systeme A*x = b
*ind : prend 0 ou 1 ; on calcule A*x si ind = 0 et b - A*x si ind = 1
* ax : vecteur contenant le resultat de la multiplication
       
      implicit none
      
      real ax(*),b(*),a(*),x(*)
      integer ind,ia(*),ja(*),n
      integer i,k
      real som

      do 2 i=1,n,1
         som=0.0
         do 3 k=ia(i),ia(i+1)-1
            som=som+a(k)*x(ja(k))
 3       continue
         ax(i)=som
 2    continue   

      if (ind .eq. 1) then  
         do 4 i=1,n
 4          ax(i)=b(i)-ax(i)
       end if
       
       return
       end
      
*** fichier remplissage_maa.f

      subroutine remplissage_maa (i,j,ia,ja,bool,empl,prem)
      
      implicit none

*  i : indice de la ligne pour la matrice de discretisation
*  j : indice de la colonne pour la matrice de discretisation
* ia : vecteur contenant la position dans a des premiers elements non nuls de chaque ligne
* ja : vecteur contenat les colonnes de chaque element de a
* bool : booleen qui prend 1 si l'element A(i,j) a deja ete implemente et 0 sinon
* empl : donne la place de l'element dans ja (et donc dans aa qui contient les elements non nuls de la matrice de discretisation) QUAND bool = 1
* prem : donne le premier emplacement vide dans le vecteur ja QUAND bool = 0


      integer ia(*),ja(*)
      integer i,j,empl,bool,prem,k

      bool=0
      prem=0
      k=ia(i) 

*     boucle tant que
   1  if ((bool .eq. 0) .and. (k .le. (ia(i+1)-2))) then

*       si element a deja implemente
	if (ja(k) .eq. j) then
           bool=1
	   k=k-1
        end if

*	si element non encore implemente : determination du premier emplacement libre
        if ((bool .eq. 0) .and. (ja(k).eq. 0) .and.
     *       ((prem .gt. k) .or.(prem .eq. 0))) then
           prem=k
        end if

        k=k+1
        go to 1
      endif  
*     fin boucle tant que

      empl=k

      return
      end
      







**************************************************************
********                     tri.f                      ******

*
* Ce sous-programme est ecrit sous forme tres compacte
* ce qui le rend difficile a la comprehension.
*
* Il s'agit de ranger les elements du tableau criter
* par ordre croissant d"indices et de modifier 
* simultanement les elements du tableau record.
*
*
*	de 'INTRODUCTION AU CALCUL SCIENTIFIQUE' 
*	B. LUCQUIN
*	O. PIRONNEAU
*	MASSON, 1996


      subroutine Tri(criter,record,n)
      
      implicit none
 
* n : nombre d'elements a ranger
* criter : vecteur contenant les indices des colonnes des elements contenus dans record
* record : vecteur contenant les elements de la matrice de discretisation
     
      integer n
      integer crit,criter(n)
      real record(n),rec
      integer l,r,j,i
      
      if (n .le. 1) then
         return
      end if
      l=n/2+1
      r=n

 2    if (l .gt. 1) then
         l=l-1
         rec=record(l)
         crit=criter(l)
      else
         rec=record(r)
         crit=criter(r)
         record(r)=record(1)
         criter(r)=criter(1)
         r=r-1
         if (r .eq. 1) then
            record(1)=rec
            criter(1)=crit
            return
         end if
      end if

 3    j=l
 4    i=j
      j=j*2
      if (j-r .lt. 0) then
         goto 5
      elseif (j .eq.r) then
         goto 6
      else 
         goto 8
      end if
 5    if (criter(j) .lt. criter(j+1)) j=j+1
 6    if (crit .ge. criter(j)) goto 8
      record(i)=record(j)
      criter(i)=criter(j)
      goto 4
 8    record(i)=rec
      criter(i)=crit
      goto 2

      end
      











*** fichier  homogab2_disp.f
c\begin{verbatim}

      subroutine homogab2_disp(nv,nm,v1,v2,kvf1,kvf2,dx,dxy,dyx,dy,x,
     *y,xx,yy,nxh,nyh,nxhf,nyhf,nxhr,nyhr,zcx,zcxy,zcyx,zcy,zx,zy,iopt)
      
c
c * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
c
c crit par : joe koebbe 
c et modifie par : benedicte VIDAL
c
c   Cette procedure permet de calculer pour un MILIEU POREUX FRACTURE 
c le tenseur homogeneise.
c
c Le tenseur contenant les diffusions est de la forme :
c
c                 [  dx    dxy  ]
c             D = [             ]
c                 [  dyx    dy  ]
c
c  Le tenseur homogeneise est donne en sortie sous la forme de la matrice D**
c                          **   [  zcx    zcxy  ] 
c                         D  = [               ]
c                              [  zcyx   zcy   ]
c
c
c * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
c
      
      implicit none

c     declaration des variables
c-------------------------------
      integer nxh,nyh,nxhf,nyhf,nxhr,nyhr,iopt,ntot,nv,nm
      integer n1,n2,n1r,n2r
      real xvalm,xvalp,yvalm,yvalp,supval
      real dx(*),dxy(*),dyx(*),dy(*),v1(*),v2(*)   
      real x(*),y(*),xx(*),yy(*)
      real zcx,zcxy,zcyx,zcy
******************
      real zcx1,zcxy1,zcyx1,zcy1
**************************
      real rcx,rcxy,rcyx,rcy
      real zx,zy
      real cgeps,eps,constant,rv1,rv2
      real sum,cellsize,term1,term2,term3,term4
      real termm1,termm2,termm3,termm4
      real delx,dely,del
      integer n1f,n2f,nvalfx,nvalfy,ntotf,ntotv,nvar,nmat,nvarf
      integer nvalx,nvaly
      integer nfxr,nfyr,nmorse
      integer kmax,i,j2,j,ind1,ind2,ind3,ind4,indx,indy,indif

*     variables pour le stockage morse  
*     vecteur pour la matrice morse
      integer mia(nv),mja(nm)                     
      real maa(nm)
      real hrhs(nv,2),hx(nv,2)
      integer first,place,bool

*     variables vitesse et moyenne
      real v01,v02
      real kvf1,kvf2
   
*     variables pour les fonctions tests
*  peij et peik pour le peclet sur cotes ij et ik resp (cotes ind1-ind2, ind1-ind3)
*   prodscij et prodscik pour v.n sur cotes ij et ik resp   
      real pe12,pe34,pe13,pe24
      real alpha12,alpha34,alpha13,alpha24
      real prodsc12,prodsc34,prodsc13,prodsc24

*     constantes pour l approximation des integrales
      real x1,x2,y1,y2
      real a,b,ab,b2,a2,det
*     fonctions
      real ft1_1,ft2_1,ft3_1,ft4_1
      real ft1_2,ft2_2,ft3_2,ft4_2
      real ft1_3,ft2_3,ft3_3,ft4_3
      real ft1_4,ft2_4,ft3_4,ft4_4
      real fb1_1,fb2_1,fb3_1,fb4_1
      real fb1_2,fb2_2,fb3_2,fb4_2
      real fb1_3,fb2_3,fb3_3,fb4_3
      real fb1_4,fb2_4,fb3_4,fb4_4
      real dfb1dx_1,dfb2dx_1,dfb3dx_1,dfb4dx_1,dft1dx_1,
     *     dft2dx_1,dft3dx_1,dft4dx_1
      real dfb1dx_2,dfb2dx_2,dfb3dx_2,dfb4dx_2,dft1dx_2,
     *     dft2dx_2,dft3dx_2,dft4dx_2
      real dfb1dx_3,dfb2dx_3,dfb3dx_3,dfb4dx_3,dft1dx_3,
     *     dft2dx_3,dft3dx_3,dft4dx_3
      real dfb1dx_4,dfb2dx_4,dfb3dx_4,dfb4dx_4,dft1dx_4,
     *     dft2dx_4,dft3dx_4,dft4dx_4
      real dfb1dy_1,dfb2dy_1,dfb3dy_1,dfb4dy_1,dft1dy_1,
     *     dft2dy_1,dft3dy_1,dft4dy_1
      real dfb1dy_2,dfb2dy_2,dfb3dy_2,dfb4dy_2,dft1dy_2,
     *     dft2dy_2,dft3dy_2,dft4dy_2
      real dfb1dy_3,dfb2dy_3,dfb3dy_3,dfb4dy_3,dft1dy_3,
     *     dft2dy_3,dft3dy_3,dft4dy_3
      real dfb1dy_4,dfb2dy_4,dfb3dy_4,dfb4dy_4,dft1dy_4,
     *     dft2dy_4,dft3dy_4,dft4dy_4
*     approximation
      real apxb11,apxb12,apxb13,apxb14
      real apxb21_1,apxb21_2,apxb21_3,apxb21_4
      real apxb22_1,apxb22_2,apxb22_3,apxb22_4
      real apxd11,apxd12,apxd13,apxd14
      real apxd21_1,apxd21_2,apxd21_3,apxd21_4
      real apxd22_1,apxd22_2,apxd22_3,apxd22_4
      real apxm11_1,apxm11_2,apxm11_3,apxm11_4
      real apxm12_1,apxm12_2,apxm12_3,apxm12_4
      real apxm13_1,apxm13_2,apxm13_3,apxm13_4
      real apxm14_1,apxm14_2,apxm14_3,apxm14_4
      real apxm21_1,apxm21_2,apxm21_3,apxm21_4
      real apxm22_1,apxm22_2,apxm22_3,apxm22_4
      real apxm23_1,apxm23_2,apxm23_3,apxm23_4
      real apxm24_1,apxm24_2,apxm24_3,apxm24_4
      real apxa11_1,apxa11_2,apxa11_3,apxa11_4
      real apxa12_1,apxa12_2,apxa12_3,apxa12_4
      real apxa13_1,apxa13_2,apxa13_3,apxa13_4
      real apxa14_1,apxa14_2,apxa14_3,apxa14_4
      real apxa21_1,apxa21_2,apxa21_3,apxa21_4
      real apxa22_1,apxa22_2,apxa22_3,apxa22_4
      real apxa23_1,apxa23_2,apxa23_3,apxa23_4
      real apxa24_1,apxa24_2,apxa24_3,apxa24_4
      real apxa31_1,apxa31_2,apxa31_3,apxa31_4
      real apxa32_1,apxa32_2,apxa32_3,apxa32_4 
      real apxa33_1,apxa33_2,apxa33_3,apxa33_4
      real apxa34_1,apxa34_2,apxa34_3,apxa34_4 
      real apxa41_1,apxa41_2,apxa41_3,apxa41_4
      real apxa42_1,apxa42_2,apxa42_3,apxa42_4
      real apxa43_1,apxa43_2,apxa43_3,apxa43_4
      real apxa44_1,apxa44_2,apxa44_3,apxa44_4

*     pour appel des fonctions
      real ft1,ft2,ft3,ft4,fb1,fb2,fb3,fb4
      real dfb1dx,dfb2dx,dfb3dx,dfb4dx
      real dfb1dy,dfb2dy,dfb3dy,dfb4dy
      real dft1dx,dft2dx,dft3dx,dft4dx
      real dft1dy,dft2dy,dft3dy,dft4dy
      real approx


* pour gmres nkr = dimension de l espace de krylov
      integer nkr

*  frac booleen =0 si dans matrice et 1 si dans fracture      
      integer frac


      common /morse/ nmat,nmorse
      common/morse1/nvalx,nvaly,nvalfx,nvalfy
      common/morse2/ntot,ntotf,ntotv,nvar,nvarf
      common/morse3/nfxr,nfyr

      
c     
c     definition de parametres utilises dans la suite
c -----------------------------------------------------
c     
      data cgeps / 0.00000001/
      
c
c     definition du nombre d'iteration maximum
c ----------------------------------------------
c
      nkr = 2
      kmax = nmat+2
      
c     
c     si le nombre d'elements ou le raffinement selon un axe 
c     est nul le mettre a 1
c -----------------------------------------------------------
c     
      if(nxh .le. 0) nxh = 1
      if(nyh .le. 0) nyh = 1
      if(nxhr .le. 0) nxhr = 1
      if(nyhr .le. 0) nyhr = 1
c     
c     posons des variables pour la suite
c --------------------------------------
c     
      n1 = nxh
      n2 = nyh
      n1r = nxhr
      n2r = nyhr
      n1f = nxhf
      n2f = nyhf

c
c     calcul de la taille de la cellule de base 
c     pour une utilisation ulterieure
c -----------------------------------------------
c     
      if(nxh .eq. 1) then
         xvalm = 0.0
         xvalp = 1.0
      else
         xvalm = 1.5 * x(1) - 0.5 * x(2)
         xvalp = 1.5 * x(nxh) - 0.5 * x(nxh-1)
      end if
      if(nyh .eq. 1) then
         yvalm = 0.0
         yvalp = 1.0
      else
         yvalm = 1.5 * y(1) - 0.5 * y(2)
         yvalp = 1.5 * y(nyh) - 0.5 * y(nyh-1)
      end if
      cellsize = ( xvalp - xvalm ) * ( yvalp - yvalm )
      
c     
c     calcul des coordonnees du nouveau centre de la cellule
c -----------------------------------------------------------
c
      zx = 0.5 * ( xvalp + xvalm )
      zy = 0.5 * ( yvalp + yvalm )
      
c     
c     recherche du maximum dans les vecteurs
c -------------------------------------------
c
      supval = 0.0
      do 1 i=1,ntotv
         if(abs(dx(i)) .gt. supval)  supval = dx(i)
         if(abs(dxy(i)) .gt. supval) supval = dxy(i)
         if(abs(dyx(i)) .gt. supval) supval = dyx(i)
         if(abs(dy(i)) .gt. supval)  supval = dy(i)
    1 continue

c
c     verification des valeurs nulles sur la diagonale en entree
c ---------------------------------------------------------------
c
      constant = 0.000001 * supval
      do 2 i=1,ntotv
c     
c     somme en valeur absolue des entrees
c -------------------------------------------
c     
         sum = abs(dx(i))  + abs(dxy(i))
     1        + abs(dyx(i)) + abs(dy(i))
c     
c     verification de la somme
c --------------------------------
c
         if(sum .lt. constant) then
            dx(i) = constant
            dy(i) = constant
         end if
         
 2    continue
      
c
c     initialisation de la matrice et du second membre
c ----------------------------------------------------
c
      do 3 i=1,nmat
         hrhs(i,1) = 0.0
         hrhs(i,2) = 0.0
         maa(i)=0.0
         mja(i)=0
         mia(i)=0  
 3    continue
      
      do 31 i=nmat+1,nmorse
         maa(i)=0.0
         mja(i)=0
 31   continue
      mia(nmat+1)=nmorse+1
      
c
c     calcul du vecteur MIA pour ZONE 1
c---------------------------------------
c
      mia(1)=1
      do 32 i=2,nmat
         mia(i)=9+mia(i-1)
 32   continue
      
c
c     boucles sur les noeuds
c ---------------------------
c   
      do 4 j=1,nvaly
         j2 = ( j - 1 ) * nvalx
c
c     recherche de la seconde coordonnee dans la grille de permeabilite
c -------------------------------------------------------------------------
c
         indy = ( j + nyhr - 1 ) / nyhr
c
c     calcul de la taille d'un element selon y
c ------------------------------------------------
c
         if(nyh .eq. 1) then
            yvalm = 0.0
            yvalp = 1.0
         else
            if(indy .eq. 1) then
               yvalm = 1.5 * y(1) - 0.5 * y(2)
               yvalp = 0.5 * ( y(indy) + y(indy+1) )
            end if
            if(indy .gt. 1 .and. indy .lt. nyh) then
               yvalm = 0.5 * ( y(indy-1) + y(indy) )
               yvalp = 0.5 * ( y(indy) + y(indy+1) )
            end if
            if(indy .eq. nyh) then
               yvalm = 0.5 * ( y(indy-1) + y(indy) )
               yvalp = 1.5 * y(indy) - 0.5 * y(indy-1)
            end if
         end if
         dely = ( yvalp - yvalm ) / float(nyhr)
         
         do 4 i=1,nvalx
c
c     recherche de la premiere coordonnee dans la grille de permeabilite
c --------------------------------------------------------------------------
c     
            indx = ( i + nxhr - 1 ) / nxhr
c
c     calcul de la taille d'un element selon x
c ------------------------------------------------
c     
            if (nxh .eq. 1) then
               xvalm = 0.0
               xvalp = 1.0
            else
               if(indx .eq. 1) then
                  xvalm = 1.5 * x(1) - 0.5 * x(2)
                  xvalp = 0.5 * ( x(indx) + x(indx+1) )
               end if
               if(indx .gt. 1 .and. indx .lt. nxh) then
                  xvalm = 0.5 * ( x(indx-1) + x(indx) )
                  xvalp = 0.5 * ( x(indx) + x(indx+1) )
               end if
               if(indx .eq. nxh) then
                  xvalm = 0.5 * ( x(indx-1) + x(indx) )
                  xvalp = 1.5 * x(indx) - 0.5 * x(indx-1)
               end if
            end if
            delx = ( xvalp - xvalm ) / float(nxhr)

*     constantes
            a=delx/2
            b=dely/2
            ab=delx*dely/4
            a2=a*a
            b2=b*b
            det=1./ab
c     
c     attribution de la diffusion
c -------------------------------------
c
            indif = indx + ( indy - 1 ) * nxh
            rcx = dx(indif)
            rcxy = dxy(indif)
            rcyx = dyx(indif)
            rcy = dy(indif)
            v01=v1(indif)
            v02=v2(indif)

c
c     calcul de la contribution de la diagonale pour l'element
c
c     calcul des indices des 4 sommets de l'element 
c     en utilisant la propriete de la periodicite
c --------------------------------------------------------------
c

            ind1 = i + j2
            if(i .eq. nvalx) then
               ind2 = 1 + j2
            else
               ind2 = ind1 + 1
            end if
            if( j .eq. nvaly) then
               ind3 = i
            else
               ind3 = ind1 + nvalx
            end if
            if(j .eq. nvaly) then
               if(i .eq. nvalx) then
                  ind4 = 1
               else
                  ind4 = i + 1
               end if
            else
               ind4 = ind2 + nvalx
            end if
            
*     si dans la matrice 
            
            if (((j .ge. nfyr+1) .and. (j .le. nvaly-nfyr)) 
     *           .and. ((i .ge. nfxr+1) .and. (i .le. nvalx-nfxr))) then        
               frac=0
            else
               frac=1
            end if
*********************************************************************************************
**********************************************************************************
************************************************************************           
c
c     calcul des alphaij et ik pour les fonctions tests
c     
*     produits scalaires selon les abscisses
            prodsc12 = - v02
            prodsc34 =   v02
*     produits scalaires selon les ordonnees
            prodsc13 = - v01
            prodsc24 =   v01
            
            pe12=delx*(prodsc12/rcx)
            pe34=delx*(prodsc34/rcx)
            pe13=dely*(prodsc13/rcx)
            pe24=dely*(prodsc24/rcx)
            
            if (prodsc12 .eq. 0.0) then
               alpha12=0.0
            else
               alpha12=1./tanh(pe12/2)-2/pe12
               alpha12=sign(alpha12,-prodsc12)
            end if
            
            if (prodsc34 .eq. 0.0) then
               alpha34=0.0
            else
               alpha34=1./tanh(pe34/2)-2/pe34
               alpha34=sign(alpha34,-prodsc34)
            end if
            
            if (prodsc13 .eq. 0.0) then
               alpha13=0.0
            else
               alpha13=1./tanh(pe13/2)-2/pe13
               alpha13=sign(alpha13,-prodsc13)
            end if
            
            if (prodsc24 .eq. 0.0) then
               alpha24=0.0
            else
               alpha24=1./tanh(pe24/2)-2/pe24
               alpha24=sign(alpha24,-prodsc24)
            end if

*     points pour l approximation
            x1=-1./sqrt(3.)
            y1=-1./sqrt(3.)
            x2=1./sqrt(3.)
            y2=1./sqrt(3.)
            ft1_1=ft1(x1,y1,alpha12,alpha13)
            ft1_2=ft1(x1,y2,alpha12,alpha13)
            ft1_3=ft1(x2,y1,alpha12,alpha13)
            ft1_4=ft1(x2,y2,alpha12,alpha13)

            ft2_1=ft2(x1,y1,alpha12,alpha24)
            ft2_2=ft2(x1,y2,alpha12,alpha24)
            ft2_3=ft2(x2,y1,alpha12,alpha24)
            ft2_4=ft2(x2,y2,alpha12,alpha24)

            ft3_1=ft3(x1,y1,alpha34,alpha13)
            ft3_2=ft3(x1,y2,alpha34,alpha13)
            ft3_3=ft3(x2,y1,alpha34,alpha13)
            ft3_4=ft3(x2,y2,alpha34,alpha13)

            ft4_1=ft4(x1,y1,alpha34,alpha24)
            ft4_2=ft4(x1,y2,alpha34,alpha24)
            ft4_3=ft4(x2,y1,alpha34,alpha24)
            ft4_4=ft4(x2,y2,alpha34,alpha24)
            
            dfb1dx_1=dfb1dx(x1,y1)
            dfb1dx_2=dfb1dx(x1,y2)
            dfb1dx_3=dfb1dx(x2,y1)
            dfb1dx_4=dfb1dx(x2,y2)

            dfb2dx_1=dfb2dx(x1,y1)
            dfb2dx_2=dfb2dx(x1,y2)
            dfb2dx_3=dfb2dx(x2,y1)
            dfb2dx_4=dfb2dx(x2,y2)

            dfb3dx_1=dfb3dx(x1,y1)
            dfb3dx_2=dfb3dx(x1,y2)
            dfb3dx_3=dfb3dx(x2,y1)
            dfb3dx_4=dfb3dx(x2,y2)

            dfb4dx_1=dfb4dx(x1,y1)
            dfb4dx_2=dfb4dx(x1,y2)
            dfb4dx_3=dfb4dx(x2,y1)
            dfb4dx_4=dfb4dx(x2,y2)

            dfb1dy_1=dfb1dy(x1,y1)
            dfb1dy_2=dfb1dy(x1,y2)
            dfb1dy_3=dfb1dy(x2,y1)
            dfb1dy_4=dfb1dy(x2,y2)

            dfb2dy_1=dfb2dy(x1,y1)
            dfb2dy_2=dfb2dy(x1,y2)
            dfb2dy_3=dfb2dy(x2,y1)
            dfb2dy_4=dfb2dy(x2,y2)

            dfb3dy_1=dfb3dy(x1,y1)
            dfb3dy_2=dfb3dy(x1,y2)
            dfb3dy_3=dfb3dy(x2,y1)
            dfb3dy_4=dfb3dy(x2,y2)

            dfb4dy_1=dfb4dy(x1,y1)
            dfb4dy_2=dfb4dy(x1,y2)
            dfb4dy_3=dfb4dy(x2,y1)
            dfb4dy_4=dfb4dy(x2,y2)

            dft1dx_1=dft1dx(x1,y1,alpha12,alpha13)
            dft1dx_2=dft1dx(x1,y2,alpha12,alpha13)
            dft1dx_3=dft1dx(x2,y1,alpha12,alpha13)
            dft1dx_4=dft1dx(x2,y2,alpha12,alpha13)

            dft2dx_1=dft2dx(x1,y1,alpha12,alpha24)
            dft2dx_2=dft2dx(x1,y2,alpha12,alpha24)
            dft2dx_3=dft2dx(x2,y1,alpha12,alpha24)
            dft2dx_4=dft2dx(x2,y2,alpha12,alpha24)

            dft3dx_1=dft3dx(x1,y1,alpha34,alpha13)
            dft3dx_2=dft3dx(x1,y2,alpha34,alpha13)
            dft3dx_3=dft3dx(x2,y1,alpha34,alpha13)
            dft3dx_4=dft3dx(x2,y2,alpha34,alpha13)

            dft4dx_1=dft4dx(x1,y1,alpha34,alpha24)
            dft4dx_2=dft4dx(x1,y2,alpha34,alpha24)
            dft4dx_3=dft4dx(x2,y1,alpha34,alpha24)
            dft4dx_4=dft4dx(x2,y2,alpha34,alpha24)

            dft1dy_1=dft1dy(x1,y1,alpha12,alpha13)
            dft1dy_2=dft1dy(x1,y2,alpha12,alpha13)
            dft1dy_3=dft1dy(x2,y1,alpha12,alpha13)
            dft1dy_4=dft1dy(x2,y2,alpha12,alpha13)

            dft2dy_1=dft2dy(x1,y1,alpha12,alpha24)
            dft2dy_2=dft2dy(x1,y2,alpha12,alpha24)
            dft2dy_3=dft2dy(x2,y1,alpha12,alpha24)
            dft2dy_4=dft2dy(x2,y2,alpha12,alpha24)

            dft3dy_1=dft3dy(x1,y1,alpha34,alpha13)
            dft3dy_2=dft3dy(x1,y2,alpha34,alpha13)
            dft3dy_3=dft3dy(x2,y1,alpha34,alpha13)
            dft3dy_4=dft3dy(x2,y2,alpha34,alpha13)

            dft4dy_1=dft4dy(x1,y1,alpha34,alpha24)
            dft4dy_2=dft4dy(x1,y2,alpha34,alpha24)
            dft4dy_3=dft4dy(x2,y1,alpha34,alpha24)
            dft4dy_4=dft4dy(x2,y2,alpha34,alpha24)         

*     termes d approximation qui intervient dans b1
            apxb11=approx(ft1_1,ft1_2,ft1_3,ft1_4)
            apxb12=approx(ft2_1,ft2_2,ft2_3,ft2_4)
            apxb13=approx(ft3_1,ft3_2,ft3_3,ft3_4)
            apxb14=approx(ft4_1,ft4_2,ft4_3,ft4_4)

*     termes d approximation qui intervient dans b2
            apxb21_1=approx(dft1dx_1,dft1dx_2,dft1dx_3,dft1dx_4)
            apxb21_2=approx(dft2dx_1,dft2dx_2,dft2dx_3,dft2dx_4)
            apxb21_3=approx(dft3dx_1,dft3dx_2,dft3dx_3,dft3dx_4)
            apxb21_4=approx(dft4dx_1,dft4dx_2,dft4dx_3,dft4dx_4)
            apxb22_1=approx(dft1dy_1,dft1dy_2,dft1dy_3,dft1dy_4)
            apxb22_2=approx(dft2dy_1,dft2dy_2,dft2dy_3,dft2dy_4)
            apxb22_3=approx(dft3dy_1,dft3dy_2,dft3dy_3,dft3dy_4)
            apxb22_4=approx(dft4dy_1,dft4dy_2,dft4dy_3,dft4dy_4)

*     termes d approximation qui intervient dans M
            apxm11_1=approx(dfb1dx_1*ft1_1,dfb1dx_2*ft1_2,dfb1dx_3*ft1_3
     *           ,dfb1dx_4*ft1_4)
            apxm11_2=approx(dfb2dx_1*ft1_1,dfb2dx_2*ft1_2,dfb2dx_3*ft1_3
     *           ,dfb2dx_4*ft1_4)
            apxm11_3=approx(dfb3dx_1*ft1_1,dfb3dx_2*ft1_2,dfb3dx_3*ft1_3
     *           ,dfb3dx_4*ft1_4)
            apxm11_4=approx(dfb4dx_1*ft1_1,dfb4dx_2*ft1_2,dfb4dx_3*ft1_3
     *           ,dfb4dx_4*ft1_4)
            apxm12_1=approx(dfb1dx_1*ft2_1,dfb1dx_2*ft2_2,dfb1dx_3*ft2_3
     *           ,dfb1dx_4*ft2_4)
            apxm12_2=approx(dfb2dx_1*ft2_1,dfb2dx_2*ft2_2,dfb2dx_3*ft2_3
     *           ,dfb2dx_4*ft2_4)
            apxm12_3=approx(dfb3dx_1*ft2_1,dfb3dx_2*ft2_2,dfb3dx_3*ft2_3
     *           ,dfb3dx_4*ft2_4)
            apxm12_4=approx(dfb4dx_1*ft2_1,dfb4dx_2*ft2_2,dfb4dx_3*ft2_3
     *           ,dfb4dx_4*ft2_4)
            apxm13_1=approx(dfb1dx_1*ft3_1,dfb1dx_2*ft3_2,dfb1dx_3*ft3_3
     *           ,dfb1dx_4*ft3_4)
            apxm13_2=approx(dfb2dx_1*ft3_1,dfb2dx_2*ft3_2,dfb2dx_3*ft3_3
     *           ,dfb2dx_4*ft3_4)
            apxm13_3=approx(dfb3dx_1*ft3_1,dfb3dx_2*ft3_2,dfb3dx_3*ft3_3
     *           ,dfb3dx_4*ft3_4)
            apxm13_4=approx(dfb4dx_1*ft3_1,dfb4dx_2*ft3_2,dfb4dx_3*ft3_3
     *           ,dfb4dx_4*ft3_4)
            apxm14_1=approx(dfb1dx_1*ft4_1,dfb1dx_2*ft4_2,dfb1dx_3*ft4_3
     *           ,dfb1dx_4*ft4_4)
            apxm14_2=approx(dfb2dx_1*ft4_1,dfb2dx_2*ft4_2,dfb2dx_3*ft4_3
     *           ,dfb2dx_4*ft4_4)
            apxm14_3=approx(dfb3dx_1*ft4_1,dfb3dx_2*ft4_2,dfb3dx_3*ft4_3
     *           ,dfb3dx_4*ft4_4)
            apxm14_4=approx(dfb4dx_1*ft4_1,dfb4dx_2*ft4_2,dfb4dx_3*ft4_3
     *           ,dfb4dx_4*ft1_4)
********
            apxm21_1=approx(dfb1dy_1*ft1_1,dfb1dy_2*ft1_2,dfb1dy_3*ft1_3
     *           ,dfb1dy_4*ft1_4)
            apxm21_2=approx(dfb2dy_1*ft1_1,dfb2dy_2*ft1_2,dfb2dy_3*ft1_3
     *           ,dfb2dy_4*ft1_4)
            apxm21_3=approx(dfb3dy_1*ft1_1,dfb3dy_2*ft1_2,dfb3dy_3*ft1_3
     *           ,dfb3dy_4*ft1_4)
            apxm21_4=approx(dfb4dy_1*ft1_1,dfb4dy_2*ft1_2,dfb4dy_3*ft1_3
     *           ,dfb4dy_4*ft1_4)
            apxm22_1=approx(dfb1dy_1*ft2_1,dfb1dy_2*ft2_2,dfb1dy_3*ft2_3
     *           ,dfb1dy_4*ft2_4)
            apxm22_2=approx(dfb2dy_1*ft2_1,dfb2dy_2*ft2_2,dfb2dy_3*ft2_3
     *           ,dfb2dy_4*ft2_4)
            apxm22_3=approx(dfb3dy_1*ft2_1,dfb3dy_2*ft2_2,dfb3dy_3*ft2_3
     *           ,dfb3dy_4*ft2_4)
            apxm22_4=approx(dfb4dy_1*ft2_1,dfb4dy_2*ft2_2,dfb4dy_3*ft2_3
     *           ,dfb4dy_4*ft2_4)
            apxm23_1=approx(dfb1dy_1*ft3_1,dfb1dy_2*ft3_2,dfb1dy_3*ft3_3
     *           ,dfb1dy_4*ft3_4)
            apxm23_2=approx(dfb2dy_1*ft3_1,dfb2dy_2*ft3_2,dfb2dy_3*ft3_3
     *           ,dfb2dy_4*ft3_4)
            apxm23_3=approx(dfb3dy_1*ft3_1,dfb3dy_2*ft3_2,dfb3dy_3*ft3_3
     *           ,dfb3dy_4*ft3_4)
            apxm23_4=approx(dfb4dy_1*ft3_1,dfb4dy_2*ft3_2,dfb4dy_3*ft3_3
     *           ,dfb4dy_4*ft3_4)
            apxm24_1=approx(dfb1dy_1*ft4_1,dfb1dy_2*ft4_2,dfb1dy_3*ft4_3
     *           ,dfb1dy_4*ft4_4)
            apxm24_2=approx(dfb2dy_1*ft4_1,dfb2dy_2*ft4_2,dfb2dy_3*ft4_3
     *           ,dfb2dy_4*ft4_4)
            apxm24_3=approx(dfb3dy_1*ft4_1,dfb3dy_2*ft4_2,dfb3dy_3*ft4_3
     *           ,dfb3dy_4*ft4_4)
            apxm24_4=approx(dfb4dy_1*ft4_1,dfb4dy_2*ft4_2,dfb4dy_3*ft4_3
     *           ,dfb4dy_4*ft1_4)

*     termes d approximation qui intervient dans A
            apxa11_1=approx(dfb1dx_1*dft1dx_1,dfb1dx_2*dft1dx_2,dfb1dx_3
     *           *dft1dx_3,dfb1dx_4*dft1dx_4)
            apxa11_2=approx(dfb2dx_1*dft1dx_1,dfb2dx_2*dft1dx_2,dfb2dx_3
     *           *dft1dx_3,dfb2dx_4*dft1dx_4)
            apxa11_3=approx(dfb3dx_1*dft1dx_1,dfb3dx_2*dft1dx_2,dfb3dx_3
     *           *dft1dx_3,dfb3dx_4*dft1dx_4)
            apxa11_4=approx(dfb4dx_1*dft1dx_1,dfb4dx_2*dft1dx_2,dfb4dx_3
     *           *dft1dx_3,dfb4dx_4*dft1dx_4)
            apxa12_1=approx(dfb1dx_1*dft2dx_1,dfb1dx_2*dft2dx_2,dfb1dx_3
     *           *dft2dx_3,dfb1dx_4*dft2dx_4)
            apxa12_2=approx(dfb2dx_1*dft2dx_1,dfb2dx_2*dft2dx_2,dfb2dx_3
     *           *dft2dx_3,dfb2dx_4*dft2dx_4)
            apxa12_3=approx(dfb3dx_1*dft2dx_1,dfb3dx_2*dft2dx_2,dfb3dx_3
     *           *dft2dx_3,dfb3dx_4*dft2dx_4)
            apxa12_4=approx(dfb4dx_1*dft2dx_1,dfb4dx_2*dft2dx_2,dfb4dx_3
     *           *dft2dx_3,dfb4dx_4*dft2dx_4) 
            apxa13_1=approx(dfb1dx_1*dft3dx_1,dfb1dx_2*dft3dx_2,dfb1dx_3
     *           *dft3dx_3,dfb1dx_4*dft3dx_4)
            apxa13_2=approx(dfb2dx_1*dft3dx_1,dfb2dx_2*dft3dx_2,dfb2dx_3
     *           *dft3dx_3,dfb2dx_4*dft3dx_4)
            apxa13_3=approx(dfb3dx_1*dft3dx_1,dfb3dx_2*dft3dx_2,dfb3dx_3
     *           *dft3dx_3,dfb3dx_4*dft3dx_4)
            apxa13_4=approx(dfb4dx_1*dft3dx_1,dfb4dx_2*dft3dx_2,dfb4dx_3
     *           *dft3dx_3,dfb4dx_4*dft3dx_4) 
            apxa14_1=approx(dfb1dx_1*dft4dx_1,dfb1dx_2*dft4dx_2,dfb1dx_3
     *           *dft4dx_3,dfb1dx_4*dft4dx_4)
            apxa14_2=approx(dfb2dx_1*dft4dx_1,dfb2dx_2*dft4dx_2,dfb2dx_3
     *           *dft4dx_3,dfb2dx_4*dft4dx_4)
            apxa14_3=approx(dfb3dx_1*dft4dx_1,dfb3dx_2*dft4dx_2,dfb3dx_3
     *           *dft4dx_3,dfb3dx_4*dft4dx_4)
            apxa14_4=approx(dfb4dx_1*dft4dx_1,dfb4dx_2*dft4dx_2,dfb4dx_3
     *           *dft4dx_3,dfb4dx_4*dft4dx_4) 
*********
            apxa21_1=approx(dfb1dy_1*dft1dx_1,dfb1dy_2*dft1dx_2,dfb1dy_3
     *           *dft1dx_3,dfb1dy_4*dft1dx_4)
            apxa21_2=approx(dfb2dy_1*dft1dx_1,dfb2dy_2*dft1dx_2,dfb2dy_3
     *           *dft1dx_3,dfb2dy_4*dft1dx_4)
            apxa21_3=approx(dfb3dy_1*dft1dx_1,dfb3dy_2*dft1dx_2,dfb3dy_3
     *           *dft1dx_3,dfb3dy_4*dft1dx_4)
            apxa21_4=approx(dfb4dy_1*dft1dx_1,dfb4dy_2*dft1dx_2,dfb4dy_3
     *           *dft1dx_3,dfb4dy_4*dft1dx_4)
            apxa22_1=approx(dfb1dy_1*dft2dx_1,dfb1dy_2*dft2dx_2,dfb1dy_3
     *           *dft2dx_3,dfb1dy_4*dft2dx_4)
            apxa22_2=approx(dfb2dy_1*dft2dx_1,dfb2dy_2*dft2dx_2,dfb2dy_3
     *           *dft2dx_3,dfb2dy_4*dft2dx_4)
            apxa22_3=approx(dfb3dy_1*dft2dx_1,dfb3dy_2*dft2dx_2,dfb3dy_3
     *           *dft2dx_3,dfb3dy_4*dft2dx_4)
            apxa22_4=approx(dfb4dy_1*dft2dx_1,dfb4dy_2*dft2dx_2,dfb4dy_3
     *           *dft2dx_3,dfb4dy_4*dft2dx_4)
            apxa23_1=approx(dfb1dy_1*dft3dx_1,dfb1dy_2*dft3dx_2,dfb1dy_3
     *           *dft3dx_3,dfb1dy_4*dft3dx_4)
            apxa23_2=approx(dfb2dy_1*dft3dx_1,dfb2dy_2*dft3dx_2,dfb2dy_3
     *           *dft3dx_3,dfb2dy_4*dft3dx_4)
            apxa23_3=approx(dfb3dy_1*dft3dx_1,dfb3dy_2*dft3dx_2,dfb3dy_3
     *           *dft3dx_3,dfb3dy_4*dft3dx_4)
            apxa23_4=approx(dfb4dy_1*dft3dx_1,dfb4dy_2*dft3dx_2,dfb4dy_3
     *           *dft3dx_3,dfb4dy_4*dft3dx_4) 
            apxa24_1=approx(dfb1dy_1*dft4dx_1,dfb1dy_2*dft4dx_2,dfb1dy_3
     *           *dft4dx_3,dfb1dy_4*dft4dx_4)
            apxa24_2=approx(dfb2dy_1*dft4dx_1,dfb2dy_2*dft4dx_2,dfb2dy_3
     *           *dft4dx_3,dfb2dy_4*dft4dx_4)
            apxa24_3=approx(dfb3dy_1*dft4dx_1,dfb3dy_2*dft4dx_2,dfb3dy_3
     *           *dft4dx_3,dfb3dy_4*dft4dx_4)
            apxa24_4=approx(dfb4dy_1*dft4dx_1,dfb4dy_2*dft4dx_2,dfb4dy_3
     *           *dft4dx_3,dfb4dy_4*dft4dx_4) 
**********
            apxa31_1=approx(dfb1dx_1*dft1dy_1,dfb1dx_2*dft1dy_2,dfb1dx_3
     *           *dft1dy_3,dfb1dx_4*dft1dy_4)
            apxa31_2=approx(dfb2dx_1*dft1dy_1,dfb2dx_2*dft1dy_2,dfb2dx_3
     *           *dft1dy_3,dfb2dx_4*dft1dy_4)
            apxa31_3=approx(dfb3dx_1*dft1dy_1,dfb3dx_2*dft1dy_2,dfb3dx_3
     *           *dft1dy_3,dfb3dx_4*dft1dy_4)
            apxa31_4=approx(dfb4dx_1*dft1dy_1,dfb4dx_2*dft1dy_2,dfb4dx_3
     *           *dft1dy_3,dfb4dx_4*dft1dy_4)
            apxa32_1=approx(dfb1dx_1*dft2dy_1,dfb1dx_2*dft2dy_2,dfb1dx_3
     *           *dft2dy_3,dfb1dx_4*dft2dy_4)
            apxa32_2=approx(dfb2dx_1*dft2dy_1,dfb2dx_2*dft2dy_2,dfb2dx_3
     *           *dft2dy_3,dfb2dx_4*dft2dy_4)
            apxa32_3=approx(dfb3dx_1*dft2dy_1,dfb3dx_2*dft2dy_2,dfb3dx_3
     *           *dft2dy_3,dfb3dx_4*dft2dy_4)
            apxa32_4=approx(dfb4dx_1*dft2dy_1,dfb4dx_2*dft2dy_2,dfb4dx_3
     *           *dft2dy_3,dfb4dx_4*dft2dy_4) 
            apxa33_1=approx(dfb1dx_1*dft3dy_1,dfb1dx_2*dft3dy_2,dfb1dx_3
     *           *dft3dy_3,dfb1dx_4*dft3dy_4)
            apxa33_2=approx(dfb2dx_1*dft3dy_1,dfb2dx_2*dft3dy_2,dfb2dx_3
     *           *dft3dy_3,dfb2dx_4*dft3dy_4)
            apxa33_3=approx(dfb3dx_1*dft3dy_1,dfb3dx_2*dft3dy_2,dfb3dx_3
     *           *dft3dy_3,dfb3dx_4*dft3dy_4)
            apxa33_4=approx(dfb4dx_1*dft3dy_1,dfb4dx_2*dft3dy_2,dfb4dx_3
     *           *dft3dy_3,dfb4dx_4*dft3dy_4) 
            apxa34_1=approx(dfb1dx_1*dft4dy_1,dfb1dx_2*dft4dy_2,dfb1dx_3
     *           *dft4dy_3,dfb1dx_4*dft4dy_4)
            apxa34_2=approx(dfb2dx_1*dft4dy_1,dfb2dx_2*dft4dy_2,dfb2dx_3
     *           *dft4dy_3,dfb2dx_4*dft4dy_4)
            apxa34_3=approx(dfb3dx_1*dft4dy_1,dfb3dx_2*dft4dy_2,dfb3dx_3
     *           *dft4dy_3,dfb3dx_4*dft4dy_4)
            apxa34_4=approx(dfb4dx_1*dft4dy_1,dfb4dx_2*dft4dy_2,dfb4dx_3
     *           *dft4dy_3,dfb4dx_4*dft4dy_4) 
**********
            apxa41_1=approx(dfb1dy_1*dft1dy_1,dfb1dy_2*dft1dy_2,dfb1dy_3
     *           *dft1dy_3,dfb1dy_4*dft1dy_4)
            apxa41_2=approx(dfb2dy_1*dft1dy_1,dfb2dy_2*dft1dy_2,dfb2dy_3
     *           *dft1dy_3,dfb2dy_4*dft1dy_4)
            apxa41_3=approx(dfb3dy_1*dft1dy_1,dfb3dy_2*dft1dy_2,dfb3dy_3
     *           *dft1dy_3,dfb3dy_4*dft1dy_4)
            apxa41_4=approx(dfb4dy_1*dft1dy_1,dfb4dy_2*dft1dy_2,dfb4dy_3
     *           *dft1dy_3,dfb4dy_4*dft1dy_4)
            apxa42_1=approx(dfb1dy_1*dft2dy_1,dfb1dy_2*dft2dy_2,dfb1dy_3
     *           *dft2dy_3,dfb1dy_4*dft2dy_4)
            apxa42_2=approx(dfb2dy_1*dft2dy_1,dfb2dy_2*dft2dy_2,dfb2dy_3
     *           *dft2dy_3,dfb2dy_4*dft2dy_4)
            apxa42_3=approx(dfb3dy_1*dft2dy_1,dfb3dy_2*dft2dy_2,dfb3dy_3
     *           *dft2dy_3,dfb3dy_4*dft2dy_4)
            apxa42_4=approx(dfb4dy_1*dft2dy_1,dfb4dy_2*dft2dy_2,dfb4dy_3
     *           *dft2dy_3,dfb4dy_4*dft2dy_4)
            apxa43_1=approx(dfb1dy_1*dft3dy_1,dfb1dy_2*dft3dy_2,dfb1dy_3
     *           *dft3dy_3,dfb1dy_4*dft3dy_4)
            apxa43_2=approx(dfb2dy_1*dft3dy_1,dfb2dy_2*dft3dy_2,dfb2dy_3
     *           *dft3dy_3,dfb2dy_4*dft3dy_4)
            apxa43_3=approx(dfb3dy_1*dft3dy_1,dfb3dy_2*dft3dy_2,dfb3dy_3
     *           *dft3dy_3,dfb3dy_4*dft3dy_4)
            apxa43_4=approx(dfb4dy_1*dft3dy_1,dfb4dy_2*dft3dy_2,dfb4dy_3
     *           *dft3dy_3,dfb4dy_4*dft3dy_4) 
            apxa44_1=approx(dfb1dy_1*dft4dy_1,dfb1dy_2*dft4dy_2,dfb1dy_3
     *           *dft4dy_3,dfb1dy_4*dft4dy_4)
            apxa44_2=approx(dfb2dy_1*dft4dy_1,dfb2dy_2*dft4dy_2,dfb2dy_3
     *           *dft4dy_3,dfb2dy_4*dft4dy_4)
            apxa44_3=approx(dfb3dy_1*dft4dy_1,dfb3dy_2*dft4dy_2,dfb3dy_3
     *           *dft4dy_3,dfb3dy_4*dft4dy_4)
            apxa44_4=approx(dfb4dy_1*dft4dy_1,dfb4dy_2*dft4dy_2,dfb4dy_3
     *           *dft4dy_3,dfb4dy_4*dft4dy_4) 
         
c     
c     calcul du meme terme pour tous les elements de la diagonale 
c ---------------------------------------------------------------
c     
           term1=det*(b2*rcx*apxa11_1+ab*rcxy*apxa21_1
     *           +ab*rcyx*apxa31_1+a2*rcy*apxa41_1)
           term2=det*(b2*rcx*apxa12_2+ab*rcxy*apxa22_2
     *           +ab*rcyx*apxa32_2+a2*rcy*apxa42_2)
           term3=det*(b2*rcx*apxa13_3+ab*rcxy*apxa23_3
     *           +ab*rcyx*apxa33_3+a2*rcy*apxa43_3)
           term4=det*(b2*rcx*apxa14_4+ab*rcxy*apxa24_4
     *          +ab*rcyx*apxa34_4+a2*rcy*apxa44_4)


**********************************************************************************************************
            
*     si dans la fracture     
            if (( frac .ne. 0) .and. ((v01 .ne. 0.0) 
     *           .or. (v02 .ne. 0.0)))then        
               termm1=b*v01*apxm11_1+a*v02*apxm21_1
               term1=termm1+term1
               termm2=b*v01*apxm12_2+a*v02*apxm22_2
               term2=termm2+term2
               termm3=b*v01*apxm13_3+a*v02*apxm23_3
               term3=termm3+term3
               termm4=b*apxm14_4+a*v02*apxm24_4
               term4=termm4+term4
            end if
            
            maa(mia(ind1+1)-1)=maa(mia(ind1+1)-1)+term1
            maa(mia(ind2+1)-1)=maa(mia(ind2+1)-1)+term2
            maa(mia(ind3+1)-1)=maa(mia(ind3+1)-1)+term3
            maa(mia(ind4+1)-1)=maa(mia(ind4+1)-1)+term4
            
c     calcul du vecteur mja
            mja(mia(ind1+1)-1)=ind1
            mja(mia(ind2+1)-1)=ind2
            mja(mia(ind3+1)-1)=ind3
            mja(mia(ind4+1)-1)=ind4
c     
c     premiere diagonale
c     -----------------------
c     
*     cas mat(ind2,ind1)
            call remplissage_maa(ind1,ind2,mia,mja,bool,place,first)
            term1=det*(b2*rcx*apxa12_1+ab*rcxy*apxa22_1
     *           +ab*rcyx*apxa32_1+a2*rcy*apxa42_1)
            if ( frac .ne. 0) then
               termm1=b*v01*apxm12_1+a*v02*apxm22_1
               term1=termm1+term1
            end if
            if (bool .eq. 1) then           
               maa(place)=maa(place) + term1          
            else
               maa(first)=maa(first) + term1             
               mja(first)=ind2
            end if
            
*     cas mat(ind1,ind2)
            call remplissage_maa(ind2,ind1,mia,mja,bool,place,first)
            term1=det*(b2*rcx*apxa11_2+ab*rcxy*apxa21_2
     *           +ab*rcyx*apxa31_2+a2*rcy*apxa41_2)
            if ( frac .ne. 0) then
               termm1=b*v01*apxm11_2+a*v02*apxm21_2
               term1=termm1+term1
            end if
            if (bool .eq. 1) then
               maa(place)=maa(place) + term1
            else
               maa(first)=maa(first) + term1
               mja(first)=ind1
            end if
            
*     cas mat(ind4,ind3)
            call remplissage_maa(ind3,ind4,mia,mja,bool,place,first)
            term1=det*(b2*rcx*apxa14_3+ab*rcxy*apxa24_3
     *           +ab*rcyx*apxa34_3+a2*rcy*apxa44_3) 
            if ( frac .ne. 0) then
               termm1=b*v01*apxm14_3+a*v02*apxm24_3
               term1=termm1+term1
            end if          
            if (bool .eq. 1) then
               maa(place)=maa(place) + term1
            else
               maa(first)=maa(first) + term1
               mja(first)=ind4
            end if
            
*     cas mat(ind3,ind4)
            call remplissage_maa(ind4,ind3,mia,mja,bool,place,first)
            term1=det*(b2*rcx*apxa13_4+ab*rcxy*apxa23_4
     *           +ab*rcyx*apxa33_4+a2*rcy*apxa43_4)    
            if ( frac .ne. 0) then
               termm1=b*v01*apxm13_4+a*v02*apxm23_4
               term1=termm1+term1
            end if
            if (bool .eq. 1) then
               maa(place)=maa(place) + term1
            else
               maa(first)=maa(first) + term1
               mja(first)=ind3
            end if
            
c     
c     secondes diagonales
c -------------------------
c
*     cas mat(ind3,ind2) 
            call remplissage_maa(ind2,ind3,mia,mja,bool,place,first)
            term1=det*(b2*rcx*apxa13_2+ab*rcxy*apxa23_2
     *           +ab*rcyx*apxa33_2+a2*rcy*apxa43_2) 
            if ( frac .ne. 0) then
               termm1=b*v01*apxm13_2+a*v02*apxm23_2
               term1=termm1+term1
            end if    
            if (bool .eq. 1) then
               maa(place)=maa(place) + term1
            else
               maa(first)=maa(first) + term1
               mja(first)=ind3
            end if
            
*     cas mat(ind2,ind3)
            callremplissage_maa(ind3,ind2,mia,mja,bool,place,first)
            term1=det*(b2*rcx*apxa12_3+ab*rcxy*apxa22_3
     *           +ab*rcyx*apxa32_3+a2*rcy*apxa42_3)
            if ( frac .ne. 0) then
               termm1=b*v01*apxm12_3+a*v02*apxm22_3
               term1=termm1+term1
            end if   
            if (bool .eq. 1) then
               maa(place)=maa(place) + term1
            else
               maa(first)=maa(first) + term1
               mja(first)=ind2
            end if
c     
c     troisiemes diagonales
c ---------------------------
c
*     cas mat(ind3,ind1)
            call remplissage_maa(ind1,ind3,mia,mja,bool,place,first)
            term1=det*(b2*rcx*apxa13_1+ab*rcxy*apxa23_1
     *           +ab*rcyx*apxa33_1+a2*rcy*apxa43_1) 
            if ( frac .ne. 0) then
               termm1=b*v01*apxm13_1+a*v02*apxm23_1
               term1=termm1+term1
            end if
            if (bool .eq. 1) then
               maa(place)=maa(place) + term1
            else
               maa(first)=maa(first) + term1
               mja(first)=ind3
            end if
            
*     cas mat(ind1,ind3)
            call remplissage_maa(ind3,ind1,mia,mja,bool,place,first)
            term1=det*(b2*rcx*apxa11_3+ab*rcxy*apxa21_3
     *           +ab*rcyx*apxa31_3+a2*rcy*apxa41_3) 
            if ( frac .ne. 0) then
               termm1=b*v01*apxm11_3+a*v02*apxm21_3
               term1=termm1+term1
            end if
            if (bool .eq. 1) then
               maa(place)=maa(place) + term1
            else
               maa(first)=maa(first) + term1
               mja(first)=ind1
            end if
            
*     cas mat(ind4,ind2)         
            call remplissage_maa(ind2,ind4,mia,mja,bool,place,first)
            term1=det*(b2*rcx*apxa14_2+ab*rcxy*apxa24_2
     *           +ab*rcyx*apxa32_4+a2*rcy*apxa44_2) 
            if ( frac .ne. 0) then
               termm1=b*v01*apxm14_2+a*v02*apxm24_2
               term1=termm1+term1
            end if
            if (bool .eq. 1) then
               maa(place)=maa(place) + term1
            else
               maa(first)=maa(first) + term1
               mja(first)=ind4
            end if
            
*     cas mat(ind2,ind4)
            call remplissage_maa(ind4,ind2,mia,mja,bool,place,first)
            term1=det*(b2*rcx*apxa12_4+ab*rcxy*apxa22_4
     *           +ab*rcyx*apxa32_4+a2*rcy*apxa42_4) 
            if ( frac .ne. 0) then
               termm1=b*v01*apxm12_4+a*v02*apxm22_4
               term1=termm1+term1
            end if 
            if (bool .eq. 1) then
               maa(place)=maa(place) + term1
            else
               maa(first)=maa(first) + term1
               mja(first)=ind2
            end if
            
c     
c     quatrieme diagonale
c ---------------------------
c     
*     cas mat(ind4,ind1)
            call remplissage_maa(ind1,ind4,mia,mja,bool,place,first)
            term1=det*(b2*rcx*apxa14_1+ab*rcxy*apxa24_1
     *           +ab*rcyx*apxa34_1+a2*rcy*apxa44_1) 
            if ( frac .ne. 0) then
               termm1=b*v01*apxm14_1+a*v02*apxm24_1
               term1=termm1+term1
            end if
            if (bool .eq. 1) then
               maa(place)=maa(place) + term1
            else
               maa(first)=maa(first) + term1
               mja(first)=ind4
            end if
            
*     cas mat(ind1,ind4)
            callremplissage_maa(ind4,ind1,mia,mja,bool,place,first)
            term1=det*(b2*rcx*apxa11_4+ab*rcxy*apxa21_4
     *           +ab*rcyx*apxa31_4+a2*rcy*apxa41_4) 
            if ( frac .ne. 0) then
               termm1=b*v01*apxm11_4+a*v02*apxm21_4
               term1=termm1+term1
            end if
            if (bool .eq. 1) then
               maa(place)=maa(place) + term1
            else
               maa(first)=maa(first) + term1
               mja(first)=ind1
            end if
            
c     
c     calcul du second membre pour chaque probleme avec formule des trapezes
c ---------------------------------------------------------------------------
c     
            hrhs(ind1,1) = hrhs(ind1,1)
     1           + b*rcx*apxb21_1+a*rcyx*apxb22_1
            hrhs(ind1,2) = hrhs(ind1,2)
     1           + b*rcxy*apxb21_1+a*rcy*apxb22_1
c     
            hrhs(ind2,1) = hrhs(ind2,1)
     1           + b*rcx*apxb21_2+a*rcyx*apxb22_2
            hrhs(ind2,2) = hrhs(ind2,2)
     1           + b*rcxy*apxb21_2+a*rcy*apxb22_2
c     
            hrhs(ind3,1) = hrhs(ind3,1)
     1           + b*rcx*apxb21_3+a*rcyx*apxb22_3
            hrhs(ind3,2) = hrhs(ind3,2)
     1           + b*rcxy*apxb21_3+a*rcy*apxb22_3
c     
            hrhs(ind4,1) = hrhs(ind4,1)
     1           + b*rcx*apxb21_4+a*rcyx*apxb22_4
            hrhs(ind4,2) = hrhs(ind4,2)
     1           + b*rcxy*apxb21_4+a*rcy*apxb22_4

*     si dans la matrice     
            if ( frac .eq. 0 ) then           
               
               hrhs(ind1,1) = hrhs(ind1,1)
     *              - ab*kvf1*apxb11         
               hrhs(ind1,2) = hrhs(ind1,2)
     1              - ab*kvf2*apxb11   
c     
               hrhs(ind2,1) = hrhs(ind2,1)
     1              - ab*kvf1*apxb12 
               hrhs(ind2,2) = hrhs(ind2,2)
     1              - ab*kvf2*apxb12   
c     
               hrhs(ind3,1) = hrhs(ind3,1)
     1              - ab*kvf1*apxb13  
               hrhs(ind3,2) = hrhs(ind3,2)
     1              - ab*kvf2*apxb13
c     
               hrhs(ind4,1) = hrhs(ind4,1)
     1              - ab*kvf1*apxb14  
               hrhs(ind4,2) = hrhs(ind4,2)
     1              - ab*kvf2*apxb14    
               
            else               
               hrhs(ind1,1) = hrhs(ind1,1)
     1              + ab*(-v01 + kvf1)*apxb11         
               hrhs(ind1,2) = hrhs(ind1,2)
     1              + ab*(-v02 + kvf2)*apxb11
c     
               hrhs(ind2,1) = hrhs(ind2,1)
     1              + ab*(-v01 + kvf1)*apxb12
               hrhs(ind2,2) = hrhs(ind2,2)
     1              + ab*(-v02 + kvf2)*apxb12
c     
               hrhs(ind3,1) = hrhs(ind3,1)
     1              + ab*(-v01 + kvf1)*apxb13
               hrhs(ind3,2) = hrhs(ind3,2)
     1              + ab*(-v02 + kvf2)*apxb13
c     
               hrhs(ind4,1) = hrhs(ind4,1)
     1              + ab*(-v01 + kvf1)*apxb14
               hrhs(ind4,2) = hrhs(ind4,2)
     1              + ab*(-v02 + kvf2)*apxb14
                          
            end if             
c     

 4    continue

c     
c     tri des indices
c -------------------
c     
      do 5 i=1,nmat
         call Tri(mja(mia(i)),maa(mia(i)),mia(i+1)-mia(i))
 5    continue



c
c     initialisation des solutions initiales 
c -------------------------------------------
c

      do 6 i=1,nmat
         hx(i,1) = 0.001
 6    continue

c     
c     resolution du premier probleme
c ------------------------------
c    

      eps=cgeps
      write(*,*)'precision : ',eps
      write(*,*)'Kmax=',kmax
      call gmres(nmat,maa,mia,mja,hx(1,1),eps,nkr,kmax,hrhs(1,1))
         
c     
c     reinitialisation
c ------------------------
c     
      do 7 i=1,nmat
         hx(i,2)= 0.001
 7    continue
c     
c     resolution du second probleme
c ---------------------------------
c
      eps=cgeps
      call gmres(nmat,maa,mia,mja,hx(1,2),eps,nkr,kmax,hrhs(1,2))

c
c     retour au programme principal sans calcul du tenseur 
c --------------------------------------------------------
c
      if(iopt .ne. 0) return
c     
c     calcul final de la moyenne basee sur les elements finis
c
c     initialisation des valeurs
c ------------------------------
c
      zcx = 0.0
      zcxy = 0.0
      zcyx = 0.0
      zcy = 0.0
c     
c     boucles sur les noeuds
c --------------------------
c     
      do 8 j=1,nvaly
         j2 = ( j - 1 ) * nvalx
c     
c     recherche de la seconde coordonnee dans la grille de permeabilite
c ---------------------------------------------------------------------
c     
         indy = ( j + nyhr - 1 ) / nyhr
c     
c     calcul de la taille d'un element selon y
c --------------------------------------------
c     
         if(nyh .eq. 1) then
            yvalm = 0.0
            yvalp = 1.0
         else
            if(indy .eq. 1) then
               yvalm = 1.5 * y(1) - 0.5 * y(2)
               yvalp = 0.5 * ( y(indy) + y(indy+1) )
            end if
            if(indy .gt. 1 .and. indy .lt. nyh) then
               yvalm = 0.5 * ( y(indy-1) + y(indy) )
               yvalp = 0.5 * ( y(indy) + y(indy+1) )
            end if
            if(indy .eq. nyh) then
               yvalm = 0.5 * ( y(indy-1) + y(indy) )
               yvalp = 1.5 * y(indy) - 0.5 * y(indy-1)
            end if
         end if
         dely = ( yvalp - yvalm ) / float(nyhr)
         
         
         do 8 i=1,nvalx
c
c     recherche de la premiere coordonnee dans la grille de permeabilite
c ----------------------------------------------------------------------
c
            indx = ( i + nxhr - 1 ) / nxhr
c     
c     calcul de la taille d'un element selon x
c --------------------------------------------
c
            if(nxh .eq. 1) then
               xvalm = 0.0
               xvalp = 1.0
            else
               if(indx .eq. 1) then
                  xvalm = 1.5 * x(1) - 0.5 * x(2)
                  xvalp = 0.5 * ( x(indx) + x(indx+1) )
               end if
               if(indx .gt. 1 .and. indx .lt. nxh) then
                  xvalm = 0.5 * ( x(indx-1) + x(indx) )
                  xvalp = 0.5 * ( x(indx) + x(indx+1) )
               end if
               if(indx .eq. nxh) then
                  xvalm = 0.5 * ( x(indx-1) + x(indx) )
                  xvalp = 1.5 * x(indx) - 0.5 * x(indx-1)
               end if
            end if
            delx = ( xvalp - xvalm ) / float(nxhr)
*     constantes
            a=delx/2
            b=dely/2
            ab=delx*dely/4
            a2=a*a
            b2=b*b
c     
c     calcul du volume de la sous cellule en cours
c ------------------------------------------------
c     
            del = delx * dely
c
c     attribution de la permeabilite
c ----------------------------------
c     
            indif = indx + ( indy - 1 ) * nxh
            rcx = dx(indif)
            rcxy = dxy(indif)
            rcyx = dyx(indif)
            rcy = dy(indif)
            rv1 = v1(indif)
            rv2 = v2(indif)
c     
c     calcul des indices des 4 sommets de l'element 
c     en utilisant la propriete de la periodicite
c --------------------------------------------------------------
c     
            ind1 = i + j2
            if(i .eq. nvalx) then
               ind2 = 1 + j2
            else
               ind2 = ind1 + 1
            end if
            if( j .eq. nvaly) then
               ind3 = i
            else
               ind3 = ind1 + nvalx
            end if
            if(j .eq. nvaly) then
               if(i .eq. nvalx) then
                  ind4 = 1
               else
                  ind4 = i + 1
               end if
            else
               ind4 = ind2 + nvalx
            end if

*     points pour l approximation
            x1=-1./sqrt(3.)
            y1=-1./sqrt(3.)
            x2=1./sqrt(3.)
            y2=1./sqrt(3.)
            fb1_1=fb1(x1,y1)
            fb1_2=fb1(x1,y2)
            fb1_3=fb1(x2,y1)
            fb1_4=fb1(x2,y2)

            fb2_1=fb2(x1,y1)
            fb2_2=fb2(x1,y2)
            fb2_3=fb2(x2,y1)
            fb2_4=fb2(x2,y2)

            fb3_1=fb3(x1,y1)
            fb3_2=fb3(x1,y2)
            fb3_3=fb3(x2,y1)
            fb3_4=fb3(x2,y2)

            fb4_1=fb4(x1,y1)
            fb4_2=fb4(x1,y2)
            fb4_3=fb4(x2,y1)
            fb4_4=fb4(x2,y2)
            
            dfb1dx_1=dfb1dx(x1,y1)
            dfb1dx_2=dfb1dx(x1,y2)
            dfb1dx_3=dfb1dx(x2,y1)
            dfb1dx_4=dfb1dx(x2,y2)

            dfb2dx_1=dfb2dx(x1,y1)
            dfb2dx_2=dfb2dx(x1,y2)
            dfb2dx_3=dfb2dx(x2,y1)
            dfb2dx_4=dfb2dx(x2,y2)

            dfb3dx_1=dfb3dx(x1,y1)
            dfb3dx_2=dfb3dx(x1,y2)
            dfb3dx_3=dfb3dx(x2,y1)
            dfb3dx_4=dfb3dx(x2,y2)

            dfb4dx_1=dfb4dx(x1,y1)
            dfb4dx_2=dfb4dx(x1,y2)
            dfb4dx_3=dfb4dx(x2,y1)
            dfb4dx_4=dfb4dx(x2,y2)

            dfb1dy_1=dfb1dy(x1,y1)
            dfb1dy_2=dfb1dy(x1,y2)
            dfb1dy_3=dfb1dy(x2,y1)
            dfb1dy_4=dfb1dy(x2,y2)

            dfb2dy_1=dfb2dy(x1,y1)
            dfb2dy_2=dfb2dy(x1,y2)
            dfb2dy_3=dfb2dy(x2,y1)
            dfb2dy_4=dfb2dy(x2,y2)

            dfb3dy_1=dfb3dy(x1,y1)
            dfb3dy_2=dfb3dy(x1,y2)
            dfb3dy_3=dfb3dy(x2,y1)
            dfb3dy_4=dfb3dy(x2,y2)

            dfb4dy_1=dfb4dy(x1,y1)
            dfb4dy_2=dfb4dy(x1,y2)
            dfb4dy_3=dfb4dy(x2,y1)
            dfb4dy_4=dfb4dy(x2,y2)

*     termes d approximation qui intervient le tenseur effectif
            apxd11=approx(fb1_1,fb1_2,fb1_3,fb1_4)
            apxd12=approx(fb2_1,fb2_2,fb2_3,fb2_4)
            apxd13=approx(fb3_1,fb3_2,fb3_3,fb3_4)
            apxd14=approx(fb4_1,fb4_2,fb4_3,fb4_4)

            apxd21_1=approx(dfb1dx_1,dfb1dx_2,dfb1dx_3,dfb1dx_4)
            apxd21_2=approx(dfb2dx_1,dfb2dx_2,dfb2dx_3,dfb2dx_4)
            apxd21_3=approx(dfb3dx_1,dfb3dx_2,dfb3dx_3,dfb3dx_4)
            apxd21_4=approx(dfb4dx_1,dfb4dx_2,dfb4dx_3,dfb4dx_4)
            apxd22_1=approx(dfb1dy_1,dfb1dy_2,dfb1dy_3,dfb1dy_4)
            apxd22_2=approx(dfb2dy_1,dfb2dy_2,dfb2dy_3,dfb2dy_4)
            apxd22_3=approx(dfb3dy_1,dfb3dy_2,dfb3dy_3,dfb3dy_4)
            apxd22_4=approx(dfb4dy_1,dfb4dy_2,dfb4dy_3,dfb4dy_4)

c     
c     ajout du terme respectif
c ----------------------------
c
c            zcx = zcx + ( ab*4*rcx +
            zcx = zcx + ( ab*4*rcx
     *           + rcx*b*2*(hx(ind1,1)*apxd21_1+hx(ind2,1)*apxd21_2
     *           + hx(ind3,1)*apxd21_3+hx(ind4,1)*apxd21_4)
     *           + rcxy*a*2*(hx(ind1,1)*apxd22_1+hx(ind2,1)*apxd22_2
     *           + hx(ind3,1)*apxd22_3+hx(ind4,1)*apxd22_4)
     *           - ab*(apxd11*hx(ind1,1)+apxd12*hx(ind2,1)
     *           + apxd13*hx(ind3,1)+apxd14*hx(ind4,1))*rv1)

c            zcxy = zcxy + ( ab*4*rcxy +
            zcxy = zcxy + ( ab*4*rcxy
     *           + rcx*b*2*(hx(ind1,2)*apxd21_1+hx(ind2,2)*apxd21_2
     *           + hx(ind3,2)*apxd21_3+hx(ind4,2)*apxd21_4)
     *           + rcxy*a*2*(hx(ind1,2)*apxd22_1+hx(ind2,2)*apxd22_2
     *           + hx(ind3,2)*apxd22_3+hx(ind4,2)*apxd22_4)
     *           - ab*(apxd11*hx(ind1,2)+apxd12*hx(ind2,2)
     *           + apxd13*hx(ind3,2)+apxd14*hx(ind4,2))*rv1)

c            zcyx = zcyx + ( ab*4*rcyx +
            zcyx = zcyx + ( ab*4*rcyx
     *           + rcyx*b*2*(hx(ind1,1)*apxd21_1+hx(ind2,1)*apxd21_2
     *           + hx(ind3,1)*apxd21_3+hx(ind4,1)*apxd21_4)
     *           + rcy*a*2*(hx(ind1,1)*apxd22_1+hx(ind2,1)*apxd22_2
     *           + hx(ind3,1)*apxd22_3+hx(ind4,1)*apxd22_4)
     *           - ab*(apxd11*hx(ind1,1)+apxd12*hx(ind2,1)
     *           + apxd13*hx(ind3,1)+apxd14*hx(ind4,1))*rv2)

c            zcy = zcy + ( ab*4*rcy +
            zcy = zcy + ( ab*4*rcy
     *           + rcyx*b*2*(hx(ind1,2)*apxd21_1+hx(ind2,2)*apxd21_2
     *           + hx(ind3,2)*apxd21_3+hx(ind4,2)*apxd21_4)
     *           + rcy*a*2*(hx(ind1,2)*apxd22_1+hx(ind2,2)*apxd22_2
     *           + hx(ind3,2)*apxd22_3+hx(ind4,2)*apxd22_4)
     *           - ab*(apxd11*hx(ind1,2)+apxd12*hx(ind2,2)
     *           + apxd13*hx(ind3,2)+apxd14*hx(ind4,2))*rv2)

********************************



          zcx1 = zcx1 + del
     *         * ( rcx + rcx * 0.5 * ( hx(ind2,1) - hx(ind1,1) 
     *         + hx(ind4,1) - hx(ind3,1) ) + rcxy * 0.5 * ( hx(ind3,1)
     *         - hx(ind1,1) + hx(ind4,1) - hx(ind2,1) )
     *         - 0.25 * rv1 * ( hx(ind2,1) + hx(ind1,1) + hx(ind4,1) 
     *         + hx(ind3,1)))
c
c     
            zcxy1 = zcxy1 + del
     *           * ( rcxy + rcx * 0.5 * ( hx(ind2,2) - hx(ind1,2)
     *           + hx(ind4,2) - hx(ind3,2) ) + rcxy * 0.5 * ( hx(ind3,2)
     *           - hx(ind1,2) + hx(ind4,2) - hx(ind2,2) )
     *           - 0.25 * rv1 * ( hx(ind2,2) + hx(ind1,2) + hx(ind4,2) 
     *           + hx(ind3,2) )  )
c     
            zcyx1 = zcyx1 + del
     *           * ( rcyx + rcyx * 0.5 * ( hx(ind2,1) - hx(ind1,1)
     *           + hx(ind4,1) - hx(ind3,1) ) + rcy * 0.5 * ( hx(ind3,1)
     *           - hx(ind1,1) + hx(ind4,1) - hx(ind2,1) )
     *           - 0.25 * rv2 * ( hx(ind2,1) + hx(ind1,1) + hx(ind4,1) 
     *              + hx(ind3,1) ) )
c     
            zcy1 = zcy1 + del
     *           * ( rcy + rcyx * 0.5 * ( hx(ind2,2) - hx(ind1,2) 
     *           + hx(ind4,2) - hx(ind3,2) ) + rcy * 0.5 * ( hx(ind3,2)
     *           - hx(ind1,2) + hx(ind4,2) - hx(ind2,2) )
     *           - 0.25 * rv2 * ( hx(ind2,2) + hx(ind1,2) + hx(ind4,2) 
     *           + hx(ind3,2) ) )

 8    continue
c
c     division par la taille de la cellule
c ---------------------------------------
c
      zcx =   zcx / cellsize
      zcxy =  zcxy / cellsize
      zcyx =  zcyx / cellsize
      zcy =   zcy / cellsize

      zcx1 =   zcx1 / cellsize
      zcxy1 =  zcxy1 / cellsize
      zcyx1 =  zcyx1 / cellsize
      zcy1 =   zcy1 / cellsize

      write(*,*)
      write(*,*)'TENSEUR HOMCOD'
      write(*,*)zcx1,'\t',zcxy1
      write(*,*)zcxy1,'\t',zcy1
c      pause

      return
      end
c\end{verbatim}


