c     Programme permfrac2d-sc.f
c     
c     Programme principal permettant de traiter des cas 
c     de milieux poreux fractures en 2 D
c  
c     on calcule le tenseur homogeneise de permeabilite en considerant que le matrice est impermeable
c     on considere simplement comme donnee la permeabilite sur la fracture
c      
c          *
c         ***
c        * * *      les permeabilites sont donnees sous cellules par sous cellules. 
c       *  *  *
c      *   *   *    On doit lire dans le fichier de donnees le vecteur contenant les permeabilites
c     *    *    *   pour chacune des sous cellules
c    *           *
c   *      *      *     RQ : le programme permfrac2d.f permet de traiter le cas  
c  *****************         ou la permeabilite est identique dans la fracture
c  


c                            FICHIERS ATTACHES
c                          ---------------------
c
c     donnees2d_permf-sc.dat
c     ICholeski.f
c     ISolvSys.f
c     IAmul.f
c     tri.f
c     gradcpmorse.f
c     remplissage_maa.f
c     homogab2_permfrac.f
c
c     simple precision :
c     makefile   : Make2s_perm-sc
c     executable : permf_sc2s
c
c     double precision :
c     makefile   : Make2d_perm-sc
c     executable : permf-sc2d


c     Ecrit par Benedicte VIDAL
c     Stage IPSN
c          suivi par Brahim AMAZIANE et Christophe SERRES
c     
c     avril - septembre 2001
      




      program permfrac2d_sc
      
*     definition des parameters ou nv est la dimension des vecteurs ie le nombre d inconnues 
*     et nm est la dimension de la matrice       
      parameter (nv=900000,nm=1500000)
      
*      implicit none
      
c     declaration des variables
      integer nxh,nyh,nxhf,nyhf,nxhr,nyhr
      integer i,nmat,nmorse,nvalx,nvaly,nvalfx,nvalfy,ntot,ntotf,
     *     ntotv,nvar,nvarf
      real k(nv),x(nv),y(nv)
      real kx(nv),ky(nv),kxy(nv),kyx(nv)
      real zkx,zky,zkxy,zkyx,zx,zy
      character*200 buf
      
      common/morse/nmat,nmorse
      common/morse1/nvalx,nvaly,nvalfx,nvalfy
      common/morse2/ntot,ntotf,ntotv,nvar,nvarf
      
c     Lecture des donnees dans le fichier de donnees
      open (10,FILE='input.dat',STATUS='old')
      read (10,*) buf,buf,nxh,buf,nyh,buf,nxhf,buf
     *     ,nyhf,buf,nxhr,buf,nyhr,buf
      read (10,*) (k(i), i=1, (nxh*nyh-nxhf*nyhf))
      close (10)
      
c     Calcul des milieux des cellules
      CALL mil_cel(nxh,nyh,x,y)
      
c     Remplissage des vecteurs kx,ky,kxy,kyx dans le fracture seulement
      CALL remplissage_k(k,nxh,nyh,nxhf,nyhf,kx,ky,kxy,kyx)

c     calcul de termes intervenant dans le calcul du tenseur K*
      nvalx=nxh*nxhr
      nvaly = nyh * nyhr
      nvalfx = nxhf * nxhr
      nvalfy = nyhf * nyhr
      ntot = nxh * nyh
      ntotf = nxhf * nyhf
      ntotv = ntot - ntotf
      nvar = nvalx * nvaly
      nvarf = nvalfx * nvalfy
      
*     nombre d inconnues
      nmat = nvar - (nvalfx-1)*(nvalfy-1)
*     taille de la matrice morse
      nmorse=((nmat-4-2*(nvalfx-1)-2*(nvalfy-1))*9+8*4
     *     +2*(nvalfx-1)*6+2*(nvalfy-1)*6-nmat)/2+nmat 
      
c     appel de la procedure pour le calcul de K*
      print *,'just before'
      CALL homogab2_permfrac(nv,nm,kx,kxy,kyx,ky,x,y,nxh,nyh,nxhf,
     *nyhf,nxhr,nyhr,zkx,zkxy,zkyx,zky,zx,zy,0)
      
*     affichage a l ecran du tenseur homogeneise
      write(*,*)'*******************************************************
     ******'
      write(*,*)'\t \t TENSEUR HOMOGENEISE'
      write(*,*)
      write(*,*)zkx,'\t',zkxy
      write(*,*)zkyx,'\t',zky
      write(*,*)
      write(*,*)'*******************************************************
     ******'
      write(*,*)
      write(*,*)'FIN DU PROGRAMME'
*      pause 
      
      end
c     fin du programme principal



********************************************************************************
c     Calcul des milieux des sous cellules

      subroutine mil_cel(nxh,nyh,x,y)

* nxh : nombre de sous cellules selon x
* nyh : nombre de sous cellules selon y
*   x : vecteur contenant les abscisses des milieux des sous cellules
*   y : vecteur contenant les ordonnees des milieux des sous cellules

      implicit none
      
      real x(*),y(*)
      integer i,nxh,nyh
      
      do 1 i=1,nxh
         x(i)=(2*FLOAT(i)-1)/(2*nxh)
 1    continue
      
      do 2 i=1,nyh
         y(i)=(2*FLOAT(i)-1)/(2*nyh)
 2    continue
      
      end

*********************************************************************************
c     Remplissage des vecteurs kx,ky,kxy,kyx 

      subroutine remplissage_k(k,nxh,nyh,nxhf,nyhf,kx,ky,kxy,kyx)

*    k : vecteur contenant les permeabilites pour chaque sous cellules
*  nxh : nombre de sous cellules selon x
*  nyh : nombre de sous cellules selon y
* nxhf : nombre de sous cellules constituant la matrice selon x
* nyhf : nombre de sous cellules constituant la matrice selon y
*   kx : vecteur contenant l'element 1,1 du tenseur de permeabilite correspondant a chaque sous cellules
*  kxy : vecteur contenant l'element 1,2 du tenseur de permeabilite correspondant a chaque sous cellules
*  kyx : vecteur contenant l'element 2,1 du tenseur de permeabilite correspondant a chaque sous cellules   
*   ky : vecteur contenant l'element 2,2 du tenseur de permeabilite correspondant a chaque sous cellules   

      implicit none

      real kx(*),ky(*),kxy(*),kyx(*)
      real k(*)
      integer i,nxh,nyh,nyhf,nxhf
      
      do 1 i=1,nxh*nyh-nxhf*nyhf
         kx(i)=k(i)
         ky(i)=kx(i)
         kxy(i)=0
         kyx(i)=0
 1    continue
      
      end

***********************************************************************************





