c     Programme permfrac2d-sc.f
c     
c     Programme principal permettant de traiter des cas 
c     de milieux poreux fractures en 2 D
c  
c     on calcule le tenseur homogeneise de permeabilite en considerant que le matrice est impermeable
c     on considere simplement comme donnee la permeabilite sur la fracture
c      
c          *
c         ***
c        * * *      les permeabilites sont donnees sous cellules par sous cellules. 
c       *  *  *
c      *   *   *    On doit lire dans le fichier de donnees le vecteur contenant les permeabilites
c     *    *    *   pour chacune des sous cellules
c    *           *
c   *      *      *     RQ : le programme permfrac2d.f permet de traiter le cas  
c  *****************         ou la permeabilite est identique dans la fracture
c  


c                            FICHIERS ATTACHES
c                          ---------------------
c
c     donnees2d_permf-sc.dat
c     ICholeski.f
c     ISolvSys.f
c     IAmul.f
c     tri.f
c     gradcpmorse.f
c     remplissage_maa.f
c     homogab2_permfrac.f
c
c     simple precision :
c     makefile   : Make2s_perm-sc
c     executable : permf_sc2s
c
c     double precision :
c     makefile   : Make2d_perm-sc
c     executable : permf-sc2d


c     Ecrit par Benedicte VIDAL
c     Stage IPSN
c          suivi par Brahim AMAZIANE et Christophe SERRES
c     
c     avril - septembre 2001
      




      program permfrac2d_sc
      
*     definition des parameters ou nv est la dimension des vecteurs ie le nombre d inconnues 
*     et nm est la dimension de la matrice       
      parameter (nv=900000,nm=1500000)
      
*      implicit none
      
c     declaration des variables
      integer nxh,nyh,nxhf,nyhf,nxhr,nyhr
      integer i,nmat,nmorse,nvalx,nvaly,nvalfx,nvalfy,ntot,ntotf,
     *     ntotv,nvar,nvarf
      real k(nv),x(nv),y(nv)
      real kx(nv),ky(nv),kxy(nv),kyx(nv)
      real zkx,zky,zkxy,zkyx,zx,zy
      character*200 buf
      
      common/morse/nmat,nmorse
      common/morse1/nvalx,nvaly,nvalfx,nvalfy
      common/morse2/ntot,ntotf,ntotv,nvar,nvarf
      
c     Lecture des donnees dans le fichier de donnees
      open (10,FILE='permInput.dat',STATUS='old')
      read (10,*) buf,buf,nxh,buf,nyh,buf,nxhf,buf
     *     ,nyhf,buf,nxhr,buf,nyhr,buf
      read (10,*) (k(i), i=1, (nxh*nyh-nxhf*nyhf))
      close (10)
      
c     Calcul des milieux des cellules
      CALL mil_cel(nxh,nyh,x,y)
      
c     Remplissage des vecteurs kx,ky,kxy,kyx dans le fracture seulement
      CALL remplissage_k(k,nxh,nyh,nxhf,nyhf,kx,ky,kxy,kyx)

c     calcul de termes intervenant dans le calcul du tenseur K*
      nvalx=nxh*nxhr
      nvaly = nyh * nyhr
      nvalfx = nxhf * nxhr
      nvalfy = nyhf * nyhr
      ntot = nxh * nyh
      ntotf = nxhf * nyhf
      ntotv = ntot - ntotf
      nvar = nvalx * nvaly
      nvarf = nvalfx * nvalfy
      
*     nombre d inconnues
      nmat = nvar - (nvalfx-1)*(nvalfy-1)
*     taille de la matrice morse
      nmorse=((nmat-4-2*(nvalfx-1)-2*(nvalfy-1))*9+8*4
     *     +2*(nvalfx-1)*6+2*(nvalfy-1)*6-nmat)/2+nmat 
      
c     appel de la procedure pour le calcul de K*
      CALL homogab2_permfrac(nv,nm,kx,kxy,kyx,ky,x,y,nxh,nyh,nxhf,
     *nyhf,nxhr,nyhr,zkx,zkxy,zkyx,zky,zx,zy,0)
      
*     affichage a l ecran du tenseur homogeneise
      write(*,*)'*******************************************************
     ******'
      write(*,*)'\t \t TENSEUR HOMOGENEISE'
      write(*,*)
      write(*,*)zkx,'\t',zkxy
      write(*,*)zkyx,'\t',zky
      write(*,*)
      write(*,*)'*******************************************************
     ******'
      write(*,*)
      write(*,*)'FIN DU PROGRAMME'

      open (20,FILE='permOutput.dat')
      write(20,*) '1'
      write(20,*) '1'
      write(20,*) '1'
      write(20,*) zkx
      write(20,*) zkxy
      write(20,*) '0.0'
      write(20,*) zkyx
      write(20,*) zky
      write(20,*) '0.0'
      write(20,*) '0.0'
      write(20,*) '0.0'
      write(20,*) '1.0'
      close (20)

*      pause 
      
      end
c     fin du programme principal



********************************************************************************
c     Calcul des milieux des sous cellules

      subroutine mil_cel(nxh,nyh,x,y)

* nxh : nombre de sous cellules selon x
* nyh : nombre de sous cellules selon y
*   x : vecteur contenant les abscisses des milieux des sous cellules
*   y : vecteur contenant les ordonnees des milieux des sous cellules

      implicit none
      
      real x(*),y(*)
      integer i,nxh,nyh
      
      do 1 i=1,nxh
         x(i)=(2*FLOAT(i)-1)/(2*nxh)
 1    continue
      
      do 2 i=1,nyh
         y(i)=(2*FLOAT(i)-1)/(2*nyh)
 2    continue
      
      end

*********************************************************************************
c     Remplissage des vecteurs kx,ky,kxy,kyx 

      subroutine remplissage_k(k,nxh,nyh,nxhf,nyhf,kx,ky,kxy,kyx)

*    k : vecteur contenant les permeabilites pour chaque sous cellules
*  nxh : nombre de sous cellules selon x
*  nyh : nombre de sous cellules selon y
* nxhf : nombre de sous cellules constituant la matrice selon x
* nyhf : nombre de sous cellules constituant la matrice selon y
*   kx : vecteur contenant l'element 1,1 du tenseur de permeabilite correspondant a chaque sous cellules
*  kxy : vecteur contenant l'element 1,2 du tenseur de permeabilite correspondant a chaque sous cellules
*  kyx : vecteur contenant l'element 2,1 du tenseur de permeabilite correspondant a chaque sous cellules   
*   ky : vecteur contenant l'element 2,2 du tenseur de permeabilite correspondant a chaque sous cellules   

      implicit none

      real kx(*),ky(*),kxy(*),kyx(*)
      real k(*)
      integer i,nxh,nyh,nyhf,nxhf
      
      do 1 i=1,nxh*nyh-nxhf*nyhf
         kx(i)=k(i)
         ky(i)=kx(i)
         kxy(i)=0
         kyx(i)=0
 1    continue
      
      end

***********************************************************************************





*******************************************************
******                ICholeski.f              ********
*
C Ce sous-programme effectue la factorisation de
C Choleski incomplete de la matrice A stockee sous
C forme Morse dans le tableau aa avec les tableaux
C auxiliaires d"indices ia et ja.
C ia(k): indice dans le tableau aa du premier element
C   de la i eme ligne de A stocke dans aa,
C ja(k): indice j de colonne de l"element Aij stocke
C   dans le tableau aa a l"indice k.
C Le resultat est stocke dans le tableau al, qui 
C utilise les memes tableaux auxiliaires d"indices
C ia et ja que aa.
C Le calcul de la factorisation incomplete se fait
C ligne par ligne en terminant par le calcul de 
C l"element diagonal.
C

*	de 'INTRODUCTION AU CALCUL SCIENTIFIQUE' 
*	B. LUCQUIN
*	O. PIRONNEAU
*	MASSON, 1996



      subroutine ICholeski(ns,aa,ia,ja,al)

      implicit none
      
      real aa(*),al(*)
      integer ia(*),ja(*)
      real alij,alii
      integer jlik,jljk,ik,jk
      integer ns,i,j,k

      do 30 i=1,ns

         do 34 k=ia(i),ia(i+1)-2  
            j=ja(k)
            alij=aa(k)
            ik=ia(i)
            jk=ia(j)

 35         if (ik .le. (ia(i+1)-2)) then
               jlik=ja(ik)

 36            if (jk .le. (ia(j+1)-2)) then                  
                  jljk=ja(jk)

                  if (jlik .gt. jljk) then
                     jk=jk+1
                     goto 36

                  elseif (jlik .lt. jljk) then
                        ik=ik+1
                        goto 35
                     
                     else
                        alij=alij-al(ik)*al(jk)
                        ik=ik+1
                        jk=jk+1
                        goto 35
                     end if
                  end if
               end if

               al(k)=alij/al(ia(j+1)-1)
               
 34       continue
         

         alii=aa(ia(i+1)-1)

         do 33 k=ia(i),ia(i+1)-2
            alii=alii-al(k)*al(k)
 33      continue
         
         al(ia(i+1)-1)=sqrt(abs(alii))

 30   continue
 
      
      return
      end





*******************************************************
******                 ISolvSys.f              ********
*
C Ce sous-programme resout le systeme lineaire
C $LL^T z = r$, ou L est une matrice triangulaire
C inferieure de taille ns x ns, stockee sous forme
C Morse dans les tableaux (al,il,jl). En entree, le 
C tableau z contient le second membre du systeme; 
C en sortie, il contient la solution z.
C La resolution se fait en 2 etapes: une etape de 
C descente, suivie d'une etape de remontee.
C

*	de 'INTRODUCTION AU CALCUL SCIENTIFIQUE' 
*	B. LUCQUIN
*	O. PIRONNEAU
*	MASSON, 1996


      subroutine ISolvSys(ns,z,al,il,jl)

      implicit none
      
      integer ns
      real z(ns),al(*)
      integer il(ns),jl(ns)
      integer i,k,i1,jlk
      real somme

      do 1 i=1,ns
         somme=z(i)
         do 2 k=il(i),il(i+1)-2
            somme=somme-al(k)*z(jl(k))
 2       continue

         z(i)=somme/al(il(i+1)-1)
 1    continue
      
      do 3 i=1,ns
         z(i)=z(i)/al(il(i+1)-1)
 3    continue

      do 4 i1=1,ns
         i=ns+1-i1
         do 4 k=il(i),il(i+1)-2
            jlk=jl(k)
            z(jlk)=z(jlk)-al(k)*z(i)/al(il(jlk+1)-1)
 4    continue

      return
      end





*******************************************************
******                  IAmul.f                ********
*
C Ce sous-programme calcule le produit de la matrice
C A, stockee sous forme Morse dans les tableaux 
C (aa,ia,ja), par le vecteur x; le resultat est dans le
C tableau b.
C Le calcul se fait d"abord en calculant la contribution
C associee a la partie triangulaire inferieure de la 
C matrice (qui est celle stockee dans le tableau aa),
C puis celle associee a la partie triangulaire superieure
C stricte.
C
*	de 'INTRODUCTION AU CALCUL SCIENTIFIQUE' 
*	B. LUCQUIN
*	O. PIRONNEAU
*	MASSON, 1996



      subroutine IAmul(ns,aa,ia,ja,x,b)
      
      implicit none
      
      integer ns
      real x(ns),b(ns),aa(*)
      integer ia(*),ja(*)
      integer i,k,jak
      real somme,xi

      do 1 i=1,ns
         somme=0.
         do 2 k=ia(i),ia(i+1)-1
            somme=somme+aa(k)*x(ja(k))
 2       continue
         b(i)=somme
 1    continue
      
      do 3 i=1,ns
         xi=x(i)
         do 3 k=ia(i),ia(i+1)-2
            jak=ja(k)
            b(jak)=b(jak)+aa(k)*xi
 3    continue

      return
      end




**************************************************************
********                     tri.f                      ******

*
* Ce sous-programme est ecrit sous forme tres compacte
* ce qui le rend difficile a la comprehension.
*
* Il s'agit de ranger les elements du tableau criter
* par ordre croissant d"indices et de modifier 
* simultanement les elements du tableau record.
*
*
*	de 'INTRODUCTION AU CALCUL SCIENTIFIQUE' 
*	B. LUCQUIN
*	O. PIRONNEAU
*	MASSON, 1996


      subroutine Tri(criter,record,n)
      
      implicit none
 
* n : nombre d'elements a ranger
* criter : vecteur contenant les indices des colonnes des elements contenus dans record
* record : vecteur contenant les elements de la matrice de discretisation
     
      integer n
      integer crit,criter(n)
      real record(n),rec
      integer l,r,j,i
      
      if (n .le. 1) then
         return
      end if
      l=n/2+1
      r=n

 2    if (l .gt. 1) then
         l=l-1
         rec=record(l)
         crit=criter(l)
      else
         rec=record(r)
         crit=criter(r)
         record(r)=record(1)
         criter(r)=criter(1)
         r=r-1
         if (r .eq. 1) then
            record(1)=rec
            criter(1)=crit
            return
         end if
      end if

 3    j=l
 4    i=j
      j=j*2
      if (j-r .lt. 0) then
         goto 5
      elseif (j .eq.r) then
         goto 6
      else 
         goto 8
      end if
 5    if (criter(j) .lt. criter(j+1)) j=j+1
 6    if (crit .ge. criter(j)) goto 8
      record(i)=record(j)
      criter(i)=criter(j)
      goto 4
 8    record(i)=rec
      criter(i)=crit
      goto 2

      end
      











*******************************************************
******               gracpmorse.f              ********
*
C Ce sous-programme resout le systeme lineaire $Ax=b$ par
C gradient conjugue preconditionne, avec au plus kMax 
C iterations. Le test d"arret est: $|Ax^k-b|<eps|Ax^0-b|$.
C La matrice $A$ du systeme lineaire, de taille $n \times n$
C est stockee sous forme Morse dans le tableau aa;
C ce stockage necessite aussi les tableaux auxiliaires
C d"indices ia et ja, comme explique dans le chapitre III.
C La matrice $C$ de preconditionnement est la factorisation
C de Choleski incomplete de $A$ ($C=L^* (L^*)^{T}$)  et la
C matrice $L^*$ est stockee sous forme Morse dans le 
C tableau al, avec les memes tableaux d"indices ia et 
C ja que $A$.
C
C     entree:
C     --------
C       matrice $A$ du systeme stockee sous forme Morse 
C         dans les tableaux (aa,ia,ja),
C       factorisee de Choleski incomplete de $A$ stockee
C         dans les tableaux (al,ia,ja);
C       second membre de l"equation stocke dans le 
C         tableau b;
C       nombre maximal kMax d"iterations, precision eps
C         du calcul;
C
C     N.B.: le calcul du second membre ne servant qu"a 
C       l"etape d"initialisation, le tableau b est ensuite
C       ecrase et utilise comme tableau auxiliaire de  
C       calcul, en particulier pour le stockage du produit 
C       de la matrice $A$ par $h$, puis pour le stockage 
C       de la solution y du systeme $C y= A h$.
C       
C                            
C     sortie:
C     ------- 
C       x:tableau contenant les valeurs de la solution 
C         en chacun des sommets du maillage;
C       gcg:scalaire contenant le carre de la norme de
C         $g$ (en pratique ce nombre doit decroitre au 
C         cours des iterations).
C                            
C     autres tableaux:
C     ----------------
C       g, cg: tableaux contenant les valeurs de $g^n$ et
C         de $C g^n$ respectivement;
C       h: tableau contenant la direction de descente $h^n$
C         (au tout debut du programme, ce tableau sert a 
C          stocker le produit de $A$ par $x^0$);
C       
C
C     sous-programmes:
C     ---------------- 
C       IAmul:effectue le produit d"une matrice par 
C         un vecteur, la matrice etant stockee sous
C         forme Morse,
C       ISolvSys:resolution du systeme lineaire 
C         $L L^T x=y$ par descente remontee, pour
C         une matrice $L$ stockee sous forme Morse;
C       Ces deux sous-programmes sont dans le fichier
C       GCPLaplac.for.
C
C

*	de 'INTRODUCTION AU CALCUL SCIENTIFIQUE' 
*	B. LUCQUIN
*	O. PIRONNEAU
*	MASSON, 1996


      subroutine gradcpmorse(n,aa,ia,ja,al,sc,x,g,h,cg,kmax,eps)
      
      implicit none
      
      integer n,i,k,kmax
      real x(*),b(n),sc(*),g(n),h(n),cg(n)
      real aa(*),al(*)
      integer ja(*),ia(*)
      real gcg,eps,hAh,ro,gamma

      do 100 i=1,n
         b(i)=sc(i)
         x(i)=0.00
         cg(i)=0.0
         h(i)=0.0
         g(i)=0.0
         
 100  continue

      do 1 k=1,kmax

c        cas ou k=1
         if(k .eq. 1) then
c           appel de la procedure pour la multiplication mat/vecteur
            call IAmul(n,aa,ia,ja,x,h)

            do 2 i=1,n
               cg(i)=b(i)-h(i)
               g(i)=cg(i)
 2          continue
            
c           appel de la procedure pour resoudre al.z=g (g=second membre et sol)
            call ISolvSys(n,g,al,ia,ja)

            gcg=0.
            do 3 i=1,n
               gcg=gcg+g(i)*cg(i)
               h(i)=g(i)
 3          continue
 
         end if
      
c        appel de la procedure pour la multiplication mat/vecteur
         call IAmul(n,aa,ia,ja,h,b)
        
         hAh=0.
         do 10 i=1,n
            hAh=hAh+h(i)*b(i)
 10      continue
         
         ro=gcg/hAh
         
         do 11 i=1,n
            cg(i)=cg(i)-ro*b(i)
 11      continue
         
c     appel de la procedure pour resoudre al.z=b (g=second membre et sol) 
         call ISolvSys(n,b,al,ia,ja)

         do 12 i=1,n
            x(i)=x(i)+ro*h(i)
            g(i)=g(i)-ro*b(i)
 12      continue

         gamma=gcg
         gcg=0.
         do 13 i=1,n
            gcg=gcg+g(i)*cg(i)
 13      continue
         
         if (k .eq. 1) eps=eps*gcg
         if (gcg .lt. eps) then
             write(*,*)'GCG<EPS'
             write(*,*)'nbre d iterations k=',k
             write(*,*)'eps=',eps
             write(*,*)'gcg=',gcg
             write(*,*)'FIN GRADCP'
             write(*,*)
             write(*,*)
*             pause 
             
            return
         end if
         gamma=gcg/gamma
         do 14 i=1,n
            h(i)=g(i)+gamma*h(i)
 14      continue

 1    continue

      write(*,*)'K=KMAX'
      write(*,*)'nbre d iterations k=',k
      write(*,*)'eps=',eps
      write(*,*)'gcg=',gcg
      write(*,*) 'FIN GRADCP'
      write(*,*)
      write(*,*)
       
      return
      end










*** fichier remplissage_maa.f

      subroutine remplissage_maa (i,j,ia,ja,bool,empl,prem)
      
      implicit none

*  i : indice de la ligne pour la matrice de discretisation
*  j : indice de la colonne pour la matrice de discretisation
* ia : vecteur contenant la position dans a des premiers elements non nuls de chaque ligne
* ja : vecteur contenat les colonnes de chaque element de a
* bool : booleen qui prend 1 si l'element A(i,j) a deja ete implemente et 0 sinon
* empl : donne la place de l'element dans ja (et donc dans aa qui contient les elements non nuls de la matrice de discretisation) QUAND bool = 1
* prem : donne le premier emplacement vide dans le vecteur ja QUAND bool = 0


      integer ia(*),ja(*)
      integer i,j,empl,bool,prem,k

      bool=0
      prem=0
      k=ia(i) 

*     boucle tant que
   1  if ((bool .eq. 0) .and. (k .le. (ia(i+1)-2))) then

*       si element a deja implemente
	if (ja(k) .eq. j) then
           bool=1
	   k=k-1
        end if

*	si element non encore implemente : determination du premier emplacement libre
        if ((bool .eq. 0) .and. (ja(k).eq. 0) .and.
     *       ((prem .gt. k) .or.(prem .eq. 0))) then
           prem=k
        end if

        k=k+1
        go to 1
      endif  
*     fin boucle tant que

      empl=k

      return
      end
      







c\begin{verbatim}
      subroutine homogab2_permfrac(nv,nm,kx,kxy,kyx,ky,x,y,nxh,
     1nyh,nxhf,nyhf,nxhr,nyhr,zkx,zkxy,zkyx,zky,zx,zy,iopt)
c
c * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
c
c crit par : joe koebbe 
c et modifie par : benedicte VIDAL
c
c   Cette procedure permet de calculer pour un MILIEU POREUX FRACTURE 
c le tenseur homogeneise.
c
c Le tenseur contenant les permeabilite est de la forme :
c
c                 [  kx    kxy  ]
c             K = [             ]
c                 [  kyx    ky  ]
c
c  Le tenseur homogeneise est donne en sortie sous la forme de la matrice K#
c                          #   [  zkx    zkxy  ] 
c                         K  = [               ]
c                              [  zkyx   zky   ]
c
c
c * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
c
      
      implicit none

      common /morse/ nmat,nmorse
      common /morse1/nvalx,nvaly,nvalfx,nvalfy
      common /morse2/ntot,ntotf,ntotv,nvar,nvarf

c     declaration des variables
c-------------------------------
      integer nxh,nyh,nxhf,nyhf,nxhr,nyhr,iopt,ntot,nv,nm
      integer n1,n2,n1r,n2r
      real sixth,quarter,third
      real xvalm,xvalp,yvalm,yvalp,supval
      real kx(*),kxy(*),kyx(*),ky(*)
      real x(*),y(*)
      real zkx,zkxy,zkyx,zky
      real rkx,rkxy,rkyx,rky
      real zx,zy
      real cgeps,eps,constant
      real sum,cellsize,term1,term2,delx,dely,del
      integer n1f,n2f,nvalfx,nvalfy,ntotf,ntotv,nvar,nmat,nvarf
      integer nvalx,nvaly
      integer nfx,nfy,nfxr,nfyr,nmorse
      integer kmax,i,ii,j2,j,ind1,ind2,ind3,ind4,indx,indy,indprm
c     variables pour le stockage morse  
c     vecteur pour la matrice morse
      integer mia(nv),mja(nm)                     
      real maa(nm),mal(nm)
      real hrhs(nv,2),hx(nv,2)
      integer first,place,bool
c     variables pour le gradient conjugue preconditionne
      real h(nv),cg(nv),g(nv)

c
c     definition de parametres utilises dans la suite
c -----------------------------------------------------
c
      data sixth,quarter,third
     1          / 0.166666666666660,.25,0.333333333333333 /
      data cgeps / 0.000001 /
c
c     definition du nombre d'iteration maximum
c ----------------------------------------------
c
      print *,'got here'
      kmax=nmat    
c
c     si le nombre d'elements ou le raffinement selon un axe 
c     est nul le mettre a 1
c -----------------------------------------------------------
c
      if(nxh .le. 0) nxh = 1
      if(nyh .le. 0) nyh = 1
      if(nxhr .le. 0) nxhr = 1
      if(nyhr .le. 0) nyhr = 1
c
c     posons des variables pour la suite
c --------------------------------------
c
      n1 = nxh
      n2 = nyh
      n1r = nxhr
      n2r = nyhr
      n1f = nxhf
      n2f = nyhf
      nfx = 0.5 * ( nxh - nxhf )
      nfy = 0.5 * ( nyh - nyhf )
      nfxr = nfx * nxhr
      nfyr = nfy * nyhr
 
c
c     calcul de la taille de la cellule de base 
c     pour une utilisation ulterieure
c -----------------------------------------------
c
      if(nxh .eq. 1) then
         xvalm = 0.0
         xvalp = 1.0
      else
         xvalm = 1.5 * x(1) - 0.5 * x(2)
         xvalp = 1.5 * x(nxh) - 0.5 * x(nxh-1)
      end if
      if(nyh .eq. 1) then
         yvalm = 0.0
         yvalp = 1.0
      else
         yvalm = 1.5 * y(1) - 0.5 * y(2)
         yvalp = 1.5 * y(nyh) - 0.5 * y(nyh-1)
      end if
      cellsize = ( xvalp - xvalm ) * ( yvalp - yvalm )

c     
c     calcul des coordonnees du nouveau centre de la cellule
c -----------------------------------------------------------
c
      zx = 0.5 * ( xvalp + xvalm )
      zy = 0.5 * ( yvalp + yvalm )

c
c     recherche du maximum dans les vecteurs
c -------------------------------------------
c
      supval = 0.0
      do 1 i=1,ntotv
         if(abs(kx(i)) .gt. supval)  supval = kx(i)
         if(abs(kxy(i)) .gt. supval) supval = kxy(i)
         if(abs(kyx(i)) .gt. supval) supval = kyx(i)
         if(abs(ky(i)) .gt. supval)  supval = ky(i)
    1 continue

c
c     verification des valeurs nulles sur la diagonale en entree
c ---------------------------------------------------------------
c
      constant = 0.000001 * supval
      do 2 i=1,ntotv
c
c     somme en valeur absolue des entrees
c -------------------------------------------
c
         sum = abs(kx(i))  + abs(kxy(i))
     1       + abs(kyx(i)) + abs(ky(i))
c
c     verification de la somme
c --------------------------------
c
         if(sum .lt. constant) then
           kx(i) = constant
           ky(i) = constant
         end if

    2 continue
      
c
c     initialisation de la matrice et du second membre
c ----------------------------------------------------
c
      do 3 i=1,nmat
         hrhs(i,1) = 0.0
         hrhs(i,2) = 0.0
         maa(i)=0.0
         mja(i)=0
         mia(i)=0  
 3    continue
     
      do 31 i=nmat+1,nmorse
         maa(i)=0.0
         mja(i)=0
 31   continue
      mia(nmat+1)=nmorse+1
      
c
c     boucles sur les noeuds
c ---------------------------
c   
      do 4 j=1,nvaly
         j2 = ( j - 1 ) * nvalx
c
c     recherche de la seconde coordonnee dans la grille de permeabilite
c -------------------------------------------------------------------------
c
         indy = ( j + nyhr - 1 ) / nyhr
c
c     calcul de la taille d'un element selon y
c ------------------------------------------------
c
         if(nyh .eq. 1) then
           yvalm = 0.0
           yvalp = 1.0
         else
           if(indy .eq. 1) then
             yvalm = 1.5 * y(1) - 0.5 * y(2)
             yvalp = 0.5 * ( y(indy) + y(indy+1) )
           end if
           if(indy .gt. 1 .and. indy .lt. nyh) then
             yvalm = 0.5 * ( y(indy-1) + y(indy) )
             yvalp = 0.5 * ( y(indy) + y(indy+1) )
           end if
           if(indy .eq. nyh) then
             yvalm = 0.5 * ( y(indy-1) + y(indy) )
             yvalp = 1.5 * y(indy) - 0.5 * y(indy-1)
           end if
         end if
         dely = ( yvalp - yvalm ) / float(nyhr)


      if (j .le. nfyr) then
       
      do 40 i=1,nvalx
c
c     recherche de la premiere coordonnee dans la grille de permeabilite
c --------------------------------------------------------------------------
c
         indx = ( i + nxhr - 1 ) / nxhr
c
c     calcul de la taille d'un element selon x
c ------------------------------------------------
c
         if(nxh .eq. 1) then
           xvalm = 0.0
           xvalp = 1.0
         else
           if(indx .eq. 1) then
             xvalm = 1.5 * x(1) - 0.5 * x(2)
             xvalp = 0.5 * ( x(indx) + x(indx+1) )
           end if
           if(indx .gt. 1 .and. indx .lt. nxh) then
             xvalm = 0.5 * ( x(indx-1) + x(indx) )
             xvalp = 0.5 * ( x(indx) + x(indx+1) )
           end if
           if(indx .eq. nxh) then
             xvalm = 0.5 * ( x(indx-1) + x(indx) )
             xvalp = 1.5 * x(indx) - 0.5 * x(indx-1)
           end if
         end if
         delx = ( xvalp - xvalm ) / float(nxhr)
c
c     attribution de la permeabilite
c -------------------------------------
c
         indprm = indx + ( indy - 1 ) * nxh
         rkx = kx(indprm)
         rkxy = kxy(indprm)
         rkyx = kyx(indprm)
         rky = ky(indprm)

c
c     calcul de la contribution de la diagonale pour l'element
c
c     calcul des indices des 4 sommets de l'element 
c     en utilisant la propriete de la periodicite
c --------------------------------------------------------------
c
         ind1 = i + j2
         if(i .eq. nvalx) then
           ind2 = 1 + j2
         else
           ind2 = ind1 + 1
         end if
         if( j .eq. nvaly) then
           ind3 = i
         else
           ind3 = ind1 + nvalx
         end if
         if(j .eq. nvaly) then
           if(i .eq. nvalx) then
             ind4 = 1
           else
             ind4 = i + 1
           end if
         else
           ind4 = ind2 + nvalx
         end if        
c
c     calcul du vecteur MIA pour ZONE 1
c---------------------------------------
c
         if (j .eq. 1) then
            if (i .eq. 1) then
               mia(ind1)=1
               mia(ind2)=2
               mia(ind2+1)=4
               mia(ind3)=1+(nvalx-2)*2+3+1
               mia(ind4)=mia(ind3)+4
               mia(ind4+1)=mia(ind4)+5
            else
               if ((i+2 .ge. 2) .and. (i+1 .le. nvalx-1)) then
                  mia(ind2+1)=mia(ind2)+2
                  mia(ind4+1)=mia(ind4)+5
               else
                  if (i+1 .eq. nvalx) then
                     mia(ind4+1)=mia(ind4)+6
                  end if
               end if
            end if
         else
            if (i .eq. 1) then
               mia(ind4)=mia(ind3)+4
               mia(ind4+1)=mia(ind4)+5
            else
              if ((i+2 .ge. 2) .and. (i+1 .le. nvalx-1)) then
                  mia(ind4+1)=mia(ind4)+5
               else
                  if (i+1 .eq. nvalx) then
                     mia(ind4+1)=mia(ind4)+6
                  end if 
               end if
            end if
         end if
c     
c     calcul du meme terme pour tous les elements de la diagonale 
c ---------------------------------------------------------------
c
         term1 = third * ( dely * rkx / delx + delx * rky / dely )
     1          + quarter * ( rkxy + rkyx )
         term2 = third * ( dely * rkx / delx + delx * rky / dely )
     1          - quarter * ( rkxy + rkyx )

c     calcul du vecteur maa
         maa(mia(ind1+1)-1)=maa(mia(ind1+1)-1)+term1
         maa(mia(ind2+1)-1)=maa(mia(ind2+1)-1)+term2
         maa(mia(ind3+1)-1)=maa(mia(ind3+1)-1)+term2
         maa(mia(ind4+1)-1)=maa(mia(ind4+1)-1)+term1
c     calcul du vecteur mja
         mja(mia(ind1+1)-1)=ind1
         mja(mia(ind2+1)-1)=ind2
         mja(mia(ind3+1)-1)=ind3
         mja(mia(ind4+1)-1)=ind4
c
c     premiere diagonale
c -----------------------
c
         if (ind1 .ge. ind2) then
            call remplissage_maa(ind1,ind2,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely
               mja(first)=ind2
            end if
         else
            call remplissage_maa(ind2,ind1,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
               mja(first)=ind1
            end if
         end if
         
         if (ind3 .ge.ind4) then
            call remplissage_maa(ind3,ind4,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
               mja(first)=ind4
            end if
         else
            call remplissage_maa(ind4,ind3,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely 
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely
               mja(first)=ind3
            end if
         end if
c     
c     seconde diagonale
c     -------------------------
c
         if (ind2 .ge. ind3) then 
            call remplissage_maa(ind2,ind3,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx )
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx )
               mja(first)=ind3
            end if
         else
            call remplissage_maa(ind3,ind2,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx ) 
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx )
               mja(first)=ind2
            end if
         end if
c     
c     troisieme diagonale
c ---------------------------
c
         if (ind1 .ge. ind3) then
            call remplissage_maa(ind1,ind3,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
               mja(first)=ind3
            end if
         else
            call remplissage_maa(ind3,ind1,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
               mja(first)=ind1
            end if
         end if
         
         if (ind2 .ge. ind4) then
            call remplissage_maa(ind2,ind4,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
               mja(first)=ind4
            end if
         else
            call remplissage_maa(ind4,ind2,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
               mja(first)=ind2
            end if
         end if
c     
c     quatrieme diagonale
c ---------------------------
c
         if (ind1 .ge. ind4) then
            call remplissage_maa(ind1,ind4,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
               mja(first)=ind4
            end if
         else
            call remplissage_maa(ind4,ind1,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
               mja(first)=ind1
            end if
         end if 
c     
c     calcul du second membre pour chaque probleme
c     ------------------------------------------------
c
         hrhs(ind1,1) = hrhs(ind1,1)
     1            + 0.5 * ( dely * rkx / delx + delx * rkyx / dely )
         hrhs(ind1,2) = hrhs(ind1,2)
     1            + 0.5 * ( dely * rkxy / delx + delx * rky / dely )
c
         hrhs(ind2,1) = hrhs(ind2,1)
     1            - 0.5 * ( dely * rkx / delx - delx * rkyx / dely )
         hrhs(ind2,2) = hrhs(ind2,2)
     1            - 0.5 * ( dely * rkxy / delx - delx * rky / dely )
c
         hrhs(ind3,1) = hrhs(ind3,1)
     1            + 0.5 * ( dely * rkx / delx - delx * rkyx / dely )
         hrhs(ind3,2) = hrhs(ind3,2)
     1            + 0.5 * ( dely * rkxy / delx - delx * rky / dely )
c
         hrhs(ind4,1) = hrhs(ind4,1)
     1            - 0.5 * ( dely * rkx / delx + delx * rkyx / dely )
         hrhs(ind4,2) = hrhs(ind4,2)
     1            - 0.5 * ( dely * rkxy / delx + delx * rky / dely )

c
 40   continue

      end if

      if ((j .ge. nfyr+1)   .and.
     *    (j .le. (nvaly-nfyr))) then
     
      do 41 i=1,nfxr
c
c     recherche de la premiere coordonnee dans la grille de permeabilite
c --------------------------------------------------------------------------
c
         indx = ( i + nxhr - 1 ) / nxhr
c
c        calcul de la taille d'un element selon x
c ------------------------------------------------
c
         if(nxh .eq. 1) then
           xvalm = 0.0
           xvalp = 1.0
         else
           if(indx .eq. 1) then
             xvalm = 1.5 * x(1) - 0.5 * x(2)
             xvalp = 0.5 * ( x(indx) + x(indx+1) )
           end if
           if(indx .gt. 1 .and. indx .lt. nxh) then
             xvalm = 0.5 * ( x(indx-1) + x(indx) )
             xvalp = 0.5 * ( x(indx) + x(indx+1) )
           end if
           if(indx .eq. nxh) then
             xvalm = 0.5 * ( x(indx-1) + x(indx) )
             xvalp = 1.5 * x(indx) - 0.5 * x(indx-1)
           end if
         end if
         delx = ( xvalp - xvalm ) / float(nxhr)
c
c        attribution de la permabilite
c -------------------------------------
c
         if (( j .ge. nfyr+1) .and. ( j .le. (nfyr + nyhr)) ) then
            indprm = indx + ( indy - 1 ) * nxh
         else
            indprm = indx + ( indy - 1 ) * nxh - (indy-1 -nfy) * nxhf
         end if
         rkx = kx(indprm)
         rkxy = kxy(indprm)
         rkyx = kyx(indprm)
         rky = ky(indprm)
c
c     calcul de la contribution de la diagonale pour la cellule
c
c     calcul des indices des 4 sommets de l'element
c     en utilisant la propriete de la periodicite
c ----------------------------------------------------------
c

         if ((j .eq. nfyr+1).or. ( j .eq. (nfyr +2 ))) then
            ind1 = i + j2
         else
            ind1 = i + j2-(j-(nfyr+2))*(nvalfx-1)
         end if
         ind2 = ind1 + 1
         if( j .eq. (nfyr + 1)) then
           ind3 = ind1 + nvalx
         else
           ind3 = ind1 + nvalx - ( nvalfx - 1 )
         end if
         ind4=ind3+1

c
c     calcul du vecteur MIA pour ZONE 2
c---------------------------------------
c

         if ((j .ne. nvaly-nfyr+1) .and. (j .ne. nvaly-1)) then
            if (i .eq. 1) then
               mia(ind4)=mia(ind3)+4
            end if
            if ((i+2 .ge. 2) .and. (i+2 .le. nfxr+1)) then
               mia(ind4+1)=mia(ind4)+5
            else
               mia(ind4+1)=mia(ind4)+4
               if (j .eq. nvaly-nfyr) then
                  do 410 ii=2,nvalfx
                     mia(ind4+ii)=mia(ind4+ii-1)+2
 410              continue
               end if             
            end if 
         else
            if (i .eq. 1) then
               mia(ind4)=mia(ind3)+7
            end if
            if (i+1 .eq. nfxr+1) then
               mia(ind4+1)=mia(ind4)+7
               if (j .eq. nvaly-nfyr) then
                  do 411 ii=2,nvalfx
                     mia(ind4+ii)=mia(ind4+ii-1)+5
 411              continue
               end if
            else
               if ((i+2 .ge. 2) .and. (i+2 .le. nfxr+2)) then
                  mia(ind4+1)=mia(ind4)+5
               end if
            end if
         end if
c
c     calcul du meme terme pour tous les elements de la diagonale
c ---------------------------------------------------------------
c
         term1 = third * ( dely * rkx / delx + delx * rky / dely )
     1          + quarter * ( rkxy + rkyx )
         term2 = third * ( dely * rkx / delx + delx * rky / dely )
     1          - quarter * ( rkxy + rkyx )
c
c     calcul du vecteur maa
         maa(mia(ind1+1)-1)=maa(mia(ind1+1)-1)+term1
         maa(mia(ind2+1)-1)=maa(mia(ind2+1)-1)+term2
         maa(mia(ind3+1)-1)=maa(mia(ind3+1)-1)+term2
         maa(mia(ind4+1)-1)=maa(mia(ind4+1)-1)+term1
c     calcul du vecteur mja
         mja(mia(ind1+1)-1)=ind1
         mja(mia(ind2+1)-1)=ind2
         mja(mia(ind3+1)-1)=ind3
         mja(mia(ind4+1)-1)=ind4

c
c     premiere diagonale
c -----------------------
c
         if (ind1 .ge. ind2) then
            call remplissage_maa(ind1,ind2,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely
               mja(first)=ind2
            end if
         else
            call remplissage_maa(ind2,ind1,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
               mja(first)=ind1
            end if
         end if
         
         if (ind3 .ge.ind4) then
            call remplissage_maa(ind3,ind4,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
               mja(first)=ind4
            end if
         else
            call remplissage_maa(ind4,ind3,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx  
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely 
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely
               mja(first)=ind3
            end if
         end if
c     
c     seconde diagonale
c -------------------------
c
         if (ind2 .ge. ind3) then 
            call remplissage_maa(ind2,ind3,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx )
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx )
               mja(first)=ind3
            end if
         else
            call remplissage_maa(ind3,ind2,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx ) 
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx )
               mja(first)=ind2
            end if
         end if
         
c     
c     troisieme diagonale
c ---------------------------
c
         if (ind1 .ge. ind3) then
            call remplissage_maa(ind1,ind3,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
               mja(first)=ind3
            end if
         else
            call remplissage_maa(ind3,ind1,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
               mja(first)=ind1
            end if
         end if
         
         if (ind2 .ge. ind4) then
            call remplissage_maa(ind2,ind4,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
               mja(first)=ind4
            end if
         else
            call remplissage_maa(ind4,ind2,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
               mja(first)=ind2
            end if
         end if
c     
c     quatrieme diagonale
c ---------------------------
c
         if (ind1 .ge. ind4) then
            call remplissage_maa(ind1,ind4,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
               mja(first)=ind4
            end if
         else
            call remplissage_maa(ind4,ind1,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
               mja(first)=ind1
            end if
         end if 
c     
c     calcul du second membre pour chacun des problemes
c     -----------------------------------------------------
c     
         hrhs(ind1,1) = hrhs(ind1,1)
     1            + 0.5 * ( dely * rkx / delx + delx * rkyx / dely )
         hrhs(ind1,2) = hrhs(ind1,2)
     1            + 0.5 * ( dely * rkxy / delx + delx * rky / dely )
c
         hrhs(ind2,1) = hrhs(ind2,1)
     1            - 0.5 * ( dely * rkx / delx - delx * rkyx / dely )
         hrhs(ind2,2) = hrhs(ind2,2)
     1            - 0.5 * ( dely * rkxy / delx - delx * rky / dely )
c
         hrhs(ind3,1) = hrhs(ind3,1)
     1            + 0.5 * ( dely * rkx / delx - delx * rkyx / dely )
         hrhs(ind3,2) = hrhs(ind3,2)
     1            + 0.5 * ( dely * rkxy / delx - delx * rky / dely )
c
         hrhs(ind4,1) = hrhs(ind4,1)
     1            - 0.5 * ( dely * rkx / delx + delx * rkyx / dely )
         hrhs(ind4,2) = hrhs(ind4,2)
     1            - 0.5 * ( dely * rkxy / delx + delx * rky / dely )

 41   continue   

      do 42 i=nvalx-nfxr+1,nvalx
c
c        recherche de la premiere coordonnee dans la grille de permeabilite
c --------------------------------------------------------------------------
c
         indx = ( i + nxhr - 1 ) / nxhr

c
c        calcul de la taille d'un element selon x
c ------------------------------------------------
c
         if(nxh .eq. 1) then
           xvalm = 0.0
           xvalp = 1.0
         else
           if(indx .eq. 1) then
             xvalm = 1.5 * x(1) - 0.5 * x(2)
             xvalp = 0.5 * ( x(indx) + x(indx+1) )
           end if
           if(indx .gt. 1 .and. indx .lt. nxh) then
             xvalm = 0.5 * ( x(indx-1) + x(indx) )
             xvalp = 0.5 * ( x(indx) + x(indx+1) )
           end if
           if(indx .eq. nxh) then
             xvalm = 0.5 * ( x(indx-1) + x(indx) )
             xvalp = 1.5 * x(indx) - 0.5 * x(indx-1)
           end if
         end if
         delx = ( xvalp - xvalm ) / float(nxhr)
c
c        attribution de la permabilite
c -------------------------------------
c
         indprm = indx + ( indy - 1 ) * nxh - (indy -nfy) * nxhf
         rkx = kx(indprm)
         rkxy = kxy(indprm)
         rkyx = kyx(indprm)
         rky = ky(indprm)

c
c     calcul de la contribution de la diagonale pour la cellule
c     
c     calcul des indices des 4 sommets de l'element 
c     en utilisant la propriete de la periodicite
c --------------------------------------------------------------
c
         if (j .eq. nfyr+1) then
            ind1 = i + j2
         else
            ind1 = i + j2-(j-(nfyr+1))*(nvalfx-1)
         end if
         if (i .eq. nvalx) then
            if (j .eq. nfyr+1) then
               ind2=1+j2
            else
               ind2=ind1-(nvalx-nvalfx)
            end if
         else
            ind2 = ind1 + 1
         end if
         if( j .eq. (nvaly-nfyr)) then
           ind3 = ind1 + nvalx
         else
           ind3 = ind1 + (nvalx-nvalfx) + 1 
         end if
         if (i .eq. nvalx) then
            if (j .eq. nvaly-nfyr) then
               ind4 = ind3 - nvalx +1
            else
               ind4=ind3-(nvalx-nvalfx)
            end if
         else
            ind4=ind3+1
         end if

c
c     calcul du vecteur MIA pour ZONE 2 bis
c-------------------------------------------
c
         if (j .ne. nvaly-nfyr) then
            if (i .eq. nvalx-nfxr+1) then
               if (i .eq. nvalx) then
                  mia(ind3+1)=mia(ind3)+4
               end if
               if (i+1 .eq. nvalx) then
                  mia(ind4)=mia(ind3)+3
                  mia(ind4+1)=mia(ind4)+6
               end if
               if (i+2 .le. nvalx) then
                  mia(ind4)=mia(ind3)+3
                  mia(ind4+1)=mia(ind4)+5
               end if
            else
               if (i .ge. nvalx-nfxr+2) then
                  if ((i+2 .ge. nvalx-nfxr+2) 
     *                 .and. (i+1 .le. nvalx-1)) then
                     mia(ind4+1)=mia(ind4)+5
                  end if
                  if (i+1 .eq. nvalx) then
                     mia(ind4+1)=mia(ind4)+6
                  end if
               end if
            end if
         else
            if ((j .eq. nvaly-nfyr) .and. (j .ne. nvaly-1)) then
               if (i .eq. nvalx-nfxr+1) then
                  if (i .eq. nvalx) then
                     mia(ind3+1)=mia(ind3)+5
                  else
                     if (i+1 .eq. nvalx) then
                        mia(ind4)=mia(ind3)+4
                        mia(ind4+1)=mia(ind4)+6
                     end if
                     if (i+2 .le. nvalx) then
                        mia(ind4)=mia(ind3)+4
                        mia(ind4+1)=mia(ind4)+5
                     end if
                  end if
               else
                  if (i+1 .eq. nvalx) then
                     mia(ind4+1)=mia(ind4)+6
                  else
                     if (i+2 .le. nvalx) then
                        mia(ind4+1)=mia(ind4)+5
                     end if
                  end if
               end if
            else
               if ((j .eq. nvaly-nfyr) .and. (j .eq. nvaly-1)) then
                  if (i .eq. nvalx-nfxr+1) then
                     if (i+1 .eq. nvalx) then
                        mia(ind4)=mia(ind3)+7
                     end if
                     if (i+2 .le. nvalx) then
                        mia(ind4)=mia(ind3)+7
                        mia(ind4+1)=mia(ind4)+8
                     end if
                  else
                     if (i+1 .eq. nvalx) then
                        mia(ind4)=mia(ind3)+8
                     end if
                     if (i+2 .le. nvalx) then
                        mia(ind4)=mia(ind3)+8
                        mia(ind4+1)=mia(ind4)+8
                     end if
                  end if
               end if
            end if
         end if
            
c
c     calcul du meme terme pour tous les elements de la diagonale
c ---------------------------------------------------------------
c
         term1 = third * ( dely * rkx / delx + delx * rky / dely )
     1          + quarter * ( rkxy + rkyx )
         term2 = third * ( dely * rkx / delx + delx * rky / dely )
     1          - quarter * ( rkxy + rkyx )

c     calcul du vecteur maa
         maa(mia(ind1+1)-1)=maa(mia(ind1+1)-1)+term1
         maa(mia(ind2+1)-1)=maa(mia(ind2+1)-1)+term2
         maa(mia(ind3+1)-1)=maa(mia(ind3+1)-1)+term2
         maa(mia(ind4+1)-1)=maa(mia(ind4+1)-1)+term1
c     calcul du vecteur mja
         mja(mia(ind1+1)-1)=ind1
         mja(mia(ind2+1)-1)=ind2
         mja(mia(ind3+1)-1)=ind3
         mja(mia(ind4+1)-1)=ind4

c
c     premiere diagonale
c -----------------------
c
         if (ind1 .ge. ind2) then
            call remplissage_maa(ind1,ind2,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely
               mja(first)=ind2
            end if
         else
            call remplissage_maa(ind2,ind1,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
               mja(first)=ind1
            end if
         end if
         
         if (ind3 .ge.ind4) then
            call remplissage_maa(ind3,ind4,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
               mja(first)=ind4
            end if
         else
            call remplissage_maa(ind4,ind3,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely 
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely
               mja(first)=ind3
            end if
         end if
c     
c     seconde diagonale
c -------------------------
c
         if (ind2 .ge. ind3) then 
            call remplissage_maa(ind2,ind3,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx )
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx )
               mja(first)=ind3
            end if
         else
            call remplissage_maa(ind3,ind2,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx ) 
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx )
               mja(first)=ind2
            end if
         end if
         
c     
c     troisieme diagonale
c ---------------------------
c
         if (ind1 .ge. ind3) then
            call remplissage_maa(ind1,ind3,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
               mja(first)=ind3
            end if
         else
            call remplissage_maa(ind3,ind1,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
               mja(first)=ind1
            end if
         end if
         
         if (ind2 .ge. ind4) then
            call remplissage_maa(ind2,ind4,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
               mja(first)=ind4
            end if
         else
            call remplissage_maa(ind4,ind2,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
               mja(first)=ind2
            end if
         end if
c     
c     quatrieme diagonale
c     ---------------------------
c     
         if (ind1 .ge. ind4) then
            call remplissage_maa(ind1,ind4,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
               mja(first)=ind4
            end if
         else
            call remplissage_maa(ind4,ind1,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
               mja(first)=ind1
            end if
         end if 
c     
c     calcul du second membre pour chacun des problemes
c -----------------------------------------------------
c

         hrhs(ind1,1) = hrhs(ind1,1)
     1            + 0.5 * ( dely * rkx / delx + delx * rkyx / dely )
         hrhs(ind1,2) = hrhs(ind1,2)
     1            + 0.5 * ( dely * rkxy / delx + delx * rky / dely )
c
         hrhs(ind2,1) = hrhs(ind2,1)
     1            - 0.5 * ( dely * rkx / delx - delx * rkyx / dely )
         hrhs(ind2,2) = hrhs(ind2,2)
     1            - 0.5 * ( dely * rkxy / delx - delx * rky / dely )
c
         hrhs(ind3,1) = hrhs(ind3,1)
     1            + 0.5 * ( dely * rkx / delx - delx * rkyx / dely )
         hrhs(ind3,2) = hrhs(ind3,2)
     1            + 0.5 * ( dely * rkxy / delx - delx * rky / dely )
c
         hrhs(ind4,1) = hrhs(ind4,1)
     1            - 0.5 * ( dely * rkx / delx + delx * rkyx / dely )
         hrhs(ind4,2) = hrhs(ind4,2)
     1            - 0.5 * ( dely * rkxy / delx + delx * rky / dely )

 42   continue

      end if


      if ((j .ge. (nvaly-nfyr+1)) .and. 
     *    (j.le. nvaly)) then

      do 43 i=1,nvalx
c
c        recherche de la premiere coordonnee dans la grille de permeabilite
c --------------------------------------------------------------------------
c
         indx = ( i + nxhr - 1 ) / nxhr
c
c        calcul de la taille d'un element selon x
c ------------------------------------------------
c
         if(nxh .eq. 1) then
           xvalm = 0.0
           xvalp = 1.0
         else
           if(indx .eq. 1) then
             xvalm = 1.5 * x(1) - 0.5 * x(2)
             xvalp = 0.5 * ( x(indx) + x(indx+1) )
           end if
           if(indx .gt. 1 .and. indx .lt. nxh) then
             xvalm = 0.5 * ( x(indx-1) + x(indx) )
             xvalp = 0.5 * ( x(indx) + x(indx+1) )
           end if
           if(indx .eq. nxh) then
             xvalm = 0.5 * ( x(indx-1) + x(indx) )
             xvalp = 1.5 * x(indx) - 0.5 * x(indx-1)
           end if
         end if
         delx = ( xvalp - xvalm ) / float(nxhr)
c
c        attribution de la permabilite
c -------------------------------------
c
         indprm = indx + ( indy - 1 ) * nxh - ntotf
         rkx = kx(indprm)
         rkxy = kxy(indprm)
         rkyx = kyx(indprm)
         rky = ky(indprm)

c
c     calcul de la contribution de la diagonale pour la cellule
c
c     calcul des indices des 4 sommets de l'element 
c     en utilisant la propriete de la periodicite
c --------------------------------------------------------------
c
         ind1 = i + j2 - ( nvalfx - 1 )*( nvalfy - 1)
         if (i .eq. nvalx) then
            ind2 = ind1 - nvalx + 1
         else
            ind2 = ind1 + 1
         end if
         if (j .eq. nvaly) then
            ind3 = i
         else
            ind3 = ind1 + nvalx
         end if
         if (i .eq. nvalx) then
            if ( j .eq. nvaly ) then
               ind4 = 1
            else
               ind4=ind2 + nvalx
            end if
         else
            ind4 = ind3 + 1
         end if

c
c     calcul du vecteur MIA pour ZONE 3
c---------------------------------------
c
         if ((j .ge. nvaly-nfyr+1) .and. (j .ne. nvaly)) then
            if (j+1 .lt. nvaly) then
               if (i .eq. 1) then
                  mia(ind4)=mia(ind3)+4
                  mia(ind4+1)=mia(ind4)+5
               end if
               if ((i+2 .ge. 2) .and. (i+1 .lt. nvalx)) then
                  mia(ind4+1)=mia(ind4)+5
               else
                  if (i+1 .eq. nvalx) then
                     mia(ind4+1)=mia(ind4)+6
                  end if
               end if
            end if
            if (j+1 .eq. nvaly) then
               if (i .eq. 1) then
                  mia(ind4)=mia(ind3)+7
                  mia(ind4+1)=mia(ind4)+8
               else
                  if ((i+2 .ge. 2) .and. (i+2 .le. nvalx)) then
                     mia(ind4+1)=mia(ind4)+8
                  end if
               end if
            end if
         end if
c
c     calcul du meme terme pour tous les elements de la diagonale
c ---------------------------------------------------------------
c
         term1 = third * ( dely * rkx / delx + delx * rky / dely )
     1          + quarter * ( rkxy + rkyx )
         term2 = third * ( dely * rkx / delx + delx * rky / dely )
     1          - quarter * ( rkxy + rkyx )

c     calcul du vecteur maa
         maa(mia(ind1+1)-1)=maa(mia(ind1+1)-1)+term1
         maa(mia(ind2+1)-1)=maa(mia(ind2+1)-1)+term2
         maa(mia(ind3+1)-1)=maa(mia(ind3+1)-1)+term2
         maa(mia(ind4+1)-1)=maa(mia(ind4+1)-1)+term1
c     calcul du vecteur mja
         mja(mia(ind1+1)-1)=ind1
         mja(mia(ind2+1)-1)=ind2
         mja(mia(ind3+1)-1)=ind3
         mja(mia(ind4+1)-1)=ind4
c
c     premiere diagonale
c -----------------------
c
         if (ind1 .ge. ind2) then
            call remplissage_maa(ind1,ind2,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely
               mja(first)=ind2
            end if
         else
            call remplissage_maa(ind2,ind1,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
               mja(first)=ind1
            end if
         end if
         
         if (ind3 .ge.ind4) then
            call remplissage_maa(ind3,ind4,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              + sixth * delx * rky / dely
               mja(first)=ind4
            end if
         else
            call remplissage_maa(ind4,ind3,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely 
            else
               maa(first)=maa(first)
     1              - third * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              + sixth * delx * rky / dely
               mja(first)=ind3
            end if
         end if
c     
c     seconde diagonale
c     -------------------------
c     
         if (ind2 .ge. ind3) then 
            call remplissage_maa(ind2,ind3,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx )
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx )
               mja(first)=ind3
            end if
         else
            call remplissage_maa(ind3,ind2,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx ) 
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              + quarter * ( rkxy + rkyx )
               mja(first)=ind2
            end if
         end if
         
c     
c     troisieme diagonale
c     ---------------------------
c     
         if (ind1 .ge. ind3) then
            call remplissage_maa(ind1,ind3,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
               mja(first)=ind3
            end if
         else
            call remplissage_maa(ind3,ind1,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
               mja(first)=ind1
            end if
         end if
         
         if (ind2 .ge. ind4) then
            call remplissage_maa(ind2,ind4,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkxy - rkyx )
     1              - third * delx * rky / dely
               mja(first)=ind4
            end if
         else
            call remplissage_maa(ind4,ind2,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
            else
               maa(first)=maa(first)
     1              + sixth * dely * rkx / delx 
     1              + quarter * ( rkyx - rkxy )
     1              - third * delx * rky / dely
               mja(first)=ind2
            end if
         end if
c     
c     quatrieme diagonale
c ---------------------------
c     
         if (ind1 .ge. ind4) then
            call remplissage_maa(ind1,ind4,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
               mja(first)=ind4
            end if
         else
            call remplissage_maa(ind4,ind1,mia,mja,bool,place,first)
            if (bool .eq. 1) then
               maa(place)=maa(place)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
            else
               maa(first)=maa(first)
     1              - sixth * ( dely * rkx / delx + delx * rky / dely )
     1              - quarter * ( rkxy + rkyx )
               mja(first)=ind1
            end if
         end if 
c     
c     calcul du second membre pour chacun des problemes
c -----------------------------------------------------
c
         hrhs(ind1,1) = hrhs(ind1,1)
     1            + 0.5 * ( dely * rkx / delx + delx * rkyx / dely )
         hrhs(ind1,2) = hrhs(ind1,2)
     1            + 0.5 * ( dely * rkxy / delx + delx * rky / dely )
c
         hrhs(ind2,1) = hrhs(ind2,1)
     1            - 0.5 * ( dely * rkx / delx - delx * rkyx / dely )
         hrhs(ind2,2) = hrhs(ind2,2)
     1            - 0.5 * ( dely * rkxy / delx - delx * rky / dely )
c
         hrhs(ind3,1) = hrhs(ind3,1)
     1            + 0.5 * ( dely * rkx / delx - delx * rkyx / dely )
         hrhs(ind3,2) = hrhs(ind3,2)
     1            + 0.5 * ( dely * rkxy / delx - delx * rky / dely )
c
         hrhs(ind4,1) = hrhs(ind4,1)
     1            - 0.5 * ( dely * rkx / delx + delx * rkyx / dely )
         hrhs(ind4,2) = hrhs(ind4,2)
     1            - 0.5 * ( dely * rkxy / delx + delx * rky / dely )

 43   continue

      end if

 4    continue
 

c
c     initialisation des solutions initiales et vecteurs iteratifs
c ----------------------------------------------------------------
c

      do 5 i=1,nmat
         hx(i,1) = 0.0
         hx(i,2)=0.0
         h(i)=0.0
         g(i)=0.0
         cg(i)=0.0
  5   continue
      
c
c     tri des indices
c     ---------------
c
      do 6  i=1,nmat
         call Tri(mja(mia(i)),maa(mia(i)),mia(i+1)-mia(i))
 6    continue
      
c
c     decomposition de choleski incomplet
c -----------------------------------------
c     
      call ICholeski(nmat,maa,mia,mja,mal)

c     
c     resolution du premier probleme
c ------------------------------
c    
      eps=cgeps
      write(*,*)'precision : ',eps
      write(*,*)'Kmax=',kmax
      call gradcpmorse(nmat,maa,mia,mja,mal,hrhs(1,1),hx(1,1)
     *     ,g,h,cg,kmax,eps)

c     
c     reinitialisation
c ------------------------
c     
      do 7 i=1,nmat
         hx(i,2)=0.0
         h(i)=0.0
         g(i)=0.0
         cg(i)=0.0
 7    continue
c
c     resolution du second probleme
c ---------------------------------
c
      eps=cgeps
      
      call gradcpmorse(nmat,maa,mia,mja,mal,hrhs(1,2),hx(1,2)
     *,g,h,cg,kmax,eps)
      
c
c     retour au programme principal sans calcul du tenseur 
c --------------------------------------------------------
c
      if(iopt .ne. 0) return
c
c     calcul final de la moyenne basee sur les elements finis
c
c     initialisation des valeurs
c ------------------------------
c
       zkx = 0.0
       zkxy = 0.0
       zkyx = 0.0
       zky = 0.0
c
c     boucles sur les noeuds
c --------------------------
c
       do 8 j=1,nvaly
          j2 = ( j - 1 ) * nvalx
c
c     recherche de la seconde coordonnee dans la grille de permeabilite
c ---------------------------------------------------------------------
c
          indy = ( j + nyhr - 1 ) / nyhr
c
c     calcul de la taille d'un element selon y
c --------------------------------------------
c
          if(nyh .eq. 1) then
             yvalm = 0.0
             yvalp = 1.0
          else
            if(indy .eq. 1) then
              yvalm = 1.5 * y(1) - 0.5 * y(2)
              yvalp = 0.5 * ( y(indy) + y(indy+1) )
            end if
            if(indy .gt. 1 .and. indy .lt. nyh) then
              yvalm = 0.5 * ( y(indy-1) + y(indy) )
              yvalp = 0.5 * ( y(indy) + y(indy+1) )
            end if
            if(indy .eq. nyh) then
              yvalm = 0.5 * ( y(indy-1) + y(indy) )
              yvalp = 1.5 * y(indy) - 0.5 * y(indy-1)
            end if
          end if
          dely = ( yvalp - yvalm ) / float(nyhr)


         if (j .le. nfyr) then
         
            do 80 i=1,nvalx
c
c     recherche de la premiere coordonnee dans la grille de permeabilite
c ----------------------------------------------------------------------
c
               indx = ( i + nxhr - 1 ) / nxhr
c
c     calcul de la taille d'un element selon x
c --------------------------------------------
c
          if(nxh .eq. 1) then
            xvalm = 0.0
            xvalp = 1.0
         else
            if(indx .eq. 1) then
               xvalm = 1.5 * x(1) - 0.5 * x(2)
               xvalp = 0.5 * ( x(indx) + x(indx+1) )
           end if
           if(indx .gt. 1 .and. indx .lt. nxh) then
              xvalm = 0.5 * ( x(indx-1) + x(indx) )
              xvalp = 0.5 * ( x(indx) + x(indx+1) )
            end if
            if(indx .eq. nxh) then
              xvalm = 0.5 * ( x(indx-1) + x(indx) )
              xvalp = 1.5 * x(indx) - 0.5 * x(indx-1)
            end if
         end if
          delx = ( xvalp - xvalm ) / float(nxhr)
c
c     calcul du volume de la sous cellule en cours
c ------------------------------------------------
c
          del = delx * dely
c
c     attribution de la permeabilite
c ----------------------------------
c
          indprm = indx + ( indy - 1 ) * nxh
          rkx = kx(indprm)
          rkxy = kxy(indprm)
          rkyx = kyx(indprm)
          rky = ky(indprm)
c
c     calcul des indices des 4 sommets de l'element 
c     en utilisant la propriete de la periodicite
c --------------------------------------------------------------
c
          ind1 = i + j2
          if(i .eq. nvalx) then
            ind2 = 1 + j2
          else
            ind2 = ind1 + 1
          end if
          if( j .eq. nvaly) then
            ind3 = i
          else
            ind3 = ind1 + nvalx
          end if
          if(j .eq. nvaly) then
            if(i .eq. nvalx) then
              ind4 = 1
            else
              ind4 = i + 1
            end if
          else
            ind4 = ind2 + nvalx
          end if
c
c     ajout du terme respectif
c ----------------------------
c
          zkx = zkx + del
     *         * ( rkx + rkx * 0.5 * ( hx(ind2,1) - hx(ind1,1) 
     *         + hx(ind4,1) - hx(ind3,1) ) + rkxy * 0.5 * ( hx(ind3,1)
     *         - hx(ind1,1) + hx(ind4,1) - hx(ind2,1) ) )
c
          zkxy = zkxy + del
     *         * ( rkxy + rkx * 0.5 * ( hx(ind2,2) - hx(ind1,2)
     *         + hx(ind4,2) - hx(ind3,2) ) + rkxy * 0.5 * ( hx(ind3,2)
     *         - hx(ind1,2) + hx(ind4,2) - hx(ind2,2) ) )
c
          zkyx = zkyx + del
     *         * ( rkyx + rkyx * 0.5 * ( hx(ind2,1) - hx(ind1,1)
     *         + hx(ind4,1) - hx(ind3,1) ) + rky * 0.5 * ( hx(ind3,1)
     *         - hx(ind1,1) + hx(ind4,1) - hx(ind2,1) ) )
c
          zky = zky + del
     *         * ( rky + rkyx * 0.5 * ( hx(ind2,2) - hx(ind1,2) 
     *         + hx(ind4,2) - hx(ind3,2) ) + rky * 0.5 * ( hx(ind3,2)
     *         - hx(ind1,2) + hx(ind4,2) - hx(ind2,2) ) )
 80    continue

      end if


      if ((j .ge. nfyr +1 ) .and. (j .le. (nvaly-nfyr))) then
         do 81 i=1,nfxr
c
c     recherche de la premiere coordonnee dans la grille de permeabilite
c ----------------------------------------------------------------------
c
          indx = ( i + nxhr - 1 ) / nxhr
c
c     calcul de la taille d'un element selon x
c --------------------------------------------
c
          if(nxh .eq. 1) then
            xvalm = 0.0
            xvalp = 1.0
          else
            if(indx .eq. 1) then
              xvalm = 1.5 * x(1) - 0.5 * x(2)
              xvalp = 0.5 * ( x(indx) + x(indx+1) )
            end if
            if(indx .gt. 1 .and. indx .lt. nxh) then
              xvalm = 0.5 * ( x(indx-1) + x(indx) )
              xvalp = 0.5 * ( x(indx) + x(indx+1) )
            end if
            if(indx .eq. nxh) then
              xvalm = 0.5 * ( x(indx-1) + x(indx) )
              xvalp = 1.5 * x(indx) - 0.5 * x(indx-1)
            end if
         end if
          delx = ( xvalp - xvalm ) / float(nxhr)
c
c     calcul du volume de la sous cellule en cours
c ------------------------------------------------
c
          del = delx * dely
c
c     attribution de la permeabilite
c ---------------------------------
c
         if (( j .ge. nfyr+1) .and. ( j .le. (nfyr + nyhr)) ) then
            indprm = indx + ( indy - 1 ) * nxh
         else
            indprm = indx + ( indy - 1 ) * nxh - (indy-1 -nfy) * nxhf
         end if
          rkx = kx(indprm)
          rkxy = kxy(indprm)
          rkyx = kyx(indprm)
          rky = ky(indprm)

c     
c     calcul des indices des 4 sommets de l'element 
c     en utilisant la propriete de la periodicite
c --------------------------------------------------------------
c
         if ((j .eq. nfyr+1).or. ( j .eq. (nfyr +2 ))) then
            ind1 = i + j2
         else
            ind1 = i + j2-(j-(nfyr+2))*(nvalfx-1)
         end if
         ind2 = ind1 + 1
         if( j .eq. (nfyr + 1)) then
           ind3 = ind1 + nvalx
         else
           ind3 = ind1 + nvalx - ( nvalfx - 1 )
         end if
         ind4=ind3+1
c
c     ajout du terme respectif
c ----------------------------
c
          zkx = zkx + del
     *         * ( rkx + rkx * 0.5 * ( hx(ind2,1) - hx(ind1,1) 
     *         + hx(ind4,1) - hx(ind3,1) ) + rkxy * 0.5 * ( hx(ind3,1)
     *         - hx(ind1,1) + hx(ind4,1) - hx(ind2,1) ) )
c
          zkxy = zkxy + del
     *         * ( rkxy + rkx * 0.5 * ( hx(ind2,2) - hx(ind1,2)
     *         + hx(ind4,2) - hx(ind3,2) ) + rkxy * 0.5 * ( hx(ind3,2)
     *         - hx(ind1,2) + hx(ind4,2) - hx(ind2,2) ) )
c
          zkyx = zkyx + del
     *         * ( rkyx + rkyx * 0.5 * ( hx(ind2,1) - hx(ind1,1)
     *         + hx(ind4,1) - hx(ind3,1) ) + rky * 0.5 * ( hx(ind3,1)
     *         - hx(ind1,1) + hx(ind4,1) - hx(ind2,1) ) )
c
          zky = zky + del
     *         * ( rky + rkyx * 0.5 * ( hx(ind2,2) - hx(ind1,2) 
     *         + hx(ind4,2) - hx(ind3,2) ) + rky * 0.5 * ( hx(ind3,2)
     *         - hx(ind1,2) + hx(ind4,2) - hx(ind2,2) ) )

 81      continue

         do 82 i=nvalx-nfxr+1, nvalx
c
c     recherche de la premiere coordonnee dans la grille de permeabilite
c ----------------------------------------------------------------------
c
          indx = ( i + nxhr - 1 ) / nxhr
c
c     calcul de la taille d'un element selon x
c --------------------------------------------
c
          if(nxh .eq. 1) then
            xvalm = 0.0
            xvalp = 1.0
          else
            if(indx .eq. 1) then
              xvalm = 1.5 * x(1) - 0.5 * x(2)
              xvalp = 0.5 * ( x(indx) + x(indx+1) )
            end if
            if(indx .gt. 1 .and. indx .lt. nxh) then
              xvalm = 0.5 * ( x(indx-1) + x(indx) )
              xvalp = 0.5 * ( x(indx) + x(indx+1) )
             end if
            if(indx .eq. nxh) then
              xvalm = 0.5 * ( x(indx-1) + x(indx) )
              xvalp = 1.5 * x(indx) - 0.5 * x(indx-1)
            end if
         end if
          delx = ( xvalp - xvalm ) / float(nxhr)
c
c     calcul du volume de la sous cellule en cours
c ------------------------------------------------
c
          del = delx * dely
c
c     attribution de la permeabilite
c ----------------------------------
c
          indprm = indx + ( indy - 1 ) * nxh - (indy -nfy) * nxhf
          rkx = kx(indprm)
          rkxy = kxy(indprm)
          rkyx = kyx(indprm)
          rky = ky(indprm)
c
c     calcul des indices des 4 sommets de l'element 
c     en utilisant la propriete de la periodicite
c --------------------------------------------------
c
         if (j .eq. nfyr+1) then
            ind1 = i + j2
         else
            ind1 = i + j2-(j-(nfyr+1))*(nvalfx-1)
         end if
         if (i .eq. nvalx) then
            if (j .eq. nfyr+1) then
               ind2=1+j2
            else
               ind2=ind1-(nvalx-nvalfx)
            end if
         else
            ind2 = ind1 + 1
         end if
         if( j .eq. (nvaly-nfyr)) then
           ind3 = ind1 + nvalx
         else
           ind3 = ind1 + (nvalx-nvalfx) + 1 
         end if
         if (i .eq. nvalx) then
            if (j .eq. nvaly-nfyr) then
               ind4 = ind3 - nvalx +1
            else
               ind4=ind3-(nvalx-nvalfx)
            end if
         else
            ind4=ind3+1
         end if
c
c     ajout du terme respectif
c ----------------------------
c
          zkx = zkx + del
     *         * ( rkx + rkx * 0.5 * ( hx(ind2,1) - hx(ind1,1) 
     *         + hx(ind4,1) - hx(ind3,1) ) + rkxy * 0.5 * ( hx(ind3,1)
     *         - hx(ind1,1) + hx(ind4,1) - hx(ind2,1) ) )
c
          zkxy = zkxy + del
     *         * ( rkxy + rkx * 0.5 * ( hx(ind2,2) - hx(ind1,2)
     *         + hx(ind4,2) - hx(ind3,2) ) + rkxy * 0.5 * ( hx(ind3,2)
     *         - hx(ind1,2) + hx(ind4,2) - hx(ind2,2) ) )
c
          zkyx = zkyx + del
     *         * ( rkyx + rkyx * 0.5 * ( hx(ind2,1) - hx(ind1,1)
     *         + hx(ind4,1) - hx(ind3,1) ) + rky * 0.5 * ( hx(ind3,1)
     *         - hx(ind1,1) + hx(ind4,1) - hx(ind2,1) ) )
c
          zky = zky + del
     *         * ( rky + rkyx * 0.5 * ( hx(ind2,2) - hx(ind1,2) 
     *         + hx(ind4,2) - hx(ind3,2) ) + rky * 0.5 * ( hx(ind3,2)
     *         - hx(ind1,2) + hx(ind4,2) - hx(ind2,2) ) )

 82      continue

      end if


      if ((j .ge. (nvaly-nfyr+1)) .and. (j.le. nvaly)) then
         do 83 i=1, nvalx
c
c     recherche de la premiere coordonnee dans la grille de permeabilite
c ----------------------------------------------------------------------
c
          indx = ( i + nxhr - 1 ) / nxhr
c
c     calcul de la taille d'un element selon x
c --------------------------------------------
c
          if(nxh .eq. 1) then
            xvalm = 0.0
            xvalp = 1.0
          else
            if(indx .eq. 1) then
              xvalm = 1.5 * x(1) - 0.5 * x(2)
              xvalp = 0.5 * ( x(indx) + x(indx+1) )
            end if
            if(indx .gt. 1 .and. indx .lt. nxh) then
              xvalm = 0.5 * ( x(indx-1) + x(indx) )
              xvalp = 0.5 * ( x(indx) + x(indx+1) )
            end if
            if(indx .eq. nxh) then
              xvalm = 0.5 * ( x(indx-1) + x(indx) )
              xvalp = 1.5 * x(indx) - 0.5 * x(indx-1)
            end if
         end if
          delx = ( xvalp - xvalm ) / float(nxhr)
c
c     calcul du volume de la sous cellule en cours
c ------------------------------------------------
c
          del = delx * dely
c
c     attribution de la permeabilite
c ----------------------------------
c
          indprm = indx + ( indy - 1 ) * nxh - ntotf
          rkx = kx(indprm)
          rkxy = kxy(indprm)
          rkyx = kyx(indprm)
          rky = ky(indprm)
c
c     calcul des indices des 4 sommets de l'element 
c     en utilisant la propriete de la periodicite
c ---------------------------------------------------
c
         ind1 = i + j2 - ( nvalfx - 1 )*( nvalfy - 1)
         if (i .eq. nvalx) then
            ind2 = ind1 - nvalx + 1
         else
            ind2 = ind1 + 1
         end if
         if (j .eq. nvaly) then
            ind3 = i
         else
            ind3 = ind1 + nvalx
         end if
         if (i .eq. nvalx) then
            if ( j .eq. nvaly ) then
               ind4 = 1
            else
               ind4=ind2 + nvalx
            end if
         else
            ind4 = ind3 + 1
         end if
c
c     ajout du terme respectif
c ----------------------------
c
          zkx = zkx + del
     *         * ( rkx + rkx * 0.5 * ( hx(ind2,1) - hx(ind1,1) 
     *         + hx(ind4,1) - hx(ind3,1) ) + rkxy * 0.5 * ( hx(ind3,1)
     *         - hx(ind1,1) + hx(ind4,1) - hx(ind2,1) ) )
c
          zkxy = zkxy + del
     *         * ( rkxy + rkx * 0.5 * ( hx(ind2,2) - hx(ind1,2)
     *         + hx(ind4,2) - hx(ind3,2) ) + rkxy * 0.5 * ( hx(ind3,2)
     *         - hx(ind1,2) + hx(ind4,2) - hx(ind2,2) ) )
c
          zkyx = zkyx + del
     *         * ( rkyx + rkyx * 0.5 * ( hx(ind2,1) - hx(ind1,1)
     *         + hx(ind4,1) - hx(ind3,1) ) + rky * 0.5 * ( hx(ind3,1)
     *         - hx(ind1,1) + hx(ind4,1) - hx(ind2,1) ) )
c
          zky = zky + del
     *         * ( rky + rkyx * 0.5 * ( hx(ind2,2) - hx(ind1,2) 
     *         + hx(ind4,2) - hx(ind3,2) ) + rky * 0.5 * ( hx(ind3,2)
     *         - hx(ind1,2) + hx(ind4,2) - hx(ind2,2) ) )

 83      continue
      end if


 8     continue
c
c     division par la taille de la cellule
c ---------------------------------------
c
      zkx =   zkx / cellsize
      zkxy =  zkxy / cellsize
      zkyx =  zkyx / cellsize
      zky =   zky / cellsize

      return
      end
c\end{verbatim}


