c    fichier th.f
c************************************************
c
c    pour compiler taper: f90 -o th th.f -o  
c
c****************************************************************
c            fichiers necessaires au programme principal
c****************************************************************
c       
c       include'./bib/permequi.f'
c       include'./bib/nfr_nint.f'
c       include'./bib/cond_bor.f'
c       include'./bib/2nd_memb.f'
c       include'./bib/aire.f'
c       include'./bib/mise_a_0.f'
c       include'./bib/ecr_vect.f'
c       include'./bib/assemb_a.f'
c       include'./bib/prelimi.f'
c       include'./bib/prelimi1.f'
c       include'./bib/prelimi21.f'
c       include'./bib/prelimi22.f'
c       include'./bib/assemb_c.f'
c       include'./bib/info_ele.f'
c       include'./bib/triangle.f'
c       include'./bib/tr_compl.f'
c       include'./bib/quadrila.f'
c       include'./bib/qu_compl.f'
c       include'./bib/gradcp.f'
c       include'./bib/icholes.f'
c       include'./bib/tri.f'
c       include'./bib/fonct_2m.f'
c       include'./bib/jc.f'

c       include'./bib/norme.f'

c****************************************************************
c                       programme  principal
c****************************************************************        

      program homogeneisation_2d

c    ce programme calcule un tenseur homogeneise suivant la
c    methode de bamberger.
c    la methode de bamberger consiste d'abord a calculer les
c    solutions les deux problemes locaux suivant:
c
c        -div[k(x).grad(pi)]=0    dans le domaine
c                                                              pour i=1,2
c              pi=xi              sur la frontiere du domaine
c
c      xi  :designe le ieme axe de coordonnee
c      pi  :designe la pression
c      k(x):designe le tenseur des permeabilites
c
c
c    l'element generique du tenseur homogeneise est alors donne
c    par le calcul suivant:
c
c     (k)ij=(1/|domaine|)*integrale(grad(pi)*grad(pj))  pour tout 1=< i,j =<2
c
c     variable du programme principal
c    **********************************
c
c
c   i,j,chele :indice de boucle
c   ntn :nombre total de noeud du maillage du domaine
c   nte :nombre total d'element
c   nti :nombre total d'inconnues du systeme lineaire
c   nnf :nombre de noeud situes sur la frontiere
c   nelema :dimension du tableau aa
c   nelemb :dimension du tableau bb
c   nelemc :dimension du tableau cc
c   kmax :nombre maximum d'iteration du solveur gradiant conjugue
c         preconditionne
c   k :tableau 2*2 contenant le tenseur des permeabilites
c   n1,n2,n3,n4 : tableau contenant le numero de l'element ainsi
c                 que les numeros des noeuds qui le constituent
c   aire :aire de la grosse maille
c   eps :precision du solveur
c   perm1,perm2: termes diagonaux des permeabilites lu dans le fichier
c                perm.dat affectes au tableau k
c   elementbis: vecteur de bouleens indiquant la "position" d'un noeud
c                false : noeud situe sur la frontiere
c                true  : noeud situe a l'interieur du domaine
c   ord_gr :dimension du vecteur vect1
c   nbndele: entier qui differencie un element triangulaire d'un element
c            quadrangulaire
c            nbndele=5 :element quadrangulaire
c            nbndele=4 :element triangulaire
c   indice: tableau donnant le rang d'un noeud appartenant a vecti ou vectf
c   nbpoint: nombre de point de la grosse maille
c   aire: aire de la grosse maille
c   tabele: tableau contenant les elements du maillage au travers des numeros
c           des noeuds qui constituent l'element
c   vecti: vecteur contenant les numeros des noeuds situes a l'interieur
c          de la grosse maille
c   vectf: vecteur contenant les numeros des noeuds situes sur la frontiere
c          de la grosse maille
c   aa: tableau contenant les contributions int-int (c'est la matrice a
c       du systeme lineaire)
c   ia(k): indice dans le tableau aa du premier element
c        de la i eme ligne de a stocke dans aa,
c   ja(k): indice j de colonne de l"element aij stocke
c          dans le tableau aa a l"indice k
c   bb: tableau contenant les contributions bord-bord
c   cc: tableau contenant les contributions bord-int
c   ib,jb: idem que ia et ja pour le tableau bb
c   ic,jc: idem que ia et ja pour le tableau cc
c   sminter: vecteur contenant le second membre du systeme lineaire
c            sans les conditions aux limites
c   sm :vecteur sminter avec le sconditions aux limites
c   x1: solution du probleme local 1
c   x2 :solution du probleme local 2
c   g: tableau contenant les coordonnees des points qui constituent la
c      grosse maille
c   coord: tableaux contenants les valeurs des vecteurs directeurs
c   condb1,condb2: tableaux contenant les conditions initiales
c                  sur le bord suivant le probleme local
c   al : tableau contenant la factorise de choleski incomplete de la
c        matrice a stockee sous forme morse dans le tableau aa
c        avec les tableaux auxiliaires d'indices ia et ja
c   gg,h,cg: vecteur necessaire au solveur. ces vecteurs sont expliques
c   dans la subroutine gradcp


      implicit none
c
c declaration des variables
c
      integer i,j,chele,ntn,ord_gr,nte,nti,nnf,nbndele 
      integer lb,indice,nelemb,nelemc,nelema,kmax,nbpoint 
      real*8  k,n1,n2,n3,n4,aire,eps,perm1,poro,poroh,ast   !perm2
      logical elementbis

c      logical b         !variables provisoires
c      real*8 z,y        !idem


      parameter (lb=50) !on peut descendre a 30     

      dimension n1(3),n2(3),n3(3),n4(3)
      dimension k(2,2),elementbis(5),indice(5)
      integer, allocatable, dimension(:,:) :: tabele
      integer, allocatable, dimension(:) :: vecti
      integer, allocatable, dimension(:) :: vectf
      integer, allocatable, dimension(:) :: ia
      integer, allocatable, dimension(:) :: ja
      integer, allocatable, dimension(:) :: ib
      integer, allocatable, dimension(:) :: jb
      integer, allocatable, dimension(:) :: ic
      integer, allocatable, dimension(:) :: jc
      integer, allocatable, dimension(:) :: vect1 

      integer, allocatable, dimension(:,:) :: mata
      integer, allocatable, dimension(:,:) :: matb
      integer, allocatable, dimension(:,:) :: matc

      real*8, allocatable, dimension(:) :: sminter 
      real*8, allocatable, dimension(:) :: sm
      real*8, allocatable, dimension(:) :: x1
      real*8, allocatable, dimension(:) :: x2
      real*8, allocatable, dimension(:,:) :: g
      real*8, allocatable, dimension(:,:) :: coord
      real*8, allocatable, dimension(:) :: condb1
      real*8, allocatable, dimension(:) :: condb2
      real*8, allocatable, dimension(:,:) :: droite
      real*8, allocatable, dimension(:) :: aa
      real*8, allocatable, dimension(:) :: al
      real*8, allocatable, dimension(:) :: bb
      real*8, allocatable, dimension(:) :: cc
      real*8, allocatable, dimension(:) :: gg
      real*8, allocatable, dimension(:) :: h
      real*8, allocatable, dimension(:) :: cg

      data kmax/500/
      data eps/1.d-10/
c
c ouverture des fichiers
c                      
      open(11,file='don/element.dat')
      open(12,file='don/noeud.dat') 
      open(13,file='don/grosmail.dat')
      open(14,file='don/perm.dat')
      open(15,file='don/nbrmail.dat')  
      open(16,status='scratch',
     @                  form='unformatted',access='direct',recl=24) 
      open(17,file='resultats/resultat.dat')

      open(20,file='divers/sys.dat')


c      write(*,*)'        lecture du maillage'
c
c on lit le nombre total de noeud et le nombre total d'element du maillage 
c     
      read(15,*)ntn
      read(15,*)nte
c
c allocation dynamique de memoire des tableaux vecti, vectf et tabele 
c      
      allocate (vecti(ntn))
      allocate (vectf(ntn))
      allocate (tabele(nte,5))
c
c on verifie que le nombre de point de la maille grossiere est superieur a 2
c
      read(13,*)nbpoint
c      if (nbpoint.le.2) then
c                 write(*,*)'erreur sur les donnees de la grosse maille'
c                 stop
c      endif
c
c allocation dynamique de memoire des tableaux g, coord et droite
c
      allocate (g(nbpoint,2))
      allocate (coord(nbpoint,2))
      allocate (droite(nbpoint,2))
c
c lecture des coordonnees des points qui constituent la grosse maille
c
      do i=1,nbpoint
         read(13,*)(g(i,j),j=1,2)
      enddo
c
c initialisation des variables nti et nnf
c
      nti=1
      nnf=1
c
c calcul des equations des droites qui constituent la maille grossiere
c
      call equations(g,coord,nbpoint,aire)
c      write(*,*)'aire1',aire
      aire=0.d0


c
c on parcourt tous les noeuds du maillage que l'on affecte a vecti ou
c vectf suivant leur position
c
      do j=1,ntn
         read(12,*)(n1(i),i=1,3)
         call remplir (nbpoint,coord,g,vecti,vectf,droite,
     @                        nti,nnf,n1(2),n1(3),n1(1))
      enddo   
      nti=nti-1
      nnf=nnf-1

c      write(*,*)'attention les vecti et les vectf doivent etre '
c      write(*,*)'ranges dans l'ordre croissant'

c      write(*,*)'vecti' 
c      write(*,*)(vecti(i),i=1,nti)
c      write(*,*)'vectf' 
c      write(*,*)(vectf(i),i=1,nnf)

c
c on rend la memoire des tableaux g, droite et coord au systeme
c      
      deallocate(g)
      deallocate(droite)
      deallocate(coord)


c      write(*,*)'rangements des elements du maillage dans un tableaux.'
c
c on range les elements du maillage dans le tableau tabele
c
      rewind(11)
      do i=1,nte
         read(11,*)(tabele(i,j),j=1,5)
c         write(20,*)(tabele(i,j),j=1,5)
      enddo
      ord_gr=nti*nnf+27
   
c      write(*,*)'ord_gr',ord_gr

c      do i=1,ntn
c       read(16,rec=i)z,y,b,j
c       write(20,*)z,y,b,j
c      enddo

      goto 12

c
c allocation dynamique de memoire des tableaux vect1,ia,ib,ic
c 
c      allocate (vect1(ord_gr))
c      allocate (ia(nti+1)) 
c      allocate (ic(nnf+1))
c      allocate (ib(nnf+1))
c
c      write(*,*)'calcul du nombre total d''elements de la matrice et '
c      write(*,*)'calcul des tableaux ia,ib,ic.' 
c
c calcul des tableaux ia,ib,ic et calcul de la dimension des 
c tableaux aa,bb,cc 
c
c      write(*,*)'66'
c      write(20,*)'66'
c
c      call preliminaire (vectf,vecti,tabele,nnf,nti,nte,ic,nelemc,vect1,
c     @                ib,nelemb,ia,nelema)
c
c      write(*,*)'77'
c
c allocation dynamique de memoire des tableaux cc,jc
c
c      allocate (cc(nelemc))
c      allocate (jc(nelemc))
c
c calcul du tableau ic et initialisation du tableau cc
c
c      call cal_jc (vect1,jc,nelemc,nnf,cc)
c
c on rend la memoire du tableau vect1 au systeme
c
c      deallocate(vect1)                                                    
c
c
c      goto 15
12    continue
      
c      write(*,*)'nouvaut'
c      write(*,*)'les tailles sont:',nti*lb,nnf*lb,nnf*lb

      allocate(mata(nti,lb))
      allocate(matb(nnf,lb))
      allocate(matc(nnf,lb))
      allocate (ia(nti+1))

c      write(*,*)'toto1'

      call preliminaire21(vectf,vecti,tabele,nnf,nti,nte,ia,
     @                         mata,matb,matc,ord_gr,lb)

c      write(*,*)'toto2'

      deallocate(mata)
      allocate (vect1(ord_gr))
      allocate (ic(nnf+1))
      allocate (ib(nnf+1))

c      write(*,*)'toto3'
      call preliminaire22(vectf,vecti,tabele,nnf,nti,nte,ic,nelemc,
     @                      vect1,ib,nelemb,ia,nelema,matb,matc)

c      write(*,*)'je que c est ici'
c      pause
      deallocate(matb)
c      write(*,*)'yyyyyyyyy'

      deallocate(matc)
c
c allocation dynamique de memoire des tableaux cc,jc
c
      allocate (cc(nelemc))
      allocate (jc(nelemc))
c
c calcul du tableau ic et initialisation du tableau cc
c
      call cal_jc (vect1,jc,nelemc,nnf,cc)
c
c on rend la memoire du tableau vect1 au systeme
c
      deallocate(vect1)

15    continue


c
c allocation dynamique de memoire des tableaux aa,bb,ja,jb,al,sminter
c      
      allocate (aa(nelema))
      allocate (al(nelema))
      allocate (ja(nelema)) 
      allocate (bb(nelemb))
      allocate (jb(nelemb))
      allocate (sminter(nti))

c      write(*,*)'calcul des contributions et assemblage'
c      write(20,*)'calcul des contributions et assemblage'
c
c initialisation de tous les tableaux a zero
c            
      call mise_a_zero(bb,jb,aa,ja,k,sminter,nti,nelema,nelemb,poroh)

c      do i=1,nelemb
c         write(20,*)i,'bb=',bb(i),'    jb=',jb(i)
c      enddo
c      write(20,*)
c      do i=1,nelema
c         write(20,*)i,'aa=',aa(i),'    ja=',ja(i)
c      enddo
c      write(20,*)
c      do i=1,nelemc
c         write(20,*)i,'cc=',cc(i),'    jc=',jc(i)
c      enddo

      
      rewind(14)
      do chele=1,nte
c
c on lit la permeabilite de l'element
c
         read(14,*)perm1      !perm2,poro

c         ast=perm1**2
c         perm1=perm1/ast

         poro=0.1d0
         k(1,1)=perm1
         k(2,2)=perm1         !k(2,2)=perm2
c
c on donne les informations de l'element
c
         call coord_t_f(tabele,elementbis,indice,nbndele,
     @                  n1,n2,n3,n4,nte,chele)
         if (nbndele.eq.5) then   
c
c calcul de la contribution et assemblage pour un element quadrangulaire
c
             call calculq1(elementbis,indice,bb,ib,jb,aa,ia,ja,sminter
     @                    ,n1,n2,n3,n4,k,cc,ic,jc,aire,poro,poroh)
                          else               
c
c calcul de la contribution et assemblage pour un element triangulaire
c
             call calculp1(elementbis,indice,bb,ib,jb,aa,ia,ja,sminter
     @                     ,n1,n2,n3,k,cc,ic,jc,aire,poro,poroh)
         endif
      enddo
c      write(*,*)
c      write(*,*)'aire2',aire

c      write(*,*)'constitution des 3 systemes lineaires et resolution'
      write(20,*)'constitution des 3 systemes lineaires et resolution'
      do i=1,nelemc
         write(20,*)i,'cc=',cc(i),'    jc=',jc(i)
      enddo
c
c on tri le tableau aa
c                 
      do i=1,nti
        call tri(ja(ia(i)),aa(ia(i)),ia(i+1)-ia(i))
      enddo
c
c on tri le tableau bb
c
      do i=1,nnf
        call tri(jb(ib(i)),bb(ib(i)),ib(i+1)-ib(i))
      enddo

      do i=1,nelemb
         write(20,*)i,'bb=',bb(i),'    jb=',jb(i)
      enddo
      do i=1,nelema
         write(20,*)i,'aa=',aa(i),'    ja=',ja(i)
      enddo
c
c  calcul de la factorise de choleski incomplete
c
      call icholeski(nti,aa,ia,ja,al)

      write(20,*)'factorise de choleski'
      do i=1,nelema
         write(20,*)'al=',al(i),'    ja=',ja(i)
      enddo
c
c on rend la memoire du tableau tabele au systeme
c
      deallocate(tabele)
c
c allocation dynamique de memoire des tableaux sm,condb1,condb2,x1,x2
c
      allocate (sm(nti))
      allocate (condb1(nnf))
      allocate (condb2(nnf))
      allocate (x1(nti))
      allocate (x2(nti))
c
c on initialise a zero le vecteur x1 et on affecte le vecteur sminter au 
c vecteur sm
c      
      do i=1,nti
         sm(i)=sminter(i)
c         write(20,*)sm(i) 
         x1(i)=0.d0
      enddo
c
c on affecte les conditions aux limites du probleme local 1 au vecteur sm
c
      call second_memb(cc,ic,jc,condb1,condb2,sm,nnf,
     @                 nti,vectf,1) 

      write(20,*)
      write(20,*)'sm1'
      do i=1,nti
         write(20,*)i,'  ',sm(i)
      enddo
c
c allocation dynamique de memoire des tableaux gg,h,cg
c           
      allocate (gg(nti))
      allocate (h(nti))
      allocate (cg(nti))
c
c on resout le systeme lineaire relatif au probleme local 1
c
      call gradc(nti,aa,ia,ja,al,sm,x1,gg,h,cg,kmax,eps)
      call ecrire(x1,nti)
      
      write(20,*)'fin de la premiere solution'      
c
c on initialise a zero le vecteur x2 et on affecte le vecteur sminter au
c vecteur sm
c
      do i=1,nti
         sm(i)=sminter(i)
         x2(i)=0.d0
      enddo
c
c on affecte les conditions aux limites du probleme local 2 au vecteur sm
c
      call second_memb(cc,ic,jc,condb1,condb2,sm,nnf,
     @                 nti,vectf,2) 
c
c on resout le systeme lineaire relatif au probleme local 2
c      
      call gradc(nti,aa,ia,ja,al,sm,x2,gg,h,cg,kmax,eps)
      call ecrire(x2,nti)
c
c on rend la memoire du tableau gg,h,cg
c
      deallocate (gg)
      deallocate (h)
      deallocate (cg)

c      call norme(vecti,vectf,x,nti,nnf)

c      write(*,*)'calcul du tenseur homogeneise'

c
c calcul du tenseur homogeneise
c

      call permeaequi(aa,ia,ja,x1,x2,nti,nnf,aire,condb1,condb2,
     @               bb,ib,jb,cc,ic,jc,poroh)
c
c fermeture des fichiers
c
      close(11)
      close(12)
      close(13)
      close(14)
      close(15)
      close(16)
      close(17)    
      close(20)
c      write(*,*)'nombre d''inconnues du systeme :',nti
c      write(*,*)
c      write(*,*)'fin de l''application'
      end


c  fichier 2mb_qua.f
c
c**************************************************************
      real*8 function integrpq(nbr1,n1,n2,n3,n4)           
c**************************************************************
c
c  cette fonction calcule l'approximation de l'integrale de
c  (f*fonction de base) sur un quadrilatere.
c  on utilise la formule p.??? du  livre 'presentation de la methode
c  des elements finis' (t.touzot) 19??.
c
c  n1,n2,n3,n4: contient les numeros, ainsi que les coordonnees
c               des quatre points qui constituent le quadrilatere.
c  nbr1: numero designant la nbr1ieme fonction de base du
c        quadrilatere.
c 

      implicit none

      integer   nbr1
      real*8    somme1,basec,n1,n2,n3,n4,det,a,fonct,x,y
      dimension n1(*),n2(*),n3(*),n4(*),basec(4)

      a=1.d0/sqrt(3.d0)
      
      call fbasec(basec,det,n1,n2,n3,n4,a,a,x,y) 
      somme1=(fonct(x,y)*basec(nbr1))           
      call fbasec(basec,det,n1,n2,n3,n4,-a,a,x,y) 
      somme1=somme1+(fonct(x,y)*basec(nbr1))            
      call fbasec(basec,det,n1,n2,n3,n4,a,-a,x,y) 
      somme1=somme1+(fonct(x,y)*basec(nbr1))             
      call fbasec(basec,det,n1,n2,n3,n4,-a,-a,x,y) 
      somme1=somme1+(fonct(x,y)*basec(nbr1))           
      
      integrpq=somme1*abs(det)     

      return
      end


c fichier 2mb_tri.f
c
c******************************************************************
      real*8 function integrp(det,n1,n2,n3,nbr1)
c******************************************************************
c
c  cette fonction calcule l'approximation de l'integrale de 
c  (f*fonction de base) sur le triangle.
c  on utilise la formule p.??? du  livre 'presentation de la methode 
c  des elements finis' (t.touzot) 19??.
c
c  det: jacobien de la transformation: triangle de reference --> triangle 
c   n1,n2,n3 : tableau contenant le numero de l'element ainsi
c              que les numeros des noeuds qui le constituent
c  nbr1: numero designant la nbr1ieme fonction de base du
c        triangle.
c
c
      implicit none

      integer   nbr1
      real*8    det,basep,somme,fonct,a,b,x,y,n1,n2,n3
      dimension basep(3),n1(*),n2(*),n3(*)

      a=(1.d0/6.d0)
      b=(2.d0/3.d0)

      call fbase(basep,n1,n2,n3,a,a,x,y)
      somme=fonct(x,y)*basep(nbr1)              
      call fbase(basep,n1,n2,n3,b,a,x,y)
      somme=somme+fonct(x,y)*basep(nbr1)       
      call fbase(basep,n1,n2,n3,a,b,x,y)
      somme=somme+fonct(x,y)*basep(nbr1)        
      integrp=somme*abs(det)*a

      return
      end









c  fichier 2nd_memb.f
c
c********************************************************************
      subroutine second_memb(cc,ic,jc,condb1,condb2,sm,nnf,nti,
     @                       vectf,numero)
c*********************************************************************
c
c   cette procedure ajoute aux second memebre les conditions aux limites
c   suivant le probleme local que l'on veut resoudre. deplus des contions
c   initiales sont sauvegardees dans un tableau en vue du calcul 
c   tenseur homogeneise.
c
c   condb1,condb2: tableaux contenant les conditions initiales
c                  sur le bord suivant le probleme local
c   sm: vecteur second membre avec conditions aux limites    
c   nnf: nombre de noeud a la frontiere
c   vectf: tableau contenant les numero des noeuds situe sur la 
c          frontiere du domaine 
c   numero: numero du probleme local resolu
c   cc: tableau des contributions bord_int
c   ic,jc: tableaux d'indices associes au tableau cc
c   nti :nombre total d'inconnues du systeme lineaire
c
      implicit none

      integer   nnf,k1,k2,j,vectf,numero,ic,jc,kk,nti
      real*8    condb1,condb2,sm,bord_pb1,bord_pb2,cc  
      dimension cc(*),ic(*),jc(*),sm(*) 
      dimension condb1(*),condb2(*),vectf(*)
      
      kk=1
c
c probleme local 1
c
      if (numero.eq.1) then 
         do k2=1,nnf
            condb1(k2)=bord_pb1(vectf(k2))
            do j=1,ic(k2+1)-ic(k2)
               k1=jc(kk)
c            write(20,*)'cc',cc(kk),'     ',kk,'  sm',sm(k1)
                 sm(k1)=-condb1(k2)*cc(kk)+sm(k1)
               kk=kk+1
            enddo
         enddo
             else
c
c probleme local 2
c
        if (numero.eq.2) then
          do k2=1,nnf
            condb2(k2)=bord_pb2(vectf(k2))
            do j=1,ic(k2+1)-ic(k2)
               k1=jc(kk)
                   sm(k1)=-condb2(k2)*cc(kk)+sm(k1)
               kk=kk+1
            enddo
          enddo
         endif
      endif

      return
      end





c  fichier aire1.f
c
c*****************************************************************
      subroutine aire_ele1(aire,n1,n2,n3)
c*****************************************************************
c
c   cette procedure calcule l'aire du triangle n1,n2,n3 
c
c   n1,n2,n3: contient les numeros, ainsi que les coordonnees des
c                trois points qui constituent le triangle
c   aire: aire du triangle
c
      implicit none
      
      real*8    aire,det,n1,n2,n3,j11,j12,j21,j22 
      dimension n1(*),n2(*),n3(*) 
c
c calcul du jacobien de la transformation
c      
      j11=(n2(2)-n1(2)) 
      j21=(n2(3)-n1(3))
      j12=(n3(2)-n1(2))
      j22=(n3(3)-n1(3))                     
      det=(j11*j22)-(j12*j21)
c
c calcul de l'aire     
c
      aire=(abs(det))/2.d0

      return
      end                                        






c  fichier aire2.f
c
c*****************************************************************
      subroutine aire_ele2(aire,n1,n2,n3)
c*****************************************************************
c
c   cette procedure calcule l'aire du triangle n1,n2,n3 
c
c   n1,n2,n3: contient les numeros, ainsi que les coordonnees des
c                trois points qui constituent le triangle
c   aire: aire du triangle
c
      implicit none
      
      real*8    aire,det,n1,n2,n3,j11,j12,j21,j22 
      dimension n1(*),n2(*),n3(*) 
c
c calcul du jacobien de la transformation
c      
      j11=(n2(2)-n1(2)) 
      j21=(n2(3)-n1(3))
      j12=(n3(2)-n1(2))
      j22=(n3(3)-n1(3))                     
      det=(j11*j22)-(j12*j21)
c
c calcul de l'aire     
c
      aire=(abs(det))/2.d0

      return
      end                                        






c    fichier aire.f
c
c**********************************************************************
      subroutine equations(g,coord,nbpoint,aire) 
c**********************************************************************
c
c   cette procedure calcule les vecteurs directeurs des droites
c   qui forment la grosse maille, ainsi que l'aire de cette derniere.
c
c   g: tableau contenant les coordonnees des points de la 
c      grosse maille
c   coord: tableaux contenants les valeurs des vecteurs directeurs
c   nbpoint :nombre de point de la grosse maille
c   aire: aire de la grosse maille
c
c
      implicit none

      integer   i,nbpoint,k
      real*8    g,coord,aire,p,l1,l2,l3,a1
      dimension g(nbpoint,2),coord(nbpoint,2)

c
c calcul des coordonnees des vecteurs directeurs
c
      do i=1,nbpoint-1 
         coord(i,1)=g(i+1,1)-g(i,1)
         coord(i,2)=g(i+1,2)-g(i,2) 
      enddo
      coord(nbpoint,1)=g(1,1)-g(nbpoint,1)
      coord(nbpoint,2)=g(1,2)-g(nbpoint,2) 

      do i=1,nbpoint
         do k=1,2
            if (abs(coord(i,k)).le.1.d-05) then
                                     coord(i,k)=0.d0
            endif
         enddo
      enddo
c
c calcul de l'aire de la grosse maille
c
      aire=0.d0
      do i=1,nbpoint-2
           l1=sqrt(((g(i+2,1)-g(1,1))**2)+((g(i+2,2)-g(1,2))**2))
           l2=sqrt(((g(i+1,1)-g(1,1))**2)+((g(i+1,2)-g(1,2))**2))
           l3=sqrt(((g(i+2,1)-g(i+1,1))**2)+((g(i+2,2)-g(i+1,2))**2))
           p=(l1+l2+l3)/2.d0
           a1=sqrt(p*(p-l1)*(p-l2)*(p-l3))
           aire=aire+a1 
      enddo
      
      return
      end


                                               
c  fichier assemb_a.f
c
c****************************************************************
      subroutine assemblage(aa,ia,ja,i,j,som1)
c****************************************************************
c
c  cette procedure effectue l'assemblage du tableau aa qui
c  contient les contributions int-int.
c
c  aa: tableau contenant les contributions int-int
c  ia,ja: tableaux d'indices associes au tableau aa
c  i,j: indices ou l'on doit effectuer l'assemblage
c  som1: contribution a assembler
c
c
      implicit none

      integer   ia,ja,i,j,k,l
      real*8    aa,som1
      dimension aa(*),ia(*),ja(*)

      k=0
      if (i.eq.j) then
         k=ia(i+1)-1
         ja(k)=i
                  else
         if (i.gt.j) then
            do l=ia(i),ia(i+1)-2
               if (ja(l).eq.0) then
                               goto 4
               endif
               if (ja(l).eq.j) then
                               k=l
                               goto 4
               endif
            enddo
4           continue
            if (k.eq.0) then 
                        k=l
                        ja(l)=j
            endif
         endif
      endif
      if (k.ne.0) then
               aa(k)=aa(k)+som1
      endif

      return
      end
c  fichier assemb_c.f
c
c****************************************************************
      subroutine assemblagebis(cc,ic,jc,i,j,som1)
c****************************************************************
c
c  cette procedure effectue l'assemblage du tableau cc qui
c  contient les contributions bord-int.
c
c  cc: tablecu contenant les contributions bord-int
c  ic,jc: tableaux d'indices associes au tableau cc 
c  i,j: indices ou l'on doit effectuer l'assemblcge
c  som1: contribution a assembler
c
c
      implicit none

      integer   ic,jc,i,j,k,l,deb
      real*8    cc,som1
      dimension cc(*),ic(*),jc(*)


      deb=ic(i)
      do l=deb,deb+(ic(i+1)-ic(i)-1)
         if (jc(l).eq.j) then
                           k=l
                           goto 5
         endif
      enddo
c      write(*,*)'probleme dcns assemb_c.f'
                    stop
5     continue
      cc(k)=cc(k)+som1


      return
      end
c  fichier base_qua.f
c
c************************************************************
      subroutine fbasec(basec,det,n1,n2,n3,n4,xch,ych,x,y)
c************************************************************
c
c   cette procedure calcule la valeur des quatres fonctions de bases
c   au point (xch,ych) appartenant au carre de reference, ainsi que
c   le jacobien de l'application: carre de reference --> quadrilatere.
c
c   basec: tableau contenant les valeurs des quatres fonctions de 
c          base au point (xch,ych)
c   det: jacobien de la transformation: carre de reference --> quadrilatere
c   n1,n2,n3,n4: contient les numeros, ainsi que les coordonnees des 
c                quatre points qui constituent le quadrilatere
c   xch,ych: coordonnees du point de calcul dans le carre de reference
c   x,y: coordonnees du point dans le quadrilatere
c
c
      implicit none

      real*8    basec,xch,ych,det,n1,n2,n3,n4
      real*8    j11,j12,j21,j22,a1,a2,b1,b2,x,y
      dimension basec(*),n1(*),n2(*),n3(*),n4(*)
      
      a1=(1.d0+xch)
      a2=(1.d0-xch)
      b1=(1.d0+ych)
      b2=(1.d0-ych)
c
c fonctions de bases
c
      basec(1)=(a2*b2)/4.d0
      basec(2)=(a1*b2)/4.d0
      basec(3)=(a1*b1)/4.d0
      basec(4)=(a2*b1)/4.d0
c
c coordonnees reelle du point de calcul
c
      x=(basec(1)*n1(2)+basec(2)*n2(2)+basec(3)*n3(2)+basec(4)*n4(2))
      y=(basec(1)*n1(3)+basec(2)*n2(3)+basec(3)*n3(3)+basec(4)*n4(3))
c
c calcul du jacobien de la transformation
c
      j11=-n1(2)+n2(2)+n3(2)-n4(2)+ych*(n1(2)-n2(2)+n3(2)-n4(2))
      j11=j11/4.d0

      j12=-n1(3)+n2(3)+n3(3)-n4(3)+ych*(n1(3)-n2(3)+n3(3)-n4(3)) 
      j12=j12/4.d0

      j21=-n1(2)-n2(2)+n3(2)+n4(2)+xch*(n1(2)-n2(2)+n3(2)-n4(2))
      j21=j21/4.d0

      j22=-n1(3)-n2(3)+n3(3)+n4(3)+xch*(n1(3)-n2(3)+n3(3)-n4(3))
      j22=j22/4.d0

      det=(j11*j22)-(j21*j12)
               
      return
      end                                        

c  fichier base_tri.f
c
c********************************************************************
      subroutine fbase(basep,n1,n2,n3,xch,ych,x,y)
c********************************************************************
c
c  cette procedure calcule les fonctions de base pour le 
c  triangle de reference.
c
c   basep: tableau contenant les valeurs des fonctions de bases
c   n1,n2,n3 : tableau contenant le numero de l'element ainsi
c              que les numeros des noeuds qui le constituent
c   xch,ych: coordonnees du point de calcul dans le triangle de reference 
c   x,y: coordonnees du point dans le triangle
c
c

      implicit none

      real*8    xch,ych,x,y,basep,n1,n2,n3
      dimension basep(*),n1(*),n2(*),n3(*)
c
c fonctions de bases 
c
      basep(1)=(1.d0-xch-ych)
      basep(2)=xch
      basep(3)=ych
c
c coordonnees reelle du point de calcul
c
      x=(basep(1)*n1(2)+basep(2)*n2(2)+basep(3)*n3(2))
      y=(basep(1)*n1(3)+basep(2)*n2(3)+basep(3)*n3(3))

      return
      end
c  fichier cond_bor.f
c
c*********************************************************************
      real*8 function bord_pb1(r)
c*********************************************************************
c 
c  cette fonction donne les conditions aux bord du probleme local 1
c 
c  r: numero du noeud situe a la frontiere du domaine
c
c
      implicit none

      real*8   x,y 
      integer  r

      read(16,rec=r)x,y
      bord_pb1=x                    !+(y*0.d0)

c      bord_pb1=80*x+25.d0*y+50.d0                    !+(y*0.d0)

      return
      end





c*********************************************************************      
      real*8 function bord_pb2(r)
c*********************************************************************
c 
c  cette fonction donne les conditions aux bord du probleme local 2
c 
c  r: numero du noeud situe a la frontiere du domaine
c
c

      implicit none

      real*8  x,y
      integer r

      read(16,rec=r)x,y
      bord_pb2=y                  !+(x*0.d0)


c      bord_pb2=3*x+68*y+10.d0                  !+(x*0.d0)   

      return
      end



c  fichier ecr_vect.f
c
c********************************************************************
        subroutine ecrire(b,n)
c********************************************************************
c
c    ce sous-programme ecrit dans un fichier et/ou a l'ecran le 
c    vecteur b en colonne.
c  
c    b: vecteur
c    n: dimension du vecteur b
c

        implicit none 

        integer i,n
        real*8  b(*)

        write(20,*)
        do i=1,n
          write(20,*) i,b(i)
        enddo

        return
        end



c   fichier essai.f
c
c*************************************************************************
       subroutine  permeq2(y1,y2,y3)  
c************************************************************************

       implicit none

       real*8 det,j,y1,y2,y3,kxx,kxy,kyy 
       real*8 a11,a12,a13,a21,a22,a23,a31,a32,a33
       real*8 alpha1,beta1,gamma1,alpha2,beta2,gamma2

       dimension j(3,3)

       write(20,*)y1,y2,y3



       alpha1=80.d0
       beta1=25.d0
c       gamma1=1.d0

       alpha2=3.d0
       beta2=68.d0
c       gamma2=1.d0


       j(1,1)=alpha1**2    
       j(2,1)=alpha1*alpha2 
       j(3,1)=alpha2**2  

       j(1,2)=2.d0*alpha1*beta1
       j(2,2)=alpha1*beta2+alpha2*beta1
       j(3,2)=2.d0*alpha2*beta2

       j(1,3)=beta1**2
       j(2,3)=beta1*beta2
       j(3,3)=beta2**2

       
       a11=j(2,2)*j(3,3)-j(3,2)*j(2,3)
       a21=j(3,1)*j(2,3)-j(2,1)*j(3,3)
       a31=j(2,1)*j(3,2)-j(3,1)*j(2,2)

       a12=j(1,3)*j(3,2)-j(1,2)*j(3,3)
       a22=j(1,1)*j(3,3)-j(1,3)*j(3,1)
       a32=j(1,2)*j(3,1)-j(3,2)*j(1,1)

       a13=j(1,2)*j(2,3)-j(1,3)*j(2,2)
       a23=j(2,1)*j(1,3)-j(2,3)*j(1,1)
       a33=j(1,1)*j(2,2)-j(1,2)*j(2,1)

       det=j(1,1)*a11+j(1,2)*a21+j(1,3)*a31      

       det=1.d0/det

       a11=a11*det
       a21=a21*det
       a31=a31*det

       a12=a12*det
       a22=a22*det
       a32=a32*det

       a13=a13*det
       a23=a23*det
       a33=a33*det
 
       kxx=a11*y1+a12*y2+a13*y3
       kxy=a21*y1+a22*y2+a23*y3
       kyy=a31*y1+a32*y2+a33*y3

c       write(*,19)
c       write(*,20)kxx,kxy,kyy


19    format(5x,'kxx',12x,'kxy',12x,'kyy')
20    format(4(e11.4,4x))



       return
       end

c  fichier fonct_2m.f
c
c*************************************************************************
      real*8 function fonct(x,y)
c*************************************************************************
c
c  cette fonction donne la valeur de la fonction f au
c  point (x,y).
c
c  x,y : coordonnee du point de calcul
c
c
      implicit none

      real*8 x,y,pi

      data pi/3.1415926000000/
      fonct=((x+y)*0.d0)

c       fonct=-(y-4.d0*x*y-y**2+4.d0*x*y**2-2.d0*y**2+2.d0*y**3)
c     @       -(x-4.d0*y*x-x**2+4.d0*y*x**2-2.d0*x**2+2.d0*x**3)
c       fonct=2.d0*x*(1.d0-x)+2.d0*y*(1.d0-y)
c      fonct=-3.d0*(x**2)/2.d0
c       fonct=dsin(x*y)*(x**2+y**2)
c        fonct=10.d0*pi**2*dsin(pi*x)*dsin(pi*y)

      return
      end
c fichier gradcp.f
c
c********************************************************************
c            fichiers necessaires a la subroutine gradc
c********************************************************************

c      include'./prod_mat.f'
c      include'./resol_sys.f'

c***********************************************************************
      subroutine gradc(n,aa,ia,ja,al,b,x,g,h,cg,kmax,eps)
c***********************************************************************
c
c ce sous-programme resout le systeme lineaire ax=b par
c gradient conjugue preconditionne, avec au plus kmax 
c iterations. le test d"arret est: |ax^k-b|<eps|ax^0-b|.
c la matrice a du systeme lineaire, de taille n*n
c est stockee sous forme morse dans le tableau aa;
c ce stockage necessite aussi les tableaux auxiliaires
c d"indices ia et ja.
c la matrice c de preconditionnement est la factorisation
c de choleski incomplete de a (c=l^* (l^*)^{t})  et la
c matrice l^* est stockee sous forme morse dans le 
c tableau al, avec les memes tableaux d"indices ia et 
c ja que a.
c
c  aa: matrice a du systeme stockee sous forme morse avec 
c      comme tableau d'indices ia et ja
c  al: factorisee de choleski incomplete de a avec 
c      comme tableau d'indices ia et ja 
c  b: second membre du systeme lineaire
c  kmax: nombre maximal d"iterations
c  eps: precision du calcul
c  x:tableau contenant les valeurs de la solution 
c  gcg:scalaire contenant le carre de la norme de
c      g (en pratique ce nombre doit decroitre au 
c         cours des iterations).
c  g, cg: tableaux contenant les valeurs de g^n et
c         de c g^n respectivement;
c  h: tableau contenant la direction de descente h^n
c
c
      implicit none
      integer n,ia,ja,kmax,i,k
      real*8  aa,al,b,x,g,h,cg,gcg,hah,ro,gamma,eps

      dimension x(*),b(*),g(*),h(*),cg(*)
      dimension aa(*),ja(*),ia(*),al(*)

      do  k=1, kmax

        if (k.eq.1) then
          call iamul(n,aa,ia,ja,x,h)
          do  i=1,n
            cg(i)=b(i)-h(i)
            g(i)=cg(i)
          enddo
          call isolvsys(n,g,al,ia,ja)
          gcg=0.d0
          do  i=1,n
            gcg=gcg + g(i)*cg(i)
            h(i)=g(i)
          enddo
        endif

        call iamul(n,aa,ia,ja,h,b)
        hah=0.d0
        do i=1,n		
          hah=hah+h(i)*b(i)
        enddo
        ro=gcg/hah
        do  i=1,n
          cg(i) = cg(i) -ro*b(i)
        enddo
        call isolvsys(n,b,al,ia,ja)
        do  i=1,n
          x(i)=x(i)+ro*h(i)
          g(i)=g(i)- ro*b(i)
        enddo
        gamma=gcg
        gcg=0.d0
        do  i=1,n
          gcg=gcg + g(i)*cg(i)
        enddo

c        if(k.eq.1) eps=eps*gcg


        if (gcg.lt.eps) then
c            write(*,*)'nombre d''iteration:',k,'  gcg=',gcg             
                         return
        endif

        gamma=gcg/gamma
        do  i=1,n
          h(i)=g(i)+gamma*h(i)
        enddo

      enddo

c      write(*,*)'le solveur gradiant conjugue ne converge pas.' 
      write(17,*)'le solveur gradiant conjugue ne converge pas.'
      stop

c      return
      end

c  fichier grad_qua.f
c
c***************************************************************
      subroutine gradiant(grad,det,n1,n2,n3,n4,xch,ych)
c***************************************************************
c
c   cette procedure calcule le gradiant des fonctions de base 
c   au point (xch,yxch) pour le carre de reference
c
c   grad: tableau contenant les valeurs des gradiants des 
c         fonctions de bases  
c   det: jacobien de la transformation: carre de reference --> quadrilatere 
c   n1,n2,n3,n4: contient les numeros, ainsi que les coordonnees des
c                quatre points qui constituent le quadrilatere
c   xch,ych: coordonnees du point de calcul
c
c

      implicit none

      real*8    grad,xch,ych,det,n1,n2,n3,n4,j11,j12,j21,j22
      dimension grad(4,*),n1(*),n2(*),n3(*),n4(*) 
c
c calcul du jacobien de la transformation
c
      j11=-n1(2)+n2(2)+n3(2)-n4(2)+ych*(n1(2)-n2(2)+n3(2)-n4(2))
      j11=j11/4.d0

      j12=-n1(3)+n2(3)+n3(3)-n4(3)+ych*(n1(3)-n2(3)+n3(3)-n4(3)) 
      j12=j12/4.d0

      j21=-n1(2)-n2(2)+n3(2)+n4(2)+xch*(n1(2)-n2(2)+n3(2)-n4(2))
      j21=j21/4.d0

      j22=-n1(3)-n2(3)+n3(3)+n4(3)+xch*(n1(3)-n2(3)+n3(3)-n4(3))
      j22=j22/4.d0

      det=(j11*j22)-(j21*j12)
c
c calcul du gradiant               
c
      grad(1,1)=(j22/det)*((ych-1)/4.d0)-(j12/det)*((xch-1)/4.d0)
      grad(1,2)=(-j21/det)*((ych-1)/4.d0)+(j11/det)*((xch-1)/4.d0)
      
      grad(2,1)=(j22/det)*((1-ych)/4.d0)-(j12/det)*((-xch-1)/4.d0)
      grad(2,2)=(-j21/det)*((1-ych)/4.d0)+(j11/det)*((-xch-1)/4.d0) 

      grad(3,1)=(j22/det)*((ych+1)/4.d0)-(j12/det)*((xch+1)/4.d0)
      grad(3,2)=(-j21/det)*((ych+1)/4.d0)+(j11/det)*((xch+1)/4.d0) 
      
      grad(4,1)=(j22/det)*((-ych-1)/4.d0)-(j12/det)*((1-xch)/4.d0)
      grad(4,2)=(-j21/det)*((-ych-1)/4.d0)+(j11/det)*((1-xch)/4.d0) 
      
      return
      end                                        
c  fichier grad_tri.f
c
c*****************************************************************
      subroutine gradiantp(gradp,det,n1,n2,n3)
c*****************************************************************
c
c   cette procedure calcule le gradiant des differentes fonctions 
c   de base pour le triangle .
c
c   n1,n2,n3: contient les numeros, ainsi que les coordonnees des
c                trois points qui constituent le triangle
c   gradp: tableau contenant les valeurs des  gradiants
c         des fonctions de base
c   det: jacobien de la transformation: triangle de reference --> triangle
c
c
      implicit none
      
      real*8    gradp,det,n1,n2,n3,j11,j12,j21,j22 
      dimension gradp(3,*),n1(*),n2(*),n3(*) 
c
c calcul du jacobien de la transformation
c      
      j11=(n2(2)-n1(2)) 
      j21=(n2(3)-n1(3))
      j12=(n3(2)-n1(2))
      j22=(n3(3)-n1(3))                     
      det=(j11*j22)-(j12*j21)
c
c calcul du gradiant     
c
      gradp(1,1)=(j21-j22)/det
      gradp(1,2)=(j12-j11)/det
      gradp(2,1)=(j22/det)
      gradp(2,2)=-(j12/det)
      gradp(3,1)=-(j21/det)
      gradp(3,2)=(j11/det)

      return
      end                                        






c fichier icholes.f
c
c******************************************************************
      subroutine icholeski(ns,aa,ia,ja,al)
c******************************************************************
c
c
c ce sous-programme effectue la factorisation de
c choleski incomplete de la matrice a stockee sous
c forme morse dans le tableau aa avec les tableaux
c auxiliaires d"indices ia et ja.
c ia(k): indice dans le tableau aa du premier element
c   de la i eme ligne de a stocke dans aa,
c ja(k): indice j de colonne de l"element aij stocke
c   dans le tableau aa a l"indice k.
c le resultat est stocke dans le tableau al, qui 
c utilise les memes tableaux auxiliaires d"indices
c ia et ja que aa.
c le calcul de la factorisation incomplete se fait
c ligne par ligne en terminant par le calcul de 
c l"element diagonal.
c
c
      implicit none
      integer   ia,ja,ns,k,jljk,jlik,ik,jk,i,j
      real*8    aa,al,alii,alij
      dimension aa(*),al(*),ia(*),ja(*)
c
      do  i=1,ns
        do  k=ia(i),ia(i+1)-2
          j=ja(k)
          alij=aa(k)
          ik=ia(i)
          jk=ia(j)
35	      if(ik.le.(ia(i+1)-2)) then
            jlik=ja(ik)
36	        if(jk.le.(ia(j+1)-2)) then
              jljk=ja(jk)
              if(jlik.gt.jljk)then
                jk=jk+1
                goto 36
              elseif(jlik.lt.jljk)then
                  ik=ik+1
                  goto 35
              else
                alij=alij-(al(ik)*al(jk))
                ik=ik+1
                jk=jk+1
                goto 35
              endif
            endif
          endif
          al(k)=alij/al(ia(j+1)-1)
        enddo 
        alii=aa(ia(i+1)-1)
        do  k=ia(i), ia(i+1)-2
          alii=alii-(al(k)*al(k))
        enddo
          al(ia(i+1)-1)=dsqrt(dabs(alii))
      enddo

      return
      end
c fichier info_ele.f
c
c****************************************************************
      subroutine coord_t_f(tabele,elementbis,indice,nbndele,
     @                     n1,n2,n3,n4,nte,k)
c****************************************************************
c
c   cette procedure donne les informations necessaires sur chaque
c   elements de la triangulation.
c
c   tabele: tableau contenant le numero de l'element, ainsi que les
c           numeros des noeuds qui constituent l'element
c   elementbis: tableau qui permet de dire si un noeud de l'element est
c              sur le bord ou non.
c   indice: tableau donnant le rang d'un noeud appartenant a vecti ou vectf
c   nbnele: nombre entier qui differencie le triangle du quadrilatere
c   n1,n2,n3,n4: contient les numeros, ainsi que les coordonnees des
c                quatre points qui constituent le quadrilatere
c   nte: nombre total d'element
c   k: numero de l'element sur lequel on veut les informations
c
c
      implicit none

      integer   nbndele,tabele,indice,k,nte
      real*8    n1,n2,n3,n4
      logical   elementbis
      dimension n1(*),n2(*),n3(*),n4(*)
      dimension tabele(nte,*),elementbis(*),indice(*)
c
c on regarde si on a affaire a un triangle ou un quadrialatere
c
      if (tabele(k,5).eq.0) then
                              nbndele=4
                            else
                              nbndele=5
      endif
c
c on lit les informations dans le fichier noeudv.dat
c      
      read(16,rec=tabele(k,2))n1(2),n1(3),elementbis(2),indice(2)
      read(16,rec=tabele(k,3))n2(2),n2(3),elementbis(3),indice(3)
      read(16,rec=tabele(k,4))n3(2),n3(3),elementbis(4),indice(4)
      if (nbndele.eq.5) then
      read(16,rec=tabele(k,5))n4(2),n4(3),elementbis(5),indice(5)
      endif

      return
      end

                                               



c    fichier jc.f
c
c************************************************************************
      subroutine cal_jc (vect1,jc,nelemc,nnf,cc)
c***********************************************************************
c
c cette procedure initialise le tableau cc et calcule le tableau
c jc final
c
c   nelemc :dimension du tableau cc
c   nnf :nombre de noeud situes sur la frontiere
c   cc: tableau contenant les contributions bord-int
c   jc: tableau d'indice associe a cc    
c   vect1: vecteur contenant les valeurs de jc non nulle
c
      implicit none 
 
      integer   i,vect1,nelemc,nnf,jc
      real*8    cc
      dimension vect1(*),jc(*),cc(*)

      do i=1,nelemc
         if (vect1(i).ne.0) then
                              jc(i)=vect1(i)
                            else
                              jc(i)=nnf+1
         endif
         write(20,*)i,'jc=',jc(i)
         cc(i)=0.d0
      enddo

      return
      end


c  fichier mise_a_0.f
c
c******************************************************************
      subroutine  mise_a_zero(bb,jb,aa,ja,k,sminter,nti,
     @                        nelema,nelemb,poroh)
c*****************************************************************
c
c  cette procedure initialise les tableaux bb,jb,aa,ja,sminter
c  a zero, ainsi que la partie non diagonale du tenseur k.
c
c  bb: tableau contenant les contributions bord-bord
c  aa: tableau contenant les contributions int-int (c'est la matrice a
c      du systeme lineaire)
c  ja(k): indice j de colonne de l"element aij stocke
c         dans le tableau aa a l"indice k
c  jb: idem que ja pour le tableau bb
c  k: tableau contenant le tenseur des permeabilites
c  sminter: vecteur contenant le second membre
c           du systeme lineaire sans les conditions aux limites 
c  nti: nombre total d'inconnues du systeme lineaire
c  nelema: dimension du tableau aa
c  nelemb: dimension du tableau bb
c
      implicit none

      integer   nti,i,ja,nelema,nelemb,jb
      real*8    sminter,aa,k,bb,poroh
      dimension sminter(*),aa(*),ja(*),k(2,*),bb(*),jb(*)


      do i=1,nti      
         sminter(i)=0.d0
      enddo


      do i=1,nelema
         aa(i)=0.d0
         ja(i)=0
      enddo

      do i=1,nelemb
         bb(i)=0.d0
         jb(i)=0
      enddo

      k(1,2)=0.d0
      k(2,1)=k(1,2)

      poroh=0.d0
      
      return
      end


 



c  fichier nfr_nint.f
c
c********************************************************************
       subroutine remplir (nbpoint,coord,g,vecti,vectf,droite,
     @                             indicei,indicef,x,y,nbr1)
c*********************************************************************
c
c   cette procedure remplie le tableau vectf des numeros des noeuds
c   situe sur la frontiere et vecti des noeuds situe a l'interieur
c   du domaine. on ecrit dans le fichier noeudv.dat les coordonnees
c   de chaque noeuds, sa position par rapport a la frontiere et son 
c   rang dans vecti ou vectf.
c
c   g: tableau contenant les coordonnees des points de la
c      grosse maille
c   coord: tableaux contenants les valeurs des vecteurs directeurs
c   nbpoint :nombre de point de la grosse maille
c   vecti: vecteur contenant les numeros des noeuds situes a l'interieur
c          de la grosse maille
c   vectf: vecteur contenant les numeros des noeuds situes sur la frontiere
c          de la grosse maille
c   droite: tableau des equations des droites   
c   indicei: indice comptant le nombre de noeuds situes a l'interieur
c   indicef: indice comptant le nombre de noeuds situes sur la frontiere 
c   x,y: coordonnees du noeud
c   nbr1: numero du noeud
c

       implicit none

       integer   i,indicei,indicef,vecti,vectf,nbpoint,entier
       real*8    x,y,g,droite,nbr1,dif,coord
       dimension g(nbpoint,*),vecti(*),vectf(*)
       dimension droite(nbpoint,*),coord(nbpoint,*)

c
c calcul des equations des droites
c
       do i=1,nbpoint
          if ((coord(i,1).eq.0.d0).or.(coord(i,2).eq.0.d0))  then
                  if (coord(i,1).eq.0.d0) then
                      droite(i,1)=x
                      droite(i,2)=g(i,1)          !  on peut
                                   else           !  ameliorer
                      droite(i,1)=y               !  cette partie
                      droite(i,2)=g(i,2) 
                   endif     
                                               else
                  droite(i,1)=(x-g(i,1))/coord(i,1)
                  droite(i,2)=(y-g(i,2))/coord(i,2) 
           endif        
       enddo
       i=1
c
c rangement des numeros des noeuds
c
c       write(*,*)
c       write(*,*)'nbr',nbr1
       do while (i.le.nbpoint)
          dif=abs(droite(i,1)-droite(i,2))
c          write(*,*)'dif',dif
          if (dif.le.1.d-03) then
                                vectf(indicef)=nbr1
                                entier=nbr1
                  write(16,rec=entier)x,y,.false.,indicef 
                                indicef=indicef+1 
                                return
                              else
                                i=i+1
          endif
       enddo
       vecti(indicei)=nbr1
       entier=nbr1
       write(16,rec=entier)x,y,.true.,indicei
       indicei=indicei+1 

        return
        end




                                               
c   fichier norme.f
c
c**********************************************************************
       subroutine norme(vecti,vectf,x,nti,nnf)
c**********************************************************************
c
c  cette procedure calcule l'erreur relative entre la solution
c  exacte et la solution approchee.
c  cette procedure est utilise pour verifier la solution element fini
c  lorsque l'on connait la solution exacte
c
c
c   vecti: vecteur contenant les numeros des noeuds situes a l'interieur
c          de la grosse maille
c   vectf: vecteur contenant les numeros des noeuds situes sur la frontiere
c          de la grosse maille
c   x: solution du probleme local
c   nti :nombre total d'inconnues du systeme lineaire
c   nnf :nombre de noeud situes sur la frontiere



      implicit none

      integer nti,nnf,i,vecti,vectf
      real*8 x,titi,ninf,toto,aux,bord_pb1
      dimension vecti(*),vectf(*),x(*)

c
c norme infini
c
      ninf=0.d0
      write(20,*)'comparaison a la solution exacte '
      do i=1,nti
         aux=bord_pb1(vecti(i))
         write(20,*)i,' ',aux,' ',x(i)
         toto=abs(aux-x(i))
         if (toto.gt.ninf) then
                             ninf=toto
         endif
      enddo
      write(20,*)
        write(20,*)'norme infinie =',ninf
      write(20,*)
c
c erreur relative l2
c
      ninf=0.d0
      titi=0.d0
      write(20,*)'norme l2 '
      do i=1,nti
         aux=bord_pb1(vecti(i))
         write(20,*)i,' ',aux,' ',x(i)
         toto=(aux-x(i))*(aux-x(i))
         titi=titi+aux**2
         ninf=ninf+toto
      enddo
      write(20,*)
      write(20,*)'erreur relative pour la norme l2 ',
     @            dsqrt(ninf)/dsqrt(titi)
      write(20,*)

      return
      end
c  fichier permequi.f
c
c         include'essai.f'



c********************************************************************
        subroutine permeaequi(aa,ia,ja,x1,x2,nti,nnf,aire,condb1,
     @                       condb2,bb,ib,jb,cc,ic,jc,poroh)
c*********************************************************************
c
c       cette procedure calcule le tenseur homogeneise a partir  
c       des 2 solutions des problemes locaux.    
c
c   aa: tableau contenant les contributions int-int (c'est la matrice a
c       du systeme lineaire)
c   ia(k): indice dans le tableau aa du premier element
c        de la i eme ligne de a stocke dans aa,
c   ja(k): indice j de colonne de l"element aij stocke
c          dans le tableau aa a l"indice k
c   bb: tableau contenant les contributions bord-bord
c   cc: tableau contenant les contributions bord-int
c   ib,jb: idem que ia et ja pour le tableau bb
c   ic,jc: idem que ia et ja pour le tableau cc
c   x1: solution du probleme local 1
c   x2 :solution du probleme local 2
c   condb1,condb2: tableaux contenant les conditions aux limites
c                  sur le bord suivant le probleme local
c   nti :nombre total d'inconnues du systeme lineaire
c   nnf :nombre de noeud situes sur la frontiere
c   aire :aire de la grosse maille
c
        implicit none

        integer   i,j,nti,nnf,l,entier,k,ia,ja,ib,jb,ic,jc,m,kk
        real*8    x1,x2,somme1,somme2,somme3,aire,poroh
        real*8    k11,k22,k12,condb1,condb2,aa,bb,cc,ast
        dimension x2(*),x1(*),aa(*),ia(*),ja(*),bb(*),ib(*),jb(*)
        dimension cc(*),ic(*),jc(*),condb1(*),condb2(*)
c
c initialisation
c
        somme1=0.d0
        somme2=0.d0
        somme3=0.d0

       write(20,*)
       write(20,*)'somme1',somme1
       write(20,*)'somme2',somme2
       write(20,*)'somme3',somme3
c
c contribution int-int
c
        m=1
        l=1
        do i=1,nti
           do k=1,(ia(m+1)-ia(m))
              j=ja(l)
           write(20,*)x1(i),x1(j),aa(l)
           write(20,*)x2(i),x2(j),aa(l)
              somme1=somme1+(x1(i)*x2(j)*aa(l))
              somme2=somme2+(x2(i)*x2(j)*aa(l))
              somme3=somme3+(x1(i)*x1(j)*aa(l))
              l=l+1
           enddo
           m=m+1
        enddo

       write(20,*)
       write(20,*)'somme1',somme1
       write(20,*)'somme2',somme2
       write(20,*)'somme3',somme3
      
        m=2
        l=2
        do i=2,nti
           do k=1,(ia(m+1)-ia(m)-1)
              j=ja(l)
              somme1=somme1+(x1(j)*x2(i)*aa(l))
              somme2=somme2+(x2(j)*x2(i)*aa(l))
              somme3=somme3+(x1(j)*x1(i)*aa(l))
              l=l+1
           enddo
           l=l+1
           m=m+1
        enddo
c
c contribution bord-int
c

       write(20,*)
       write(20,*)'somme1',somme1
       write(20,*)'somme2',somme2
       write(20,*)'somme3',somme3

       kk=1
       do i=1,nnf
          do j=1,ic(i+1)-ic(i)
              entier=jc(kk)
                somme1=somme1+(condb2(i)*cc(kk)*x1(entier))
                somme2=somme2+(condb2(i)*cc(kk)*x2(entier))
                somme3=somme3+(condb1(i)*cc(kk)*x1(entier))
              kk=kk+1
          enddo
       enddo

       write(20,*)
       write(20,*)'somme1',somme1
       write(20,*)'somme2',somme2
       write(20,*)'somme3',somme3


       kk=1
       do i=1,nnf
          do j=1,ic(i+1)-ic(i)
              entier=jc(kk)
                somme1=somme1+(condb1(i)*cc(kk)*x2(entier))
                somme2=somme2+(condb2(i)*cc(kk)*x2(entier))
                somme3=somme3+(condb1(i)*cc(kk)*x1(entier))
              kk=kk+1
          enddo
       enddo

c
c contribution bord-bord
c
       write(20,*)
       write(20,*)'somme1',somme1
       write(20,*)'somme2',somme2
       write(20,*)'somme3',somme3

        m=1
        l=1
        do i=1,nnf
           do k=1,(ib(m+1)-ib(m))
              j=jb(l)
              somme1=somme1+(condb1(i)*condb2(j)*bb(l))
              somme2=somme2+(condb2(i)*condb2(j)*bb(l))
              somme3=somme3+(condb1(i)*condb1(j)*bb(l))
              l=l+1
           enddo
           m=m+1
        enddo

       write(20,*)
       write(20,*)'somme1',somme1
       write(20,*)'somme2',somme2
       write(20,*)'somme3',somme3

        m=2
        l=2
        do i=2,nnf
           do k=1,(ib(m+1)-ib(m)-1)
              j=jb(l)
              somme1=somme1+(condb1(j)*condb2(i)*bb(l))
              somme2=somme2+(condb2(j)*condb2(i)*bb(l))
              somme3=somme3+(condb1(j)*condb1(i)*bb(l))
              l=l+1
           enddo
           l=l+1
           m=m+1
        enddo

       write(20,*)
       write(20,*)'somme1',somme1
       write(20,*)'somme2',somme2
       write(20,*)'somme3',somme3
c
c division par l'aire le la grosse maille
c
      
      k12=somme1/aire
      k22=somme2/aire
      k11=somme3/aire
      poroh=poroh/aire


      if (abs(k11).le.1.d-05) then
                            k11=0.d0
      endif
      if (abs(k12).le.1.d-05) then
                            k12=0.d0
      endif
      if (abs(k22).le.1.d-05) then
                            k22=0.d0
      endif
c      write(*,*)
c      write(*,19) 
c      write(*,20)k11,k12,k22,poroh

c modifications for the JHomogenizer interface
      open(37,file='thlbcOutput.dat')
      write(37,*) 1
      write(37,*) 1
      write(37,*) 1
      write(37,22) k11
      write(37,22) k12
      write(37,22) 0.0
      write(37,22) k12
      write(37,22) k22
      write(37,22) 0.0
      write(37,22) 0.0
      write(37,22) 0.0
      write(37,22) 1.0
      close(37)

c      write(17,*)
      write(17,*)k11,k12,k22         !,poroh


19    format(5x,'kxx',12x,'kxy',12x,'kyy',12x,'poro')
20    format(4(e11.4,4x))
22    format(e11.4)

c     c'est un essai
c       write(*,*)
       
c       call permeq2(k11,k12,k22)

c     c''est pour trouver un minorant
c      ast=(k11*k22)-(k12**2)
c      k11=k11/ast
c      k12=k12/ast
c      k22=k22/ast
c      write(*,20)k11,k12,k22

      return
      end
                                             
c   fichier perm_kx.f
c
c
c*************************************************************************
      real*8 function k11t(x,y)
c*************************************************************************
c
      implicit none
      real*8 x,y

c      k11t=1.d0         !x+y
      k11t=x+y

      return
      end

c*************************************************************************
      real*8 function k22t(x,y)
c*************************************************************************
c
      implicit none
      real*8 x,y

c      k22t=1.d0           !x+y
      k22t=x+y

      return
      end

c*************************************************************************
      real*8 function k12t(x,y)
c*************************************************************************
c
      implicit none
      real*8 x,y

      k12t=0.d0       !x+y

      return
      end
c      fichier prelimi1.f
c
c******************************************************************
      subroutine preliminaire(vectf,vecti,tabele,nnf,nti,nte,ic,nelemc,
     @                      vect1,ib,nelemb,ia,nelema)    
c*****************************************************************
c
c  cette procedure affecte aux tableaux ia,ib,ic les valeurs definitives
c  en vue de l'assemblage des tableaux aa,bb et cc. deplus les dimensions 
c  des tableaux aa,bb et cc sont calcules.
c
c  vectf: tableau contenant les numero des noeuds situe sur la
c         frontiere du domaine
c  vecti: tableau contenant les numero des noeuds situe a 
c         l'interieur du domaine
c  tabele: tableaux contenant les numeros des noeuds qui constituent 
c          les elements
c  nti: nombre total d'inconnue du systeme lineaire
c  nte: nombre total d'elements
c  ia(k): indice dans le tableau aa du premier element
c         de la i eme ligne de a stocke dans aa
c  ib,ic: idem que ia pour les tableaux bb,cc
c  nelema: dimension du tableau aa
c  nelemb: dimension du tableau bb
c  nelemc: dimension du tableau cc
c  vect1: tableau d'entiers en vue de calculer le tableau jc.
c  nnf :nombre de noeud situes sur la frontiere
c
c
      implicit none

      integer   nte,i,j,entif,vectf,vect1,ib,nelemb,deb,pos1,pos2
      integer   lb,k,indice,nbndele,l,entiv,ind,vecti,nti,nelema
      integer   tt,ic,nelemc,nnf,tabele,a,b,ivois,ia,m,pos,nb
      real*8    n1,n2,n3,n4
      logical   elementbis
      dimension vectf(*),indice(5),elementbis(5),ib(*),ia(*),vecti(*)
      dimension n1(3),n2(3),n3(3),n4(3)
      dimension ic(*),tabele(nte,*),vect1(*) 

      integer, allocatable, dimension(:,:) :: mata 
      integer, allocatable, dimension(:,:) :: matb
      integer, allocatable, dimension(:,:) :: matc
c
c allocation dynamique de memoire des tableaux intermediaires
c mata, matb et matc 
c

      write(20,*)'uuuuu0'

      lb=nti+27

      allocate(mata(nti,lb))
      allocate(matb(nnf,lb))
      allocate(matc(nnf,lb))

      write(20,*)'uuuuu1'

      do i=1,nti
         do j=1,lb
            mata(i,j)=-1
         enddo
      enddo

      do i=1,nnf
         do j=1,lb
            matb(i,j)=-1
            matc(i,j)=-1
         enddo
      enddo

      nb=1
      ind=1
      do k=1,nte
c         write(*,*) 
c         write(20,*)'element',k
         call coord_t_f(tabele,elementbis,indice,nbndele,
     @                        n1,n2,n3,n4,nte,k)        
c         write(20,*)(tabele(k,j),j=1,5)
c         write(20,*)(elementbis(j),j=1,5)
c         write(*,*)'nbndele',nbndele
         do j=2,(nbndele-1)
c         write(*,*)'j',j
                deb=j+1
                pos1=indice(j)
                entif=tabele(k,j)
                if (elementbis(j)) then
                    do m=deb,nbndele
                       entiv=tabele(k,m)
                       pos2=indice(m)
                       if (elementbis(m)) then
                           if (entiv.lt.entif) then
                           call rangement(mata,pos1,entif,entiv,lb,nti) 
                                               else
                           call rangement(mata,pos2,entif,entiv,lb,nti)
                           endif
                                          else
                           call rangement(matc,pos2,entif,entiv,lb,nnf)
                           nb=nb+1
                       endif
                    enddo
                                    else
                    do m=deb,nbndele
                       entiv=tabele(k,m)
                       pos2=indice(m)
                       if (.not.(elementbis(m))) then
                           if (entiv.lt.entif) then
                           call rangement(matb,pos1,entif,entiv,lb,nnf)
                                               else
                           call rangement(matb,pos2,entif,entiv,lb,nnf)
                           endif
                                                 else
                           call rangement(matc,pos1,entif,entiv,lb,nnf)
                           nb=nb+1
                       endif
                    enddo
                endif
           enddo 
       enddo      
                    
       do i=1,nti
          l=1
          do while (mata(i,l).ne.-1) 
             l=l+1
          enddo
          ia(i)=l-1
       enddo

       deallocate(mata)


c        write(*,*)'nb=',nb
c       nb=nb+nnf
c       allocate(vect1(nb))

        write(20,*)'ord_gr=',nb+nnf

c        write(*,*)'ord_gr=',nb+nnf

       do i=1,nnf
          write(20,*)
          write(20,*)'matb',(matb(i,tt),tt=1,10)
       enddo 

       do i=1,nnf
          write(20,*)
          write(20,*)'matc',(matc(i,tt),tt=1,10)
       enddo
 
c       pause 

       do i=1,nnf
          write(20,*)
          write(20,*)'matc',(matc(i,tt),tt=1,10)
          l=1
          j=1
          do while (matb(i,l).ne.-1)
             l=l+1
          enddo
          do while (matc(i,j).ne.-1)
             nb=matc(i,j)-vectf(i)
             pos=1
             do while ((nb.ne.vecti(pos)).and.(pos.le.nti)) 
                pos=pos+1
             enddo
             vect1(ind)=pos
             ind=ind+1
             j=j+1
          enddo
          ib(i)=l-1
          write(20,*)'ib',ib(i)
          ic(i)=j-1
          if (ic(i).eq.0) then
             vect1(ind)=0
             ind=ind+1
             ic(i)=1
          endif
       enddo

       write(20,*)'vect1',(vect1(a),a=1,ind+3)
      pause

c      deallocate(mata)
      deallocate(matb)


      deallocate(matc)

      write(20,*)'ic                        prelimi1111111 '

c      pause
      do i=1,nnf
          write(20,*)'ic',ic(i),'   i',i
      enddo
      write(20,*)'ic final'


c      pause
c
c calcul du tableau ic final ainsi que nelemc
c
      a=ic(1)
      ic(1)=1
      write(20,*)ic(1)
      do i=1,nnf-1
         b=ic(i+1)
         ic(i+1)=ic(i)+a
         write(20,*)ic(i)
         a=b
      enddo
      ic(nnf+1)=ic(nnf)+a
      nelemc=ic(nnf)+a-1

      write(20,*)'ic final'
      do i=1,nnf
          write(20,*)ic(i)
      enddo

      write(20,*)'nelemc',nelemc


      do i=1,nnf
         write(20,*)'ib',ib(i)
      enddo


      write(20,*)
      write(20,*)'ib final'
c
c calcul du tableau ib final ainsi que nelemb
c
      nelemb=0
      do i=1,nnf
         ivois=ib(i)
         ib(i)=nelemb+1
         write(20,*)ib(i)
         nelemb=nelemb+ivois+1
      enddo
      ib(nnf+1)=nelemb+1
      write(20,*)ib(nnf+1)
      write(20,*)'nelemb',nelemb


      do i=1,nti
         write(20,*)'ia',ia(i)
      enddo


      write(20,*)
      write(20,*)'ia final'
c
c calcul du tableau ia final ainsi que nelema
c
      nelema=0
      do i=1,nti
         ivois=ia(i)
         ia(i)=nelema+1
         write(20,*)ia(i)
         nelema=nelema+ivois+1
      enddo
      ia(nti+1)=nelema+1
      write(20,*)ia(nti+1)
      write(20,*)'nelema',nelema


      return
      end



c*****************************************************************
       subroutine rangement(mata,pos,entif,entiv,nti,borne)
c*****************************************************************
c
c
       implicit none
       integer mata,l,nti,pos,entif,entiv,borne

       dimension mata(borne,*)


       l=1
       do while (l.le.nti)
          if (mata(pos,l).ne.entif+entiv) then      
              if (mata(pos,l).eq.-1) then
                  mata(pos,l)=entif+entiv
c                  write(*,*)'mat',(mata(pos,k),k=1,l)
                  return
                                     else
                  l=l+1
c               write(*,*)'l=',l
              endif
                                          else
              return
          endif
       enddo
       
       return
       end

c      fichier prelimi1.f
c
c******************************************************************
      subroutine preliminaire21(vectf,vecti,tabele,nnf,nti,nte,ia,
     @                         mata,matb,matc,ord_gr,lb)    
c*****************************************************************
c
c  cette procedure affecte aux tableaux ia,ib,ic les valeurs definitives
c  en vue de l'assemblage des tableaux aa,bb et cc. deplus les dimensions 
c  des tableaux aa,bb et cc sont calcules.
c
c  vectf: tableau contenant les numero des noeuds situe sur la
c         frontiere du domaine
c  vecti: tableau contenant les numero des noeuds situe a 
c         l'interieur du domaine
c  tabele: tableaux contenant les numeros des noeuds qui constituent 
c          les elements
c  nti: nombre total d'inconnue du systeme lineaire
c  nte: nombre total d'elements
c  ia(k): indice dans le tableau aa du premier element
c         de la i eme ligne de a stocke dans aa
c  ib,ic: idem que ia pour les tableaux bb,cc
c  nelema: dimension du tableau aa
c  nelemb: dimension du tableau bb
c  nelemc: dimension du tableau cc
c  vect1: tableau d'entiers en vue de calculer le tableau jc.
c  nnf :nombre de noeud situes sur la frontiere
c
c
      implicit none

      integer   nte,i,j,entif,vectf,deb,pos1,pos2
      integer   lb,k,indice,nbndele,l,entiv,ind,vecti,nti
      integer   nnf,tabele,ia,m,nb,iii,jjj
      integer   ord_gr,mata,matb,matc
      real*8    n1,n2,n3,n4
      logical   elementbis
      dimension mata(nti,*),matb(nnf,*),matc(nnf,*)
      dimension vectf(*),indice(5),elementbis(5),ia(*),vecti(*)
      dimension n1(3),n2(3),n3(3),n4(3)
      dimension tabele(nte,*) 

c      integer, allocatable, dimension(:,:) :: mata 
c      integer, allocatable, dimension(:,:) :: matb
c      integer, allocatable, dimension(:,:) :: matc
c
c allocation dynamique de memoire des tableaux intermediaires
c mata, matb et matc 
c

      write(20,*)'uuuuu0'

c      lb=nti+27

c      allocate(mata(nti,lb))
c      allocate(matb(nnf,lb))
c      allocate(matc(nnf,lb))

      write(20,*)'uuuuu1',lb,nnf,nti

      do i=1,nti
         do j=1,lb
            mata(i,j)=-1
         enddo
      enddo

      do i=1,nnf
         do j=1,lb
            matb(i,j)=-1
            matc(i,j)=-1
         enddo
      enddo

c      do i=1,nti
c           write(20,*)'mata',(mata(i,kkk),kkk=1,100)
c      enddo

c      do i=1,nnf
c           write(20,*)'matb',(matb(i,kkk),kkk=1,100)
c      enddo

c      do i=1,nnf
c           write(20,*)'matc',(matc(i,kkk),kkk=1,100)
c      enddo
c      pause

      nb=1
      ind=1
      do k=1,nte
         write(20,*) 
         write(20,*)'element',k
         call coord_t_f(tabele,elementbis,indice,nbndele,
     @                        n1,n2,n3,n4,nte,k)        
         write(20,*)(tabele(k,j),j=1,5)
         write(20,*)(elementbis(j),j=1,5)
         write(20,*)'nbndele',nbndele
         do j=2,(nbndele-1)
c         write(*,*)'j',j
                deb=j+1
                pos1=indice(j)
                entif=tabele(k,j)
                if (elementbis(j)) then
                    do m=deb,nbndele
                       entiv=tabele(k,m)
                       pos2=indice(m)
                       if (elementbis(m)) then
                           if (entiv.lt.entif) then
c                 write(20,*)'mata'
                           call rangement(mata,pos1,entif,entiv,lb,nti) 
                                               else
c                 write(20,*)'mata'
                           call rangement(mata,pos2,entif,entiv,lb,nti)
                           endif
                                          else
c                 write(20,*)'matc'
                           call rangement(matc,pos2,entif,entiv,lb,nnf)
                           nb=nb+1
                       endif
                    enddo
                                    else
                    do m=deb,nbndele
                       entiv=tabele(k,m)
                       pos2=indice(m)
                       if (.not.(elementbis(m))) then
                           if (entiv.lt.entif) then
c                 write(20,*)'matb'
                           call rangement(matb,pos1,entif,entiv,lb,nnf)
c                 write(20,*)'mat',(matb(pos1,kkk),kkk=1,100)
                                               else
c                 write(20,*)'matb'
                           call rangement(matb,pos2,entif,entiv,lb,nnf)
c                 write(20,*)'mat',(matb(pos2,kkk),kkk=1,100)
                           endif
                                                 else
c                 write(20,*)'matc'
                           call rangement(matc,pos1,entif,entiv,lb,nnf)
                           nb=nb+1
                       endif
                    enddo
                endif
           enddo 
       enddo      

                write(20,*)'***********************'
                do iii=1,nti
                  write(20,*)'mata',(mata(iii,jjj),jjj=1,lb)
                enddo
                   write(20,*)
                do iii=1,nnf
                   write(20,*)'matb',(matb(iii,jjj),jjj=1,lb)
                enddo
                   write(20,*)
                do iii=1,nnf
                   write(20,*)'matc',(matc(iii,jjj),jjj=1,lb)
                enddo
                write(20,*)'***********************'

                    
       do i=1,nti
          l=1
          do while (mata(i,l).ne.-1) 
             l=l+1
          enddo
          ia(i)=l-1
       enddo

       ord_gr=nb+nnf+100

c       write(*,*)'nouvel ord-gr',ord_gr 

       return
       end





c*****************************************************************
c       subroutine rangement(mata,pos,entif,entiv,lb,borne)
c*****************************************************************
c
c
c       implicit none
c       integer mata,l,lb,pos,entif,entiv,borne,k
c
c       dimension mata(borne,*)
c
c       write(20,*)'entif,entiv,pos',entif,' ',entiv,' ',pos
c       l=1
c       do while (l.le.lb)
c          if (mata(pos,l).ne.entif+entiv) then      
c              if (mata(pos,l).eq.-1) then
c                  mata(pos,l)=entif+entiv
c                  write(20,*)'mat',(mata(pos,k),k=1,100)
c                  return
c                                     else
c                  l=l+1
c               write(*,*)'l=',l
c              endif
c                                          else
c              return
c          endif
c       enddo
c       
c       return
c       end


c      fichier prelimi1.f
c
c******************************************************************
      subroutine preliminaire22(vectf,vecti,tabele,nnf,nti,nte,ic,
     @               nelemc,vect1,ib,nelemb,ia,nelema,matb,matc)    
c*****************************************************************
c
c  cette procedure affecte aux tableaux ia,ib,ic les valeurs definitives
c  en vue de l'assemblage des tableaux aa,bb et cc. deplus les dimensions 
c  des tableaux aa,bb et cc sont calcules.
c
c  vectf: tableau contenant les numero des noeuds situe sur la
c         frontiere du domaine
c  vecti: tableau contenant les numero des noeuds situe a 
c         l'interieur du domaine
c  tabele: tableaux contenant les numeros des noeuds qui constituent 
c          les elements
c  nti: nombre total d'inconnue du systeme lineaire
c  nte: nombre total d'elements
c  ia(k): indice dans le tableau aa du premier element
c         de la i eme ligne de a stocke dans aa
c  ib,ic: idem que ia pour les tableaux bb,cc
c  nelema: dimension du tableau aa
c  nelemb: dimension du tableau bb
c  nelemc: dimension du tableau cc
c  vect1: tableau d'entiers en vue de calculer le tableau jc.
c  nnf :nombre de noeud situes sur la frontiere
c
c
      implicit none

      integer   nte,i,j,vectf,vect1,ib,nelemb
      integer   tt,l,ind,vecti,nti,nelema
      integer   ic,nelemc,nnf,tabele,a,b,ivois,ia,pos,nb
      integer   matb,matc
c      real*8    n1,n2,n3,n4
      dimension matb(nnf,*),matc(nnf,*)
      dimension vectf(*),ib(*),ia(*),vecti(*)
c      dimension n1(3),n2(3),n3(3),n4(3)
      dimension ic(*),tabele(nte,*),vect1(*) 

c       do i=1,nnf
c          write(20,*)
c          write(20,*)'matb',(matb(i,tt),tt=1,10)
c       enddo 

c       do i=1,nnf
c          write(20,*)
c          write(20,*)'matc',(matc(i,tt),tt=1,10)
c       enddo
 
c       pause 

       ind=1
       do i=1,nnf
c          write(20,*)
c          write(20,*)'matc',(matc(i,tt),tt=1,10)
          l=1
          j=1
          do while (matb(i,l).ne.-1)
             l=l+1
          enddo
          do while (matc(i,j).ne.-1)
             nb=matc(i,j)-vectf(i)
             pos=1
             do while ((nb.ne.vecti(pos)).and.(pos.le.nti)) 
                pos=pos+1
             enddo
             vect1(ind)=pos
             ind=ind+1
             j=j+1
          enddo
          ib(i)=l-1
c          write(20,*)'ib',ib(i)
          ic(i)=j-1
          if (ic(i).eq.0) then
             vect1(ind)=0
             ind=ind+1
             ic(i)=1
          endif
       enddo

c       write(20,*)'vect1',(vect1(a),a=1,ind+3)
c      pause

c      deallocate(mata)
c      deallocate(matb)


c      deallocate(matc)

      write(20,*)'ic                        prelimi1111111 '

c      pause
c      do i=1,nnf
c          write(20,*)'ic',ic(i),'   i',i
c      enddo
c      write(20,*)'ic final'


c      pause
c
c calcul du tableau ic final ainsi que nelemc
c
      a=ic(1)
      ic(1)=1
c      write(20,*)ic(1)
      do i=1,nnf-1
         b=ic(i+1)
         ic(i+1)=ic(i)+a
c         write(20,*)ic(i)
         a=b
      enddo
      ic(nnf+1)=ic(nnf)+a
      nelemc=ic(nnf)+a-1

      write(20,*)'ic final'
      do i=1,nnf
          write(20,*)ic(i)
      enddo

      write(20,*)'nelemc',nelemc
c      write(*,*)'le veritable ord_gr',nelemc

c      do i=1,nnf
c         write(20,*)'ib',ib(i)
c      enddo


      write(20,*)
      write(20,*)'ib final'
c
c calcul du tableau ib final ainsi que nelemb
c
      nelemb=0
      do i=1,nnf
         ivois=ib(i)
         ib(i)=nelemb+1
         write(20,*)ib(i)
         nelemb=nelemb+ivois+1
      enddo
      ib(nnf+1)=nelemb+1
      write(20,*)ib(nnf+1)
      write(20,*)'nelemb',nelemb


      do i=1,nti
c         write(20,*)'ia',ia(i)
      enddo


      write(20,*)
      write(20,*)'ia final'
c
c calcul du tableau ia final ainsi que nelema
c
      nelema=0
      do i=1,nti
         ivois=ia(i)
         ia(i)=nelema+1
         write(20,*)ia(i)
         nelema=nelema+ivois+1
      enddo
      ia(nti+1)=nelema+1
      write(20,*)ia(nti+1)
      write(20,*)'nelema',nelema


      return
      end



c      fichier prelimi.f
c
c******************************************************************
c      subroutine preliminaire(vectf,vecti,tabele,nnf,nti,nte,ic,nelemc,
c     @                      vect1,ib,nelemb,ia,nelema)    
c*****************************************************************
c
c  cette procedure affecte aux tableaux ia,ib,ic les valeurs definitives
c  en vue de l'assemblage des tableaux aa,bb et cc. deplus les dimensions 
c  des tableaux aa,bb et cc sont calcules.
c
c  vectf: tableau contenant les numero des noeuds situe sur la
c         frontiere du domaine
c  vecti: tableau contenant les numero des noeuds situe a 
c         l'interieur du domaine
c  tabele: tableaux contenant les numeros des noeuds qui constituent 
c          les elements
c  nti: nombre total d'inconnue du systeme lineaire
c  nte: nombre total d'elements
c  ia(k): indice dans le tableau aa du premier element
c         de la i eme ligne de a stocke dans aa
c  ib,ic: idem que ia pour les tableaux bb,cc
c  nelema: dimension du tableau aa
c  nelemb: dimension du tableau bb
c  nelemc: dimension du tableau cc
c  vect1: tableau d'entiers en vue de calculer le tableau jc.
c  nnf :nombre de noeud situes sur la frontiere
c
c
c      implicit none
c
c      integer   nte,i,j,entf,enti,vectf,vect1,ib,nelemb
c      integer   k,indice,nbndele,l,ent1,ll1,ti,ind,ll2,vecti,nti,nelema
c      integer   ic,nelemc,nnf,tabele,a,b,ivois,ll3,ia,r,s
c      real*8    n1,n2,n3,n4
c      logical   elementbis,bolf,boli
c      dimension vectf(*),indice(5),elementbis(5),ib(*),ia(*),vecti(*)
c      dimension n1(3),n2(3),n3(3),n4(3)
c      dimension ic(*),tabele(nte,*),vect1(*) 
c
c      integer, allocatable, dimension(:) :: vect 
c      integer, allocatable, dimension(:) :: vect2
c      integer, allocatable, dimension(:) :: vect3
c
c allocation dynamique de memoire des tableaux intermediaires
c vect, vect2 et vect3
c
c      allocate(vect(nnf))
c      allocate(vect2(nnf))
c      allocate(vect3(nti))
c      bolf=.true.
c      boli=.true.
c      r=1
c      s=1
c      ind=1
c      do while (boli.or.bolf)
c         ll1=1
c         ll2=1
c         ll3=1
c         do ti=1,nnf
c            vect(ti)=-1
c            vect2(ti)=-1
c         enddo
c         do ti=1,nti
c            vect3(ti)=-1
c         enddo
c         entf=vectf(r)
c         enti=vecti(s)
c         do k=1,nte
c            if (bolf.and.((tabele(k,2).eq.entf).or.(tabele(k,3).eq.entf)
c     @          .or.(tabele(k,4).eq.entf).or.(tabele(k,5).eq.entf)))then
c                call coord_t_f(tabele,elementbis,indice,nbndele,
c     @                        n1,n2,n3,n4,nte,k)
c                          do j=2,nbndele            
c                             if (elementbis(j)) then
c                                l=1
c                                ent1=tabele(k,j)
c                              if (ent1.ne.0) then
c                                do while (l.le.nnf) 
c                                   if (vect(l).ne.ent1+entf) then
c                                      if (vect(l).eq.-1) then
c                                                    vect(ll1)=ent1+entf
c                                                    vect1(ind)=indice(j)
c                                                    ind=ind+1
c                                                    ll1=ll1+1
c                                                        else
c                                                     l=l+1
c                                      endif
c                                                     else
c                                                     goto 10
c                                   endif
c                                enddo
c                              endif
c                                                else
c                                l=1
c                                ent1=tabele(k,j)
c                              if((ent1.lt.entf).and.(ent1.ne.0)) then
c                                do while (l.le.nnf) 
c                                   if (vect2(l).ne.ent1+entf) then
c                                      if (vect2(l).eq.-1) then
c                                                    vect2(ll2)=ent1+entf
c                                                    ll2=ll2+1
c                                                        else
c                                                     l=l+1
c                                      endif
c                                                     else
c                                                     goto 10
c                                   endif
c                                enddo
c                              endif
c                             endif
c10                           continue
c                          enddo
c            endif
c            if(boli.and.((tabele(k,2).eq.enti).or.(tabele(k,3).eq.enti)
c     @          .or.(tabele(k,4).eq.enti).or.(tabele(k,5).eq.enti)))then
c                call coord_t_f(tabele,elementbis,indice,nbndele,
c     @                        n1,n2,n3,n4,nte,k)
c                          do j=2,nbndele
c                             if (elementbis(j)) then
c                                l=1
c                                ent1=tabele(k,j)
c                              if ((ent1.lt.enti).and.(ent1.ne.0)) then
c                                do while (l.le.nti) 
c                                   if (vect3(l).ne.ent1+enti) then
c                                      if (vect3(l).eq.-1) then
c                                                    vect3(ll3)=ent1+enti
c                                                     ll3=ll3+1
c                                                        else
c                                                     l=l+1
c                                      endif
c                                                     else
c                                                     goto 11
c                                   endif
c                                enddo
c                              endif
c                             endif
c11                           continue
c                          enddo
c            endif
c         enddo
c          if (r.le.nnf) then
c                    write(20,*)'vect',(vect(a),a=1,ll1+1)
c                    ic(r)=ll1-1
c                    ib(r)=ll2-1
cc                    write(20,*)'vect1',(vect1(a),a=1,ll1+1)
c                    if (ic(r).eq.0) then
c                           vect1(ind)=0
c                           ind=ind+1
c                           ic(r)=1
c                    endif
c          endif
c          if (s.le.nti) then
c                    ia(s)=ll3-1
c          endif
c         r=r+1
c         if (r.gt.nnf) then
c                         bolf=.false.
c         endif
c         s=s+1
c         if (s.gt.nti) then
c                         boli=.false.
c         endif
c      enddo
c      write(20,*)'vect1',(vect1(a),a=1,ind+3)
c
c      do i=1,nnf
c         write(*,*)'ic',ic(i),'         i=',i
c         write(20,*)'ic',ic(i),'         i=',i
c      enddo
cc
cc on rend la memoire des tableaux vect, vect2 et vect3 au systeme
cc
c      deallocate(vect)
c      deallocate(vect2)
c      deallocate(vect3)
c
c      write(20,*)'nnf',nnf
c      write(20,*)ic(1)
c      write(20,*)'ic final'
c
cc
cc calcul du tableau ic final ainsi que nelemc
cc    
c      a=ic(1)
c      ic(1)=1    
cc      write(20,*)ic(1)
c      do i=1,nnf-1
c         b=ic(i+1)
c         ic(i+1)=ic(i)+a
c         a=b
c      enddo                          
c      ic(nnf+1)=ic(nnf)+a
c      nelemc=ic(nnf)+a-1
c
c          
c      write(20,*)'ic final'
c      do i=1,nnf
c          write(20,*)ic(i)
c      enddo
c
c      write(20,*)'nelemc',nelemc
c
c
c      do i=1,nnf
c         write(20,*)'ib',ib(i)
c      enddo
c
c    
c      write(20,*)
c      write(20,*)'ib final'
cc
cc calcul du tableau ib final ainsi que nelemb
cc
c      nelemb=0
c      do i=1,nnf
c         ivois=ib(i)
c         ib(i)=nelemb+1
c         write(20,*)ib(i)
c         nelemb=nelemb+ivois+1
c      enddo
c      ib(nnf+1)=nelemb+1
c      write(20,*)ib(nnf+1)
c      write(20,*)'nelemb',nelemb
c
c
c      do i=1,nti
c         write(20,*)'ia',ia(i)
c      enddo
c
c      
c      write(20,*)
c      write(20,*)'ia final'
cc
cc calcul du tableau ia final ainsi que nelema
cc
cc      nelema=0
c      do i=1,nti
c         ivois=ia(i)
c         ia(i)=nelema+1
c         write(20,*)ia(i)
c         nelema=nelema+ivois+1
c      enddo
c      ia(nti+1)=nelema+1
c      write(20,*)ia(nti+1)
c      write(20,*)'nelema',nelema
c
c
c      return
c      end





c fichier prod_mat.f
c
c
c******************************************************************
      subroutine iamul(ns,aa,ia,ja,x,b)
c******************************************************************
c
c ce sous-programme calcule le produit de la matrice
c a, stockee sous forme morse dans les tableaux 
c (aa,ia,ja), par le vecteur x; le resultat est dans le
c tableau b.
c le calcul se fait d"abord en calculant la contribution
c associee a la partie triangulaire inferieure de la 
c matrice (qui est celle stockee dans le tableau aa),
c puis celle associee a la partie triangulaire superieure
c stricte.
c
c
      implicit none
      integer ns,i,k,jak,ia,ja
      real*8  aa,x,xi,b,somme

      dimension x(*),b(*)
      dimension aa(*),ia(*),ja(*)

      do  i=1,ns
        somme=0.d0
        do  k=ia(i),ia(i+1)-1
          somme=somme+aa(k)*x(ja(k))
        enddo
        b(i)=somme
      enddo
      do  i=1,ns
        xi=x(i)
        do  k=ia(i),ia(i+1)-2
          jak=ja(k)
          b(jak)=b(jak)+aa(k)*xi
        enddo
      enddo

      return
      end


c  fichier prod_qua.f
c
c*******************************************************     
      real*8 function produit (k,grad,nbr1,nbr2)
c*******************************************************
c
c   cette fonction calcule le produit scalaire de [k(x)grad(u).grad(v)],
c   u et v designant deux fonctions de base quelconque du
c   quadrilatere.  
c
c   k: tableau contenant le tenseur des permeabilites
c   grad: tableau contenant les valeurs des  gradiants
c         des fonctions de base
c   nbr1,nbr2: numero designant la nbr1ieme et la nbr2ieme
c              fonction de base
c    


      implicit none
      integer   nbr1,nbr2   
      real*8    k,grad
      dimension k(2,*),grad(4,*)
c
c produit scalaire
c
      produit=k(1,1)*grad(nbr1,1)*grad(nbr2,1)+
     @        k(1,2)*grad(nbr1,2)*grad(nbr2,1)+
     @        k(2,1)*grad(nbr1,1)*grad(nbr2,2)+
     @        k(2,2)*grad(nbr1,2)*grad(nbr2,2)
      
      return
      end
                         

                         

c fichier prod_tri.f
c
c******************************************************************
      real*8 function produitp (k,gradp,nbr1,nbr2)
c******************************************************************
c
c  cette fonction calcule le produit scalaire (k(x)grad(u).grad(v))
c  pour un triangle.
c
c   k: tableau contenant le tenseur des permeabilites
c   gradp: tableau contenant les valeurs des  gradiants
c         des fonctions de base
c   nbr1,nbr2: numero designant la nbr1ieme et la nbr2ieme
c              fonction de base

      implicit none

      integer   nbr1,nbr2     
      real*8    k,gradp
      dimension gradp(3,*),k(2,*)

c
c produit scalaire
c
      produitp=k(1,1)*gradp(nbr1,1)*gradp(nbr2,1)+
     @         k(1,2)*gradp(nbr1,2)*gradp(nbr2,1)+
     @         k(2,1)*gradp(nbr1,1)*gradp(nbr2,2)+
     @         k(2,2)*gradp(nbr1,2)*gradp(nbr2,2)


      return
      end


c fichier quadrila.f
c
c*********************************************************************
c                 fichiers necessaires a la subroutine calculq1
c*********************************************************************

c      include'./grad_qua.f'
c      include'./prod_qua.f'
c      include'./rige_qua.f'
c      include'./base_qua.f'
c      include'./2mb_qua.f'
c      include'./aire2.f'

c*******************************************************************
      subroutine calculq1(elementbis,indice,bb,ib,jb,aa,ia,ja,sminter
     @                    ,n1,n2,n3,n4,k,cc,ic,jc,aire,poro,poroh)
c*******************************************************************
c
c  cette procedure calcule et assemble les differentes contributions amenees
c  par un element de type quadrilatere dans le tableau aa. deplus 
c  certains resultats necessaire en vue du calcul du tenseur homogeneise
c  sont sauvegardes dans les tableaux bb et cc.
c
c  elementbis: tableau qui permet de dire si un noeud de l'element est 
c              sur le bord ou non.
c  indice:  tableau donnant le rang d'un noeud appartenant a vecti ou vectf
c  bb: tableau contenant les contributions bord-bord
c  aa: tableau contenant les contributions int-int (c'est la matrice a
c      du systeme lineaire) 
c  cc: tableau contenant les contributions bord-int
c  ia(k): indice dans le tableau aa du premier element
c        de la i eme ligne de a stocke dans aa,
c  ja(k): indice j de colonne de l"element aij stocke
c         dans le tableau aa a l"indice k
c  ib,jb: idem que ia et ja pour le tableau bb
c  ic,jc: idem que ia et ja pour le tableau cc
c  sminter:vecteur contenant le second membre du systeme lineaire
c            sans les conditions aux limites
c   n1,n2,n3,n4: contient les numeros, ainsi que les coordonnees des
c                quatre points qui constituent le quadrilatere
c   k: tableau contenant le tenseur des permeabilites
c
c
      implicit none           

      integer   ele,l,nbr1,indice,ia,ja,ib,jb,ic,jc,nbr2,k1,k2,ent  
      real*8    k,n1,n2,n3,n4,grad,det,aa,bb,cc,sminter,aire0,aire1 
      real*8    integrale,som1,integrpq,aire,poro,poroh
      logical   elementbis,log1,log2
      dimension n1(*),n2(*),n3(*),n4(*),k(2,*),grad(4,2)  
      dimension elementbis(*),indice(*),bb(*),ib(*),jb(*)  
      dimension aa(*),ia(*),ja(*),sminter(*),cc(*),ic(*),jc(*)  
     
      call aire_ele2(aire0,n1,n2,n3)
      call aire_ele2(aire1,n3,n4,n1)
      aire=aire+(aire0+aire1)
      poroh=poroh+(poro*(aire0+aire1))

 
       do ele=2,5
         nbr1=ele-1
         log2=.not.elementbis(ele)
         do l=ele,5
            k1=indice(ele)
            k2=indice(l)
            nbr2=l-1
            log1=.not.elementbis(l)
c
c calcul de la contribution
c
            som1=integrale(grad,n1,n2,n3,n4,k,det,nbr1,nbr2)
c
c assemblage
c            
            if (log1.and.log2) then     
                  if (k2.ge.k1) then                         
                           ent=k1
                           k1=k2
                           k2=ent
                  endif
                  call assemblage(bb,ib,jb,k1,k2,som1)
                               else
    
            if (elementbis(ele).and.elementbis(l)) then
                  if (k2.ge.k1) then                         
                           ent=k1
                           k1=k2
                           k2=ent
                  endif
                  call assemblage(aa,ia,ja,k1,k2,som1)
                  else

            log1=.not.elementbis(l)
        if (elementbis(ele).and.log1) then
                   call assemblagebis(cc,ic,jc,k2,k1,som1)
                else

          if (elementbis(l)) then               
                     k1=indice(l)
                     k2=indice(ele)
                     call assemblagebis(cc,ic,jc,k2,k1,som1) 
            endif
           endif
         endif
        endif
       enddo
c
c calcul de la contribution du second membre et assemblage
c
       if (elementbis(ele)) then
            k1=indice(ele)
            sminter(k1)=sminter(k1)+integrpq(nbr1,n1,n2,n3,n4)
       endif
      enddo

      return
      end
         
c fichier qu_compl.f
c
c*********************************************************************
c                 fichiers necessaires a la subroutine calculq1
c*********************************************************************

c      include'bib/grad_qua.f'
c      include'bib/prod_qua.f'
c      include'bib/rige2_qu.f'
c      include'bib/base_qua.f'
c      include'bib/2mb_qua.f'
c      include'bib/perm_kx.f'

c  fichier resol_sys.f
c
c******************************************************************
            subroutine isolvsys(ns,z,al,il,jl)
c******************************************************************
c
c ce sous-programme resout le systeme lineaire
c ll^t z = r, ou l est une matrice triangulaire
c inferieure de taille ns x ns, stockee sous forme
c morse dans les tableaux (al,il,jl). en entree, le 
c tableau z contient le second membre du systeme; 
c en sortie, il contient la solution z.
c la resolution se fait en 2 etapes: une etape de 
c descente, suivie d'une etape de remontee.
c
c

      implicit none
      integer ns,i,i1,k,jlk,il,jl
      real*8  al,z,somme
      dimension z(*),al(*),il(*),jl(*)

      do  i=1,ns
        somme=z(i)
        do  k=il(i), il(i+1)-2
          somme=somme-al(k)*z(jl(k))
        enddo
        z(i)=somme/al(il(i+1)-1)
      enddo
      do  i=1,ns
        z(i) = z(i)/al(il(i+1)-1)
      enddo
      do  i1=1,ns
        i=ns+1-i1
        do  k=il(i), il(i+1)-2
          jlk=jl(k)
          z(jlk) = z(jlk) - al(k)*z(i)/al(il(jlk+1)-1)
        enddo
      enddo

      return
      end
c  fichier  rige2_qu.f
c
c*****************************************************************
      real*8 function integrale(grad,n1,n2,n3,n4,k,det,nbr1,nbr2)
c*****************************************************************
c
c   cette fonction calcule l'integrale de k(x)grad(u).grad(v) sur
c   un quadrilatere. u et v designant deux fonctions de base quelconque
c   du quadrilatere.
c
c   n1,n2,n3,n4: contient les numeros, ainsi que les coordonnees des
c                quatre points qui constituent le quadrilatere
c   k: tableau contenant le tenseur des permeabilites
c   grad: tableau contenant les valeurs des  gradiants
c         des fonctions de base
c   nbr1,nbr2: numero designant la nbr1ieme et la nbr2ieme
c              fonction de base
c   det: jacobien de la transformation: carre de reference --> quadrilatere  
c
c

      implicit none

      integer   nbr1,nbr2          
      real*8    somme,grad,n1,n2,n3,n4,det,k,produit,point
      dimension n1(*),n2(*),n3(*),n4(*),k(2,*),grad(4,*)

      point=1.d0/sqrt(3.d0)


      write(20,*)
      call gradiant(grad,det,n1,n2,n3,n4,point,point) 
      call val_tens(k,n1,n2,n3,n4,point,point)
      somme=produit(k,grad,nbr1,nbr2)*abs(det)
      call gradiant(grad,det,n1,n2,n3,n4,-point,point) 
      call val_tens(k,n1,n2,n3,n4,-point,point)
      somme=somme+produit(k,grad,nbr1,nbr2)*abs(det)
      call gradiant(grad,det,n1,n2,n3,n4,-point,-point) 
      call val_tens(k,n1,n2,n3,n4,-point,point)
      somme=somme+produit(k,grad,nbr1,nbr2)*abs(det)
      call gradiant(grad,det,n1,n2,n3,n4,point,-point) 
      call val_tens(k,n1,n2,n3,n4,point,-point)
      somme=somme+produit(k,grad,nbr1,nbr2)*abs(det)
      integrale=somme    
      
      return
      end


c*****************************************************************
      subroutine val_tens(k,n1,n2,n3,n4,xch,ych)    
c*****************************************************************
c
c
      implicit none

      real*8    basec,xch,ych,n1,n2,n3,n4
      real*8    a1,a2,b1,b2,x,k,y,k11t,k22t,k12t
      dimension basec(4),n1(*),n2(*),n3(*),n4(*),k(2,*)

      a1=(1.d0+xch)
      a2=(1.d0-xch)
      b1=(1.d0+ych)
      b2=(1.d0-ych)

      basec(1)=(a2*b2)/4.d0
      basec(2)=(a1*b2)/4.d0
      basec(3)=(a1*b1)/4.d0
      basec(4)=(a2*b1)/4.d0

      x=(basec(1)*n1(2)+basec(2)*n2(2)+basec(3)*n3(2)+basec(4)*n4(2))
      y=(basec(1)*n1(3)+basec(2)*n2(3)+basec(3)*n3(3)+basec(4)*n4(3))

      k(1,1)=k11t(x,y)
      k(2,2)=k22t(x,y)
      k(1,2)=k12t(x,y)
      k(2,1)=k(1,2)

      return
      end








c  fichier  rige2_tr.f
c
c*****************************************************************
      real*8 function integralep(gradp,n1,n2,n3,k,det,nbr1,nbr2)
c*****************************************************************
c
c   cette fonction calcule l'integrale de k(x)grad(u).grad(v) sur
c   un quadrilatere. u et v designant deux fonctions de base quelconque
c   du quadrilatere.
c
c   n1,n2,n3: contient les numeros, ainsi que les coordonnees des
c             trois points qui constituent le triangle
c   k: tableau contenant le tenseur des permeabilites
c   grad: tableau contenant les valeurs des  gradiants
c         des fonctions de base
c   nbr1,nbr2: numero designant la nbr1ieme et la nbr2ieme
c              fonction de base
c   det: jacobien de la transformation: triangle de reference --> triangle  
c
c

      implicit none

      integer   nbr1,nbr2          
      real*8    somme,gradp,n1,n2,n3,det,k,produitp,a,b
      dimension n1(*),n2(*),n3(*),k(2,*),gradp(3,*)

      a=(1.d0/6.d0)
      b=(2.d0/3.d0)


      write(20,*)
      call val_tenst(k,n1,n2,n3,a,a)
      somme=produitp(k,gradp,nbr1,nbr2)*abs(det)*a
      call val_tenst(k,n1,n2,n3,b,a)
      somme=somme+produitp(k,gradp,nbr1,nbr2)*abs(det)*a
      call val_tenst(k,n1,n2,n3,a,b)
      somme=somme+produitp(k,gradp,nbr1,nbr2)*abs(det)*a
      integralep=somme    
      
      return
      end


c*****************************************************************
      subroutine val_tenst(k,n1,n2,n3,xch,ych)    
c*****************************************************************
c
c
      implicit none

      real*8    basep,xch,ych,n1,n2,n3
      real*8    x,k,y,k11t,k22t,k12t
      dimension basep(3),n1(*),n2(*),n3(*),k(2,*)


      basep(1)=(1.d0-xch-ych)
      basep(2)=xch
      basep(3)=ych

      x=(basep(1)*n1(2)+basep(2)*n2(2)+basep(3)*n3(2))
      y=(basep(1)*n1(3)+basep(2)*n2(3)+basep(3)*n3(3))

      k(1,1)=k11t(x,y)
      k(2,2)=k22t(x,y)
      k(1,2)=k12t(x,y)
      k(2,1)=k(1,2)

      return
      end








c  fichier  rige_qua.f
c
c*****************************************************************
c      real*8 function integrale(grad,n1,n2,n3,n4,k,det,nbr1,nbr2)
c*****************************************************************
c
c   cette fonction calcule l'approxiamtion de l'integrale de 
c   [k(x)grad(u).grad(v)] sur un quadrilatere.
c    u et v designant deux fonctions de base quelconque
c   du quadrilatere.
c   on utilise la formule p.??? du  livre 'presentation de la methode
c   des elements finis' (t.touzot) 19??.
c
c   n1,n2,n3,n4: contient les numeros, ainsi que les coordonnees des
c                quatre points qui constituent le quadrilatere
c   k: tableau contenant le tenseur des permeabilites
c   grad: tableau contenant les valeurs des  gradiants
c         des fonctions de base
c   nbr1,nbr2: numero designant la nbr1ieme et la nbr2ieme
c              fonction de base
c   det: jacobien de la transformation: carre de reference --> quadrilatere  
c
c
c
c      implicit none
c
c      integer   nbr1,nbr2          
c      real*8    somme,grad,n1,n2,n3,n4,det,k,produit,point
c      dimension n1(*),n2(*),n3(*),n4(*),k(2,*),grad(4,*)
c
c      point=1.d0/sqrt(3.d0)
c
c      call gradiant(grad,det,n1,n2,n3,n4,point,point) 
c      somme=produit(k,grad,nbr1,nbr2)                   
c      call gradiant(grad,det,n1,n2,n3,n4,-point,point) 
c      somme=somme+produit(k,grad,nbr1,nbr2)             
c      call gradiant(grad,det,n1,n2,n3,n4,-point,-point) 
c      somme=somme+produit(k,grad,nbr1,nbr2)            
c      call gradiant(grad,det,n1,n2,n3,n4,point,-point) 
c      somme=somme+produit(k,grad,nbr1,nbr2)             
c      
c      integrale=somme*abs(det)    
c      
c      return
c      end
c  fichier triangle.f
c
c********************************************************************
c            fichiers necessaires a la subroutine calculp1 
c********************************************************************

c      include'./grad_tri.f'
c      include'./prod_tri.f'   
c      include'./base_tri.f'
c      include'./2mb_tri.f'
c      include'./aire1.f'

c********************************************************************
      subroutine calculp1(elementbis,indice,bb,ib,jb,aa,ia,ja,sminter
     @               ,n1,n2,n3,k,cc,ic,jc,aire,poro,poroh)
c********************************************************************
c
c  cette procedure calcule les differentes contributions amenees par
c  un element triangulaire, et effectue l'assemblage.
c  on sauvegarde egalement certains resultats necessaire en vue du
c  calcul du tenseur homogeneise dans les tableaux bb et cc.
c
c  elementbis: tableau qui permet de dire si un noeud de l'element est
c              sur le bord ou non.
c  indice:  tableau donnant le rang d'un noeud appartenant a vecti ou vectf
c  bb: tableau contenant les contributions bord-bord
c  aa: tableau contenant les contributions int-int (c'est la matrice a
c      du systeme lineaire)
c  cc: tableau contenant les contributions bord-int
c  ia(k): indice dans le tableau aa du premier element
c        de la i eme ligne de a stocke dans aa,
c  ja(k): indice j de colonne de l"element aij stocke
c         dans le tableau aa a l"indice k
c  ib,jb: idem que ia et ja pour le tableau bb
c  ic,jc: idem que ia et ja pour le tableau cc
c  sminter:vecteur contenant le second membre du systeme lineaire
c            sans les conditions aux limites 
c   n1,n2,n3: contient les numeros, ainsi que les coordonnees des
c             trois points qui constituent le triangle
c   k: tableau contenant le tenseur des permeabilites
c

      implicit none

      integer   indice,ele,l,nbr1,ia,ja,ib,jb,ic,jc,nbr2,k1,k2,ent
      real*8    k,n1,n2,n3,det,det_aire,gradp,aire,poro,poroh 
      real*8    som1,produitp,aa,bb,cc,sminter,integrp,aire0  
      logical   elementbis,log1,log2    
      dimension n1(*),n2(*),n3(*),k(2,*),gradp(3,2)
      dimension elementbis(*),indice(*),bb(*),ib(*),jb(*)
      dimension aa(*),ia(*),ja(*),sminter(*),cc(*),ic(*),jc(*)

      call aire_ele1(aire0,n1,n2,n3) 
      aire=aire+aire0
      poroh=poroh+(poro*aire0)

      call gradiantp(gradp,det,n1,n2,n3)
      det_aire=abs(det)*0.5d0

      do ele=2,4
         nbr1=ele-1 
         log2=.not.elementbis(ele)
         do l=ele,4
            k1=indice(ele) 
            k2=indice(l)
            nbr2=l-1
            log1=.not.elementbis(l) 
c
c calcul de la contribution
c
            som1=produitp(k,gradp,nbr1,nbr2)*det_aire
            write(20,*)'prod',produitp(k,gradp,nbr1,nbr2)            
            write(20,*)'som1',som1
c
c assemblage
c            
            if (log1.and.log2) then
                     if (k2.gt.k1) then
                               ent=k1
                               k1=k2
                               k2=ent
                     endif
                     call assemblage(bb,ib,jb,k1,k2,som1)
            else
            
            if (elementbis(ele).and.elementbis(l)) then
                     if (k2.gt.k1) then
                               ent=k1
                               k1=k2
                               k2=ent
                     endif
                     call assemblage(aa,ia,ja,k1,k2,som1)
            else

            log1=.not.elementbis(l)
        if (elementbis(ele).and.log1) then
                     call assemblagebis(cc,ic,jc,k2,k1,som1)
                else

          if (elementbis(l)) then 
                     k1=indice(l)
                     k2=indice(ele)
                     call assemblagebis(cc,ic,jc,k2,k1,som1)
           endif
          endif
         endif
        endif
       enddo
c
c calcul de la contribution du second membre et assemblage
c
       if (elementbis(ele)) then
           k1=indice(ele)
           sminter(k1)=sminter(k1)+integrp(det,n1,n2,n3,nbr1)
       endif
      enddo

      return
      end





c fichier tri.f
c
c*******************************************************************
      subroutine tri(criter,record,n)
c*******************************************************************
c
c   il s'agit de ranger les elements du tableau criter
c   par ordre croissant d"indices et de modifier 
c   simultanement les elements du tableau record.
c
c   criter,record: tableaux
c   n: dimension des tableaux criter et record


      implicit none
      integer n
      integer crit,criter(n),l,r,j,i  
      real*8  record(n),rec

      if(n.le.1) return
      l=n/2+1
      r=n
2     if(l.gt.1)then 
        l=l-1
        rec=record(l)
        crit=criter(l)
      else
        rec=record(r)
        crit=criter(r)
        record(r)=record(1)
        criter(r)=criter(1)
        r=r-1
        if(r.eq.1) then
          record(1)=rec
          criter(1)=crit
          return
        endif
      endif
3     j=l
4     i=j
      j=2*j
      if(j-r.lt.0)then
        goto 5
      elseif(j.eq.r)then
        goto 6
      else
        goto 8
      endif
5     if(criter(j).lt.criter(j+1))j=j+1
6     if(crit.ge.criter(j))goto 8
      record(i)=record(j)
      criter(i)=criter(j)
      goto 4
8     record(i)=rec
      criter(i)=crit
      goto 2

c      return
      end
