c  Fichier base_hex_c.f
c
c************************************************************************
      subroutine FBASEC_C(BASEC,det,N1,N2,N3,N4,N5,N6,N7,N8,
     @                  xch,ych,zch,x,y,z)
c************************************************************************
c
c   Cette procedure calcule la valeur des  fonctions de bases
c   au point (xch,ych,zch) appartenant au cube de reference, ainsi que
c   le jacobien de l'application: cube  de reference --> Hexaedre    
c
c   BASEC: tableau contenant les valeurs des fonctions de
c          base au point (xch,ych,zch)
c   det: jacobien de la transformation: cube  de reference --> hexaedre   
c   N1,N2,N3,N4,N5,N6,N7,N8: contient les numeros, ainsi que les 
c                coordonnees des points qui constituent l'hexaedre 
c   xch,ych,zch: coordonnees du point de calcul
c   x,y,z: coordonnees reelles du point de calcul
c
      implicit none

      real*8    BASEC,xch,ych,zch,x,y,z,det,N1,N2,N3,N4,N5,N6,N7,N8
      real*8    J11,J12,J13,J21,J22,J23,J31,J32,J33,a1,a2,b1,b2,c1,c2 
      dimension BASEC(*),N1(*),N2(*),N3(*),N4(*),N5(*),N6(*),N7(*),N8(*)

c fonctions de bases
      
      a1=(1.d0+xch)
      a2=(1.d0-xch)
      b1=(1.d0+ych)
      b2=(1.d0-ych)
      c1=(1.d0+zch)
      c2=(1.d0-zch)

      BASEC(1)=(a2*b2*c2)/8.d0
      BASEC(2)=(a1*b2*c2)/8.d0
      BASEC(3)=(a1*b1*c2)/8.d0
      BASEC(4)=(a2*b1*c2)/8.d0
      BASEC(5)=(a2*b2*c1)/8.d0
      BASEC(6)=(a1*b2*c1)/8.d0
      BASEC(7)=(a1*b1*c1)/8.d0
      BASEC(8)=(a2*b1*c1)/8.d0

c coordonnees reelle du point de calcul

      x=(BASEC(1)*N1(2)+BASEC(2)*N2(2)+BASEC(3)*N3(2)+BASEC(4)*N4(2))
     @ +(BASEC(5)*N5(2)+BASEC(6)*N6(2)+BASEC(7)*N7(2)+BASEC(8)*N8(2))
      y=(BASEC(1)*N1(3)+BASEC(2)*N2(3)+BASEC(3)*N3(3)+BASEC(4)*N4(3))
     @ +(BASEC(5)*N5(3)+BASEC(6)*N6(3)+BASEC(7)*N7(3)+BASEC(8)*N8(3))
      z=(BASEC(1)*N1(3)+BASEC(2)*N2(3)+BASEC(3)*N3(3)+BASEC(4)*N4(3))
     @ +(BASEC(5)*N5(3)+BASEC(6)*N6(3)+BASEC(7)*N7(3)+BASEC(8)*N8(3))

c calcul du jacobien de la transformation

      J11=((-b2*c2)*N1(2)+(b2*c2)*N2(2)+(b1*c2)*N3(2)+(-b1*c2)*N4(2)+
     @    (-b2*c1)*N5(2)+(b2*c1)*N6(2)+(b1*c1)*N7(2)+(-b1*c1)*N8(2))/8

      J21=((-a2*c2)*N1(2)+(-a1*c2)*N2(2)+(a1*c2)*N3(2)+(a2*c2)*N4(2)+
     @    (-a2*c1)*N5(2)+(-a1*c1)*N6(2)+(a1*c1)*N7(2)+(a2*c1)*N8(2))/8

      J31=((-a2*b2)*N1(2)+(-a1*b2)*N2(2)+(-a1*b1)*N3(2)+(-a2*b1)*N4(2)+
     @    (a2*b2)*N5(2)+(a1*b2)*N6(2)+(a1*b1)*N7(2)+(a2*b1)*N8(2))/8
      
      J12=((-b2*c2)*N1(3)+(b2*c2)*N2(3)+(b1*c2)*N3(3)+(-b1*c2)*N4(3)+
     @    (-b2*c1)*N5(3)+(b2*c1)*N6(3)+(b1*c1)*N7(3)+(-b1*c1)*N8(3))/8

      J22=((-a2*c2)*N1(3)+(-a1*c2)*N2(3)+(a1*c2)*N3(3)+(a2*c2)*N4(3)+
     @    (-a2*c1)*N5(3)+(-a1*c1)*N6(3)+(a1*c1)*N7(3)+(a2*c1)*N8(3))/8

      J32=((-a2*b2)*N1(3)+(-a1*b2)*N2(3)+(-a1*b1)*N3(3)+(-a2*b1)*N4(3)+
     @    (a2*b2)*N5(3)+(a1*b2)*N6(3)+(a1*b1)*N7(3)+(a2*b1)*N8(3))/8
      
      J13=((-b2*c2)*N1(4)+(b2*c2)*N2(4)+(b1*c2)*N3(4)+(-b1*c2)*N4(4)+
     @    (-b2*c1)*N5(4)+(b2*c1)*N6(4)+(b1*c1)*N7(4)+(-b1*c1)*N8(4))/8

      J23=((-a2*c2)*N1(4)+(-a1*c2)*N2(4)+(a1*c2)*N3(4)+(a2*c2)*N4(4)+
     @    (-a2*c1)*N5(4)+(-a1*c1)*N6(4)+(a1*c1)*N7(4)+(a2*c1)*N8(4))/8

      J33=((-a2*b2)*N1(4)+(-a1*b2)*N2(4)+(-a1*b1)*N3(4)+(-a2*b1)*N4(4)+
     @    (a2*b2)*N5(4)+(a1*b2)*N6(4)+(a1*b1)*N7(4)+(a2*b1)*N8(4))/8

      
      det=J11*(J22*J33-J32*J23)+J12*(J31*J23-J21*J33)+
     @     J13*(J21*J32-J31*J22)

      RETURN
      end                                        

